@@ -4,15 +4,13 @@ from .serializable import serialize, ATTR, LEN_ATTR
 from .classes import Classes
 from ..utils.common import lls
 from ..dataset.metadata import guess_value
+from ..dataset.finetime import FineTime
 
 import logging
 import json
-import codecs
 import gzip
 import binascii
 import array
-from collections import ChainMap
-import builtins
 import urllib
 from .odict import ODict
 from collections.abc import MutableMapping as MM
@@ -109,20 +107,41 @@ def constructSerializable(obj, lookup=None, int_key=False, debug=False):
                 print(spaces + 'Find _STID <%s>.' % (ostid))
         # process types wrapped in a dict
         if PY3:
-            if classname == 'bytes':
-                inst = codecs.decode(obj['code'], 'hex')
+            if classname == 'datetime,tai':
+                inst = FineTime(obj['code']).toDatetime()
                 if debug:
-                    print(spaces + 'Instanciate hex')
+                    print(spaces + 'Instanciate datetime')
                 indent -= 1
                 return inst
-            elif classname == 'bytes_gz':
+            if classname == 'bytes':
+                inst = bytes.fromhex(obj['code'])
+                # inst = codecs.decode(obj['code'], 'hex')
+                if debug:
+                    print(spaces + 'Instanciate bytes')
+                indent -= 1
+                return inst
+            elif classname == 'bytes,gz' or classname == 'bytes,gz,b64':
                 inst = gzip.decompress(binascii.a2b_base64(obj['code']))
                 if debug:
                     print(spaces + 'Instanciate hex_gz')
                 indent -= 1
                 return inst
+            elif classname == 'bytearray':
+                inst = bytearray.fromhex(obj['code'])
+                if debug:
+                    print(spaces + 'Instanciate bytearray')
+                indent -= 1
+                return inst
+            elif classname == 'bytearray,gz' or classname == 'bytearray,gz,b64':
+                inst = bytearray(gzip.decompress(
+                    binascii.a2b_base64(obj['code'])))
+                if debug:
+                    print(spaces + 'Instanciate bytearray_gz')
+                indent -= 1
+                return inst
             elif classname.startswith('a.array'):
-                array_t, tcode = tuple(classname.rsplit('_', 1))
+                # format is "...a.array_I,..."
+                array_t, tcode = tuple(classname.split(',', 1)[0].split('_'))
                 if array_t == 'a.array':
                     inst = array.array(tcode, binascii.a2b_hex(obj['code']))
                     if debug:
@@ -151,6 +170,15 @@ def constructSerializable(obj, lookup=None, int_key=False, debug=False):
             o = constructSerializable(
                 obj['obj'], lookup=lookup, int_key=int_key, debug=debug)
             inst = lookup[classname](o)
+            obj = inst
+            try:
+                typ = obj.type
+            except (LookupError, AttributeError) as e:
+                typ = None
+            if typ in ['image/svg']:
+                ser = obj[ATTR+'data']
+                ser = urllib.parse.unquote(ser)
+
             if debug:
                 print(spaces + 'Instanciate defined %s' % obj['obj'])
             indent -= 1
@@ -159,6 +187,9 @@ def constructSerializable(obj, lookup=None, int_key=False, debug=False):
             if debug:
                 print(spaces + 'Instanciate type %s' % obj['obj'])
             inst = lookup[obj['obj']]
+            if inst is None:
+                __import__('pdb').set_trace()
+
             indent -= 1
             return inst
         else:
@@ -290,18 +321,36 @@ class IntDecoderOD(IntDecoder):
             return o
 
 
+Class_Look_Up = Classes.mapping
+
+
 def deserialize(js, lookup=None, debug=False, usedict=True, int_key=False, verbose=False):
-    """ Loads classes with _STID from the results of serialize.
+    """ Create (Load) live object from string, bytes that are generated by serialization.
+
+
 
-    if usedict is True dict insted of ODict will be used.
     Parameters
     ----------
-
+    js : str, bytes.
+         JSON string or bytes, made with `json.load`.
+    lookup : mapping
+         A `globals` or `locals` -like mapping that gives class object if the class name is given.
+    debug : bool
+         A if set `True`, print out step-by-step report of how a JSON document is deserialized into an object. Default `False`.
+    usedict: bool
+        If is `True` `dict` insted of `ODict` will be used during deserialization. Default `True`.
+    init_key: bool
+        If set, dictionary keys that are integers in string form, e.g. '1', are casted to integers. Default `False`.
+    verbose: bool
+        Give even more info.
     Returns
     -------
+    object
+       Deserialized object.
     """
 
-    lookup = Class_Look_Up
+    if lookup is None:
+        lookup = Class_Look_Up
 
     if not isinstance(js, strset) or len(js) == 0:
         return None
@@ -326,8 +375,6 @@ def deserialize(js, lookup=None, debug=False, usedict=True, int_key=False, verbo
     return constructSerializable(obj, lookup=lookup, int_key=int_key, debug=debug)
 
 
-Class_Look_Up = ChainMap(Classes._classes, globals(), vars(builtins))
-
 Serialize_Args_Sep = '__'
 SAS_Avatar = '~'
 
