@@ -5,7 +5,7 @@ from ..utils.common import findShape
 import logging
 # create logger
 logger = logging.getLogger(__name__)
-#logger.debug('level %d' %  (logger.getEffectiveLevel()))
+# logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
 
 class Shaped():
@@ -23,10 +23,14 @@ class Shaped():
         -------
         """
 
-        #print(__name__ + str(kwds))
+        # print(__name__ + str(kwds))
         super().__init__(**kwds)
 
     def updateShape(self, **kwds):
 
-        self.shape = findShape(self.data, **kwds)
+        self.__setattr__('shape', findShape(self.data, **kwds))
+        # this is not working for the firist call as shape is not in
+        # `attributable.__init__`s initializing list because 'shape'
+        # is not in `ArrayDAtaset.init` args.
+        # self.shape = findShape(self.data, **kwds)
         return self.shape
