@@ -18,6 +18,7 @@ import sys
 from collections.abc import Sequence
 from collections import OrderedDict
 import itertools
+from copy import copy
 
 if sys.version_info[0] + 0.1 * sys.version_info[1] >= 3.3:
     PY33 = True
@@ -92,10 +93,11 @@ class TableModel():
         self.getColumn(columnIndex).data[rowIndex] = value
 
 
-def maybe2rows(header_names, units, col_width, one_row=False, linebreak='\n'):
-    """ makes one-row or two-row header
+def maybe2rows(header_names, units, col_width, sep='.', one_row=False, linebreak='\n'):
+    """ makes one-row or two-row column headers
 
-    :one_row: Force one row but add line breaks at '.'
+    :sep: a string of separator characters to split header into two fragments. a header only uses the first matching char from left. Grouping does not distinguish which sep-char was used to split a header. Example: '.' (default), '._/'
+    :one_row: Force one row but add line breaks at sep
 
     """
     if col_width is None:
@@ -107,7 +109,7 @@ def maybe2rows(header_names, units, col_width, one_row=False, linebreak='\n'):
     for x in header_names:
         try:
             # only test if there is '.'
-            if not '.' in x:
+            if not any(s in x for s in sep):
                 raise ValueError()
             f = float(x)
             hd.append(x)
@@ -116,14 +118,23 @@ def maybe2rows(header_names, units, col_width, one_row=False, linebreak='\n'):
                 last = ''
         except ValueError:
             # 'a.b', 'a.c' -> ('a','b'), ('a','c')
-            p = str(x).rsplit('.', 1)
+            r = str(x)
+            # try splitting with all sep chars until the first successful split to get the right-most fragment
+
+            for s in sep:
+                p = r.rsplit(s, 1)
+                if len(p) > 1:
+                    break
             # p0 is the group. p1 the sub-name
             if len(p) > 1:
+                # 'foo.'.rsplit('.') == ['foo','']
                 p0 = p[0]
-                hd.append(p[1])
+                p1 = p[1]
             else:
+                # 'bar'.rsplit('.') == ['bar']
                 p0 = ''
-                hd.append(p[0])
+                p1 = r
+            hd.append(p1)
             hd2.append(p0)
             # repeat is never found if one row
             if not one_row and not found_repeat:
@@ -135,6 +146,7 @@ def maybe2rows(header_names, units, col_width, one_row=False, linebreak='\n'):
     hdr1 = [wls(x.replace('.', '.\n') if one_row else x,
                 width=col_width, linebreak=linebreak)
             for x in (hd if found_repeat else header_names)]
+    # name and unit rows.
     hdr = list('%s%s(%s)' % (nu[0], linebreak, nu[1])
                for nu in zip(hdr1, units))
 
@@ -187,12 +199,11 @@ class TableDataset(CompositeDataset, TableModel, Shaped):
         self._list = []
 
         # collect MDPs from args-turned-local-variables.
-        metasToBeInstalled = OrderedDict(
-            itertools.filterfalse(
-                lambda x: x[0] in ('self', '__class__', 'zInfo',
-                                   'kwds'),
-                locals().items())
-        )
+        metasToBeInstalled = copy(locals())
+        metasToBeInstalled.pop('__class__', None)
+        metasToBeInstalled.pop('kwds', None)
+        metasToBeInstalled.pop('self', None)
+        metasToBeInstalled.pop('zInfo', None)
 
         global Model
         if zInfo is None:
@@ -586,9 +597,9 @@ Default is to return all columns.
     def __repr__(self):
         return self.toString(level=2)
 
-    def toString(self, level=0,
+    def toString(self, level=0, extra=False, param_widths=None,
                  tablefmt='grid', tablefmt1='simple', tablefmt2='plain',
-                 width=0, param_widths=None, matprint=None, trans=True,
+                 width=0, matprint=None, trans=True,
                  heavy=True, center=-1, **kwds):
         """
         tablefmt2: format of 2D data, others see `MetaData.toString`.
@@ -601,15 +612,15 @@ Default is to return all columns.
         if level > 1:
             s = cn + '('
             s += self.meta.toString(
-                level=level,
+                level=level, extra=extra,  param_widths=param_widths,
                 tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
-                width=width, param_widths=param_widths,
+                width=width,
                 **kwds)
             return s + 'data= {' + \
                 ', '.join('"%s": %s' % (k, v.toString(
-                    level=level, heavy=heavy,
+                    level=level, extra=extra,  param_widths=param_widths,
                     tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt,
-                    width=width, param_widths=param_widths, **kwds))
+                    width=width, heavy=heavy, **kwds))
                     for k, v in self.getColumnMap().items()) + \
                 '})'
 
@@ -617,10 +628,10 @@ Default is to return all columns.
         br = '<br>' if html else '\n'
         if html:
             tablefmt = tablefmt2 = 'unsafehtml'
-        s, last = make_title_meta_l0(self, level=level, width=width, heavy=heavy,
+        s, last = make_title_meta_l0(self, level=level, extra=extra, param_widths=param_widths,
                                      tablefmt=tablefmt, tablefmt1=tablefmt1,
                                      tablefmt2=tablefmt2, center=center,
-                                     param_widths=param_widths,
+                                     width=width, heavy=heavy,
                                      html=html, excpt=['description'],
                                      **kwds)
         width = len(last)-1
@@ -634,12 +645,16 @@ Default is to return all columns.
         rowlimit = 2 if level > 1 else 20 if level == 1 else None
 
         cols = self.getData().values()
+        # This is obsolete for tabulate >= 0.8.10:
         # widest width in all of default and in param_widths
-        w = MetaData.MaxDefWidth if param_widths == -1 or param_widths is None else \
-            max(MetaData.MaxDefWidth, max(param_widths.values()))
-        if html:
-            w = w
+        # w = MetaData.MaxDefWidth if param_widths == -1 or \
+        #    param_widths is None else \
+        #    max(MetaData.MaxDefWidth, max(param_widths.values()))
+        # if html:
+        #     w = w
+        w = MetaData.MaxDefWidth
         coldata = [list(itertools.islice(x.data, rowlimit)) for x in cols]
+        # table headers
         hdr = maybe2rows(self.getData().keys(),
                          (str(x.unit) for x in cols),
                          col_width=w, one_row=html,
@@ -655,6 +670,7 @@ Default is to return all columns.
         return f'{s}\n{d}{last}\n'
 
     string = toString
+    txt = toString
 
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
