@@ -1,5 +1,6 @@
 from fdi.dataset.product import _Model_Spec as PPI
 from .product import Product
+from .baseproduct import BaseProduct
 from .numericparameter import NumericParameter
 from .dateparameter import DateParameter
 from .stringparameter import StringParameter
@@ -18,21 +19,52 @@ import random
 from os import path as op
 
 
+class TB(BaseProduct):
+    def __init__(self, *args, **kwds):
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'TB'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
+
+
 class TP(Product):
-    pass
+    def __init__(self, *args, **kwds):
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'TC'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
 
 
 class TC(Context):
-    pass
+    def __init__(self, *args, **kwds):
+
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'TC'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
+
+
+class TCC(TC):
+    def __init__(self, *args, **kwds):
+
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'TCC'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
 
 
 class TM(MapContext):
-    pass
+    def __init__(self, *args, **kwds):
 
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'TM'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
 
 # sub-classing testing class
 # 'version' of subclass is int, not string
 
+
 sp = copy.deepcopy(PPI)
 sp['name'] = 'SP'
 sp['metadata']['version']['data_type'] = 'integer'
@@ -84,60 +116,70 @@ def makeCal2D(width=11, height=11):
     return z
 
 
-def get_demo_product():
+class DemoProduct(MapContext):
+    def __init__(self, *args, **kwds):
+
+        super().__init__(*args, **kwds)
+        self.zInfo['name'] = 'DemoProduct'
+        self.zInfo['description'] = 'Test class %s.' % self.zInfo['name']
+        self.zInfo['metadata']['type']['default'] = self.zInfo['name']
+
+
+def get_demo_product(desc=''):
     """
     A complex context product as a reference for testing and demo.
 
-    ```
+::
+
 |__ meta                                          <MetaData>
-|   |__ description                                 <string>
-|   |__ type                                        <string>
-|   |__ level                                       <string>
-|   |__ creator                                     <string>
-|   |__ creationDate                              <finetime>
-|   |__ rootCause                                   <string>
-|   |__ version                                     <string>
-|   |__ FORMATV                                     <string>
-|   |__ speed                                       <vector>
-|   \__ listeners                               <ListnerSet>
-|__ measurements                          <CompositeDataset>
-|   |__ meta                                      <MetaData>
-|   |   \__ listeners                           <ListnerSet>
-|   |__ Time_Energy_Pos               <TableDataset> (5, 20)
-|   |   |__ meta                                  <MetaData>
-|   |   |   |__ description                         <string>
-|   |   |   |__ shape                                <tuple>
-|   |   |   |__ type                                <string>
-|   |   |   |__ version                             <string>
-|   |   |   |__ FORMATV                             <string>
-|   |   |   \__ listeners                       <ListnerSet>
-|   |   |__ Time                              <Column> (20,)
-|   |   |__ Energy                            <Column> (20,)
-|   |   |__ Error                             <Column> (20,)
-|   |   |__ y                                 <Column> (20,)
-|   |   \__ z                                 <Column> (20,)
-|   |__ calibration                  <ArrayDataset> (11, 11)
-|   \__ dset                                           <str>
-|__ Environment Temperature              <ArrayDataset> (7,)
-|__ Browse                               <image/png> (5976,)
-|__ refs                                      <RefContainer>
-|   |__ a_reference                             <ProductRef>
-|   \__ a_different_name                        <ProductRef>
-|__ history                                        <History>
-|   |__ PARAM_HISTORY                                  <str>
-|   |__ TASK_HISTORY                                   <str>
-|   \__ meta                                      <MetaData>
-|       \__ listeners                           <ListnerSet>
-\__ listeners                                   <ListnerSet>
-    ```
+    |   |__ description                                 <string>
+    |   |__ type                                        <string>
+    |   |__ level                                       <string>
+    |   |__ creator                                     <string>
+    |   |__ creationDate                              <finetime>
+    |   |__ rootCause                                   <string>
+    |   |__ version                                     <string>
+    |   |__ FORMATV                                     <string>
+    |   |__ speed                                       <vector>
+    |   \__ listeners                               <ListenerSet>
+    |__ measurements                          <CompositeDataset>
+    |   |__ meta                                      <MetaData>
+    |   |   \__ listeners                           <ListenerSet>
+    |   |__ Time_Energy_Pos               <TableDataset> (5, 20)
+    |   |   |__ meta                                  <MetaData>
+    |   |   |   |__ description                         <string>
+    |   |   |   |__ shape                                <tuple>
+    |   |   |   |__ type                                <string>
+    |   |   |   |__ version                             <string>
+    |   |   |   |__ FORMATV                             <string>
+    |   |   |   \__ listeners                       <ListenerSet>
+    |   |   |__ Time                              <Column> (20,)
+    |   |   |__ Energy                            <Column> (20,)
+    |   |   |__ Error                             <Column> (20,)
+    |   |   |__ y                                 <Column> (20,)
+    |   |   \__ z                                 <Column> (20,)
+    |   |__ calibration                  <ArrayDataset> (11, 11)
+    |   \__ dset                                           <str>
+    |__ Environment Temperature              <ArrayDataset> (7,)
+    |__ Browse                               <image/png> (5976,)
+    |__ refs                                      <RefContainer>
+    |   |__ a_reference                             <ProductRef>
+    |   \__ a_different_name                        <ProductRef>
+    |__ history                                        <History>
+    |   |__ PARAM_HISTORY                                  <str>
+    |   |__ TASK_HISTORY                                   <str>
+    |   \__ meta                                      <MetaData>
+    |       \__ listeners                           <ListenerSet>
+    \__ listeners                                   <ListenerSet>
 
     """
-    prodx = MapContext('A complex product for demonstration.')
+
+    prodx = DemoProduct(desc if desc else 'A complex product for demo/test.')
     prodx.creator = 'Frankenstein'
     # add a parameter with validity descriptors to the product
     prodx.meta['speed'] = NumericParameter(
         description='an extra param',
-        value=Vector((1.1, 2.2, 3.3)),
+        value=Vector((1.1, 2.2, 3.3)),  # do not use list
         valid={(1, 22): 'normal', (30, 33): 'fast'}, unit='meter')
 
     # A CompositeDataset 'measurements' of two sub-datasets: calibration and measurements
