@@ -1,14 +1,23 @@
 # -*- coding: utf-8 -*-
 
+from .. import checkpath, PM_S
 from ..model.user import auth
 
 from ...dataset.deserialize import deserialize
 from ...dataset.serializable import serialize
 from ...dataset.mediawrapper import MediaWrapper
 from ...pal.urn import makeUrn
-from ...pal.poolmanager import PoolManager as PM
+from ...pal.poolmanager import PoolManager
 from ...dataset.classes import Classes
-from ...utils.common import trbk, getUidGid
+from ...utils.common import (trbk,
+                             getUidGid,
+                             lls,
+                             logging_ERROR,
+                             logging_WARNING,
+                             logging_INFO,
+                             logging_DEBUG
+                             )
+
 from ...utils.fetch import fetch
 from ...pns.fdi_requests import POST_PRODUCT_TAG_NAME
 
@@ -17,7 +26,6 @@ from ...pns.fdi_requests import POST_PRODUCT_TAG_NAME
 # import mysql.connector
 # from mysql.connector import Error
 
-from flasgger import swag_from
 
 from flask import request, make_response, Blueprint, current_app
 from flask.wrappers import Response
@@ -27,8 +35,6 @@ import sys
 import os
 import time
 import builtins
-import functools
-from pathlib import Path
 import importlib
 import logging
 
@@ -51,75 +57,39 @@ data_api = Blueprint('httppool_server', __name__)
 WRITE_LIST = ['POST', 'PUT', 'DELETE', 'PATCH']
 
 
-@functools.lru_cache(6)
-def checkpath(path, un):
-    """ Checks  the directories and creats if missing.
-
-    path: str. can be resolved with Path.
-    un: server user name
-    """
-    logger.debug('path %s user %s' % (path, un))
-
-    p = Path(path).resolve()
-    if p.exists():
-        if not p.is_dir():
-            msg = str(p) + ' is not a directory.'
-            logger.error(msg)
-            return None
-        else:
-            # if path exists and can be set owner and group
-            if p.owner() != un or p.group() != un:
-                msg = str(p) + ' owner %s group %s. Should be %s.' % \
-                    (p.owner(), p.group(), un)
-                logger.warning(msg)
-    else:
-        # path does not exist
-
-        msg = str(p) + ' does not exist. Creating...'
-        logger.debug(msg)
-        p.mkdir(mode=0o775, parents=True, exist_ok=True)
-        logger.info(str(p) + ' directory has been made.')
-
-    # logger.info('Setting owner, group, and mode...')
-    if not setOwnerMode(p, un):
-        logger.info('Cannot set owner %s to %s.' % (un, str(p)))
-        return None
-
-    logger.debug('checked path at ' + str(p))
-    return p
-
-
 # =============HTTP POOL=========================
 
-# @data_api.before_app_first_request
 
 def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', length=80, req_auth=False):
     """
     Make response.
 
+    :result: if is `Response`, this is returned req_auth applied.
     :ctype: Content-Type. Default is `application/json`
     :serialize_out: if True `result` is in already in serialized form.
     """
+
     # return if `result` is already a Response
     if issubclass(result.__class__, Response):
-        return result
-    if ctype == 'application/json':
-        if serialize_out:
-            # result is already in serialized form
-            p = 'no-serialization-result-place-holder'
-            t = serialize({"result": p, "msg": msg, "time": ts})
-            w = t.replace('"'+p+'"', result)
-        else:
-            w = serialize({"result": result, "msg": msg, "time": ts})
+        resp = result
     else:
-        w = result
+        if ctype == 'application/json':
+            if serialize_out:
+                # result is already in serialized form
+                p = 'no-serialization-result-place-holder'
+                t = serialize({"result": p, "msg": msg, "time": ts})
+                w = t.replace('"'+p+'"', result)
+            else:
+                w = serialize({"result": result, "msg": msg, "time": ts})
+        else:
+            w = result
 
-    if logger.getEffectiveLevel() <= logging.DEBUG:
-        from ...utils.common import lls
-        logger.debug(lls(w, length))
-    # logger.debug(pprint.pformat(w, depth=3, indent=4))
-    resp = make_response(w, code)
-    resp.headers['Content-Type'] = ctype
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug(lls(w, length))
+            # logger.debug(pprint.pformat(w, depth=3, indent=4))
+
+        resp = make_response(w, code)
+        resp.headers['Content-Type'] = ctype
 
     if req_auth:
         resp.headers['WWW-Authenticate'] = 'Basic'
@@ -127,8 +97,8 @@ def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', l
 
 
 def excp(e, code=400, msg='', serialize_out=True):
-    if current_app.config['DEBUG']:
-        raise e
+    # if current_app.config['DEBUG']:
+    #    raise e
     result = '"FAILED"' if serialize_out else 'FAILED'
     msg = '%s\n%s: %s.\nTrace back: %s' % (
         msg, e.__class__.__name__, str(e), trbk(e))
@@ -137,27 +107,24 @@ def excp(e, code=400, msg='', serialize_out=True):
 
 
 def check_readonly(usr, meth, logger):
+    """ If both 'read_write' and 'read_only' present, 'read_write' has precedence.
+    """
     return None
     if usr is None:
         msg = 'Unknown user %s.' % usr
-        logger.debug(msg)
-        return unauthorized(msg)
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug(msg)
+        return msg, 401
 
-    if meth in WRITE_LIST and usr.role == 'read_only':
+    if meth in WRITE_LIST and ('read_only' in usr.roles) and ('read_write' not in usr.roles):
         msg = 'User %s is Read-Only, not allowed to %s.' % (usr.name, meth)
-        logger.debug(msg)
-        return unauthorized(msg)
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug(msg)
+        return msg, 401
 
     return None
 
 
-@data_api.before_request
-def before_request_callback():
-    path = request.path
-    method = request.method
-
-    print("data_api " + path + " >>>>>>[" + method + "]<<<<<<")
-
 ######################################
 ####  /urn{parts} get data ####
 ######################################
@@ -190,6 +157,7 @@ def urn(parts):
 ######################################
 
 
+@ data_api.route('/urn<path:parts>/', methods=['DELETE'])
 @ data_api.route('/urn<path:parts>', methods=['DELETE'])
 @ auth.login_required(role='read_write')
 def delete_urn(parts):
@@ -202,7 +170,7 @@ def delete_urn(parts):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug('get data for URN parts ' + parts)
+    logger.info('delete data for URN parts ' + parts)
 
     paths = parts2paths(parts)
     # if paths[-1] == '':
@@ -245,15 +213,15 @@ def delete_product(paths, serialize_out=False):
     urn = makeUrn(poolname=poolname, typename=typename, index=indexstr)
     # resourcetype = fullname(data)
 
-    if not PM.isLoaded(poolname):
+    if not PM_S.isLoaded(poolname):
         result = FAILED
         msg = 'Pool not found or not registered: ' + poolname
-        code = 400
+        code = 404
         logger.error(msg)
         return code, result, msg
     logger.debug('DELETE product urn: ' + urn)
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         poolobj.remove(urn)
         result = 0
         msg = 'remove product ' + urn + ' OK.'
@@ -270,8 +238,8 @@ def delete_product(paths, serialize_out=False):
 ######################################
 
 
-# @ data_api.route('/<string:pool>', methods=['POST'])
 @ data_api.route('/<string:pool>/', methods=['POST'])
+@ data_api.route('/<string:pool>', methods=['POST'])
 @ auth.login_required(role='read_write')
 def save_data(pool):
     """
@@ -282,14 +250,15 @@ def save_data(pool):
 
     ts = time.time()
     logger = current_app.logger
-    logger.debug(f'save to ' + pool)
+    # logger.debug(f'save to ' + pool)
 
     # do not deserialize if set True. save directly to disk
     serial_through = True
 
-    if request.data is None:
-        result, msg = '"FAILED"', 'No REquest data for command '+request.method
-        code = 404
+    if not request.data:
+        result, msg = '"FAILED"', 'No Request data for command '+request.method
+        code = 400
+        logger.warning(f'SAVE product to {pool} {msg}.')
         return resp(code, result, msg, ts, serialize_out=True)
 
     # save product
@@ -299,14 +268,14 @@ def save_data(pool):
         tags = None
 
     paths = [pool]
-    logger.debug('*** method %s pool %s tags %s' %
-                 (request.method, pool, str(tags)))
+    # logger.debug('*** method:%s pool:%s tags:%s' %
+    #             (request.method, pool, str(tags)))
 
     if serial_through:
         data = str(request.data, encoding='ascii')
 
         code, result, msg = save_product(
-            data, paths, tags, serialize_in=not serial_through, serialize_out=serial_through)
+            data, paths, tags, serialize_in=not serial_through, serialize_out=serial_through, logger=logger)
     else:
         try:
             data = deserialize(request.data)
@@ -315,15 +284,16 @@ def save_data(pool):
                 e,
                 msg='Class needs to be included in pool configuration.',
                 serialize_out=serial_through)
+            logger.info(f'SAVE product to {poolurl} {msg}.')
         else:
             code, result, msg = save_product(
-                data, paths, tags, serialize_in=not serial_through)
+                data, paths, tags, serialize_in=not serial_through, logger=logger)
             # save_action(username=username, action='SAVE', pool=paths[0])
 
     return resp(code, result, msg, ts, serialize_out=serial_through)
 
 
-def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False):
+def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False, logger=logger):
     """Save products and returns URNs.
 
     Saving Products to HTTPpool will have data stored on the server side. The server only returns URN strings as a response. ProductRefs will be generated by the associated httpclient pool which is the front-end on the user side.
@@ -342,19 +312,23 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False)
     if checkpath(fullpoolpath, current_app.config['PC']['self_username']) is None:
         result = FAILED
         msg = 'Pool directory error: ' + fullpoolpath
+        logger.info(f'SAVE product to {poolurl} {msg}.')
         return 400, result, msg
 
-    logger.debug('SAVE product to: ' + poolurl)
-    # logger.debug(str(id(PM._GlobalPoolList)) + ' ' + str(PM._GlobalPoolList))
+    # logger.debug(str(id(PM_S._GlobalPoolList)) + ' ' + str(PM_S._GlobalPoolList))
 
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.saveProduct(
             product=data, tag=tags, geturnobjs=True, serialize_in=serialize_in, serialize_out=serialize_out)
-        msg = 'Save data to ' + poolurl + ' OK.'
+        # seriaized:'"urn:test_remote_pool:fdi.dataset.product.Product:0"'
+        urn = result.split(poolname, 1)[1][1:-1] if serialize_out else result
+        msg = f'Save {urn} to {poolurl} OK.'
         code = 200
+        logger.info(msg)
     except Exception as e:
         code, result, msg = excp(e, serialize_out=serialize_out)
+        logger.info(f'SAVE product to {poolurl} {msg}.')
     return code, result, msg
 
 ######################################
@@ -373,16 +347,13 @@ def data_paths(pool, data_paths):
 
     ts = time.time()
     logger = current_app.logger
-    logger.debug(f'datapath of ' + pool)
-
-    logger = current_app.logger
+    serial_through = True
 
     # do not deserialize if set True. save directly to disk
-    serial_through = True
 
     paths = [pool] + parts2paths(data_paths)
 
-    logger.debug('*** method= %s pool= %s data_paths= %s paths= %s' %
+    logger.debug('>>>[%4s] %s data_paths= %s paths= %s' %
                  (request.method, pool, str(data_paths), str(paths)))
 
     code, result, msg = getProduct_Or_Component(
@@ -407,7 +378,10 @@ def getProduct_Or_Component(paths, serialize_out=False):
         pp = paths[1]
         mp = pp.rsplit('.', 1)
         if len(mp) < 2:
-            return 422, '"FAILED"', 'Need a dot-separated full type name, not %s.' % pp
+            msg = 'Need a dot-separated full type name, not %s.' % pp
+            code = 422
+            logger.info(f'{code} {msg}')
+            return code, '"FAILED"', msg
         modname, ptype = mp[0], mp[1]
         cls = Classes.mapping[ptype]
         mod = importlib.import_module(modname)  # TODO
@@ -416,13 +390,14 @@ def getProduct_Or_Component(paths, serialize_out=False):
         except AttributeError:
             mInfo = cls().zInfo
         # non-serialized
-        return 0, resp(200, mInfo,
-                       'Getting API info for %s OK' % paths[1],
-                       ts, serialize_out=False), 0
+        code = 200
+        msg = 'Getting API info for %s OK' % paths[1]
+        logger.info(f'{code} {msg}')
+        return 0, resp(code, mInfo,  msg, ts, serialize_out=False), 0
     # elif lp == 3 and paths[-1]=='':
 
     #     try:
-    #         poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+    #         poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
     #         result = poolobj.readHK(hkname, serialize_out=serialize_out)
     #         code, msg = 200, hkname + ' HK data returned OK'
     #     except Exception as e:
@@ -431,7 +406,12 @@ def getProduct_Or_Component(paths, serialize_out=False):
         return get_component_or_method(paths, mInfo, serialize_out=serialize_out)
 
     else:
-        return 400, '"FAILED"', 'Unknown path %s' % str(paths)
+        __import__("pdb").set_trace()
+
+        code = 400
+        msg = 'Unknown path %s' % str(paths)
+        logger.info(f'{code} {msg}')
+        return code, '"FAILED"', msg
 
 
 HTML_STYLE = """
@@ -462,13 +442,13 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
 
     if paths[-1] == '':
         # command is '' and url endswith a'/'
-        compo, path_str, prod = load_component_at(1, paths[:-1], mInfo)
-        if compo:
+        compo, path_str, prod = load_component_at(1, paths[:], mInfo)
+        if compo is not None:
             ls = [m for m in dir(compo) if not m.startswith('_')]
             return 0, resp(200, ls, 'Getting %s members/attrbutes OK' % (path_str),
                            ts, serialize_out=False), 0
         else:
-            return 400, FAILED, '%s' % (path_str)
+            return 400, FAILED, 'Cannot get %s for "%s".' % ('/'.join(paths[:-1]), path_str)
     elif lp == 3:
         # url ends with index
         # no cmd, ex: test/fdi.dataset.Product/4
@@ -480,12 +460,14 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
         # grand tour
         compo, path_str, prod = load_component_at(1, paths, mInfo)
         # see :func:`fetch`
+        # e.g. path_str is like '.string()' '["text"]'  '.meta["speed"].isValid()'
         if compo or 'has no' not in path_str:
             code = 200
             msg = f'Getting {path_str} OK'
             compo_meth_name = path_str.split('.')[-1]
             if compo_meth_name[:8] == 'toString' or \
-               compo_meth_name[:6] == 'string':
+               compo_meth_name[:6] == 'string' or \
+               compo_meth_name[:3] == 'txt':
                 if 'html' in compo_meth_name:
                     ct = 'text/html'
                     result = HTML_STYLE + compo
@@ -503,12 +485,12 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
                 ct = 'text/html'
                 result = HTML_STYLE + compo
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
-            elif compo_meth_name == 'yaml()' or compo_meth_name == 'tree()':
+            elif compo_meth_name.startswith('yaml(') or compo_meth_name.startswith('tree('):
                 ct = 'text/plain;charset=utf-8'
                 # 'font-family: "Courier New",monospace;\n'
                 result = compo
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
-            elif compo_meth_name == 'fits()':
+            elif compo_meth_name.startswith('fits('):
                 ct = 'application/fits'
                 result = compo
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
@@ -517,6 +499,12 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
                 result = compo.data
 
                 return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
+            elif compo_meth_name.startswith('graph(') or \
+                    compo_meth_name.startswith('getTaskHistory('):
+                ct = 'image/png'
+                result = compo
+                return 0, resp(code, result, msg, ts, ctype=ct, serialize_out=False), 0
+
             else:
                 return 0, resp(code, compo, msg, ts, serialize_out=False), 0
 
@@ -571,7 +559,7 @@ def load_product(p, paths, serialize_out=False):
 
     logger.debug('LOAD product: ' + urn)
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.loadProduct(urn=urn, serialize_out=serialize_out)
         msg = ''
         code = 200
@@ -580,36 +568,14 @@ def load_product(p, paths, serialize_out=False):
             msg = 'Not found: ' + poolname
             code = 404
         else:
+            __import__("pdb").set_trace()
+
             msg, code = '', 400
         code, result, msg = excp(
             e, code=code, msg=msg, serialize_out=serialize_out)
     return code, result, msg
 
 
-def setOwnerMode(p, username):
-    """ makes UID and GID set to those of self_username given in the config file. This function is usually done by the initPTS script.
-    """
-
-    logger.debug('set owner, group to %s, mode to 0o775' % username)
-
-    uid, gid = getUidGid(username)
-    if uid == -1 or gid == -1:
-        logger.debug(f'user {username} uid={uid} gid{gid}')
-        return None
-    try:
-        os.chown(str(p), uid, gid)
-        os.chmod(str(p), mode=0o775)
-    except Exception as e:
-        code, result, msg = excp(
-            e,
-            msg='cannot set input/output dirs owner to ' +
-            username + ' or mode. check config. ')
-        logger.error(msg)
-        return None
-
-    return username
-
-
 Builtins = vars(builtins)
 
 
