@@ -1,17 +1,27 @@
 # -*- coding: utf-8 -*-
 
 from .getswag import swag
-from .httppool_server import resp, excp, checkpath, check_readonly
-from ..model.user import auth, getUsers
+from .. import checkpath, PM_S
+from .httppool_server import (
+    excp,
+    resp
+)
+# from .. import auth
+from ..model.user import auth
+
 from ..._version import __version__
 from ...dataset.deserialize import deserialize_args, deserialize
-from ...pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
 from ...pal.productpool import PoolNotFoundError
 from ...pal.webapi import WebAPI
-from ...pal.urn import parseUrn
+# from ...pal.urn import parseUrn
 from ...pal.dicthk import HKDBS
-from ...utils.common import lls
-
+from ...utils.getconfig import getConfig
+from ...utils.common import (lls,
+                             logging_ERROR,
+                             logging_WARNING,
+                             logging_INFO,
+                             logging_DEBUG
+                             )
 from flask import Blueprint, jsonify, request, current_app, url_for, abort
 from werkzeug.exceptions import HTTPException
 # from flasgger import swag_from
@@ -19,11 +29,9 @@ from werkzeug.exceptions import HTTPException
 import shutil
 import time
 import copy
-import json
 import os
 from os.path import join, expandvars
 from itertools import chain
-from http import HTTPStatus
 
 
 endp = swag['paths']
@@ -31,52 +39,6 @@ endp = swag['paths']
 pools_api = Blueprint('pools', __name__)
 
 
-######################################
-####  /user/login GET, POST  ####
-######################################
-
-
-@ pools_api.route('/user/login', methods=['GET', 'POST'])
-@ auth.login_required(role=['read_only', 'read_write'])
-def login():
-    """ Logging in on the server.
-
-    :return: response made from http code, poolurl, message
-    """
-
-    logger = current_app.logger
-    ts = time.time()
-    logger.debug('login')
-
-    msg = 'User %s logged-in %s.' % (auth.current_user().username,
-                                     auth.current_user().role)
-    logger.debug(msg)
-
-    return resp(200, 'OK', msg, ts, req_auth=True)
-
-######################################
-####  /user/logout GET, POST  ####
-######################################
-
-
-@ pools_api.route('/user/logout', methods=['GET', 'POST'])
-@ auth.login_required(role=['read_only', 'read_write'])
-def logout():
-    """ Logging in on the server.
-
-    :return: response made from http code, poolurl, message
-    """
-
-    logger = current_app.logger
-    ts = time.time()
-    logger.debug('logout')
-
-    msg = 'User %s logged-out %s.' % (auth.current_user().username,
-                                      auth.current_user().role)
-    logger.debug(msg)
-
-    return resp(401, 'OK. Bye.', msg, ts)
-
 ######################################
 ####   get_pools_url   ####
 ######################################
@@ -90,20 +52,29 @@ def get_pools_url():
 
     ts = time.time()
     path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
-    logger.debug('Listing all directories from ' + path)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Listing all directories from ' + path)
 
     result = get_name_all_pools(path)
 
+    burl = request.base_url
     if issubclass(result.__class__, list):
-        res = dict((x, request.base_url+'/'+x) for x in result)
+        res = dict((x, '/'.join((burl, x))) for x in result)
     else:
         res = {}
 
-    dvers = expandvars('$DOCKER_VERSION')
-    svers = expandvars('$SERVER_VERSION')
+    dvers = getConfig('docker_version')
+    svers = getConfig('server_version')
+
+    from ..model.user import getUsers
+    # report lru_cache info
+    lru = 'LRU user '
+    for name, usr in current_app.config['USERS'].items():
+        lru += '%s: %s ' % (name, usr.getCacheInfo())
 
     msg = '%d pools found. Versiosn: fdi %s docker %s pool server %s' % (
         len(result), __version__, dvers, svers)
+    msg += lru
     code = 200
     return resp(code, res, msg, ts)
 
@@ -121,7 +92,8 @@ def get_pools():
 
     ts = time.time()
     path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
-    logger.debug('Listing all directories from ' + path)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Listing all directories from ' + path)
 
     result = get_name_all_pools(path)
 
@@ -136,6 +108,7 @@ def get_name_all_pools(path):
 
     """
 
+    logger = current_app.logger
     alldirs = []
     os.makedirs(path, exist_ok=True)
     allfilelist = os.listdir(path)
@@ -143,27 +116,67 @@ def get_name_all_pools(path):
         filepath = join(path, file)
         if os.path.isdir(filepath):
             alldirs.append(file)
-    current_app.logger.debug(path + ' has ' + str(alldirs))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(path + ' has ' + lls(alldirs, 100))
     return alldirs
 
-######################################
-#### /pools  get_registered_pools/  ####
-######################################
+
+####################################################
+#### /pools   get_all_pools_size_counts/        ####
+#### 333#######################################V#####
 
 
-# @ pools_api.route('/pools', methods=['GET'])
+@ pools_api.route('/pools', methods=['GET'])
+@ auth.login_required(role=['read_only', 'read_write'])
+def get_all_pools_size_counts():
+    """ Returns a list of Pool IDs (pool names) of all pools and their size and file counts.
+    ---
+    """
+
+    logger = current_app.logger
+    ts = time.time()
+    serialize_out = False
+    FAILED = '"FAILED"' if serialize_out else 'FAILED'
+    data_path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Information of all  pools.')
+
+    if not os.path.exists(data_path):
+        return resp(404, FAILED, f'Find no directory at {data_path}')
+    pools = {}
+    s = c = 0
+    for i in os.listdir(data_path):
+        _, count, msg, size = get_data_count(None, i)
+        pools[i] = {'size_in_bytes': size, 'number_of_data_files': count}
+        s += size
+        c += count
+    result = pools
+    msg = 'There is/are %d pools holding %d products. %.1d MB.' % \
+        (len(pools), c, s/1000000)
+    code = 200
+    return resp(code, result, msg, ts)
+
+##########################################
+#### /pools/   get_registered_pools/  ####
+##########################################
+
+
+@ pools_api.route('/pools', methods=['GET'])
 @ pools_api.route('/pools/', methods=['GET'])
 @ auth.login_required(role=['read_only', 'read_write'])
 def get_registered_pools():
     """ Returns a list of Pool IDs (pool names) of all pools registered with the Global PoolManager.
     ---
     """
+
+    logger = current_app.logger
     ts = time.time()
     path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
-    current_app.logger.debug('Listing all registered pools.')
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Listing all registered pools.')
 
-    # [p.getPoolurl() for p in PM.getMap()()]
-    result = list(PM.getMap())
+    # [p.getPoolurl() for p in PM_S.getMap()()]
+    result = list(PM_S.getMap())
     msg = 'There is/are %d pools registered to the PoolManager.' % len(result)
     code = 200
     return resp(code, result, msg, ts)
@@ -175,20 +188,25 @@ def get_registered_pools():
 
 
 @ pools_api.route('/pools/register_all', methods=['PUT'])
-# @ pools_api.route('/pools/register_all/', methods=['PUT'])
+@ pools_api.route('/pools/register_all/', methods=['PUT'])
 @auth.login_required(role='read_write')
 def register_all():
     """ Register (Load) all pools on tme server.
 
 
     """
-    ts = time.time()
 
+    logger = current_app.logger
+    ts = time.time()
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Load all pools.')
+    # will have / at the end
+    burl = request.base_url.rsplit('register_all', 1)[0]
     result, bad = load_pools(None, auth.current_user())
     code = 400 if len(bad) else 200
     # result = ', '.join(pmap.keys())
     if issubclass(result.__class__, dict):
-        result = dict((x, request.base_url+x) for x in result)
+        result = dict((x, burl+x) for x in result)
     msg = '%d pools successfully loaded. Troubled: %s' % (
         len(result), str(bad))
     return resp(code, result, msg, ts)
@@ -207,17 +225,20 @@ def load_pools(poolnames, usr):
     path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
     pmap = {}
     bad = {}
-    logger.debug('loading all from ' + path)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('loading all from ' + path)
     alldirs = poolnames if poolnames else get_name_all_pools(path)
     for nm in alldirs:
-        # must save the link or PM._GLOBALPOOLLIST will remove as dead weakref
+        # must save the link or PM_S._GlobalPoolList will remove as dead weakref
         code, thepool, msg = register_pool(nm, usr=usr)
         if code == 200:
             pmap[nm] = thepool
         else:
             bad[nm] = nm+': '+msg
 
-    logger.debug("Registered pools: %s in %s" % (str(list(pmap.keys())), path))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug("Registered pools: %s in %s" %
+                     (str(list(pmap.keys())), path))
     return pmap, bad
 
 ######################################
@@ -226,13 +247,14 @@ def load_pools(poolnames, usr):
 
 
 @ pools_api.route('/pools/unregister_all', methods=['PUT'])
-# @ pools_api.route('/pools/unregister_all/', methods=['PUT'])
+@ pools_api.route('/pools/unregister_all/', methods=['PUT'])
 @ auth.login_required(role='read_write')
 def unregister_all():
 
     logger = current_app.logger
     ts = time.time()
-    logger.debug('unregister-all ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('unregister-all ' + pool)
 
     good, bad = unregister_pools()
     code = 200 if not bad else 416
@@ -254,8 +276,10 @@ def unregister_pools(poolnames=None):
 
     good = []
     notgood = []
-    all_pools = poolnames if poolnames else copy.copy(list(PM.getMap().keys()))
-    logger.debug('unregister pools ' + str(all_pools))
+    all_pools = poolnames if poolnames else copy.copy(
+        list(PM_S.getMap().keys()))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('unregister pools ' + str(all_pools))
 
     for nm in all_pools:
         code, res, msg = unregister_pool(nm)
@@ -280,7 +304,8 @@ def wipe_all():
     """
     logger = current_app.logger
     ts = time.time()
-    logger.debug('Wipe-all pool')
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Wipe-all pool')
 
     good, bad = wipe_pools(None, auth.current_user())
     code = 200 if not bad else 416
@@ -300,7 +325,8 @@ def wipe_pools(poolnames, usr):
     """
     logger = current_app.logger
     path = current_app.config['FULL_BASE_LOCAL_POOLPATH']
-    logger.debug('DELETING pools contents from ' + path)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('DELETING pools contents from ' + path)
 
     # alldirs = poolnames if poolnames else get_name_all_pools(path)
 
@@ -313,7 +339,7 @@ def wipe_pools(poolnames, usr):
         try:
             thepool.removeAll()
             shutil.rmtree(join(path, nm))
-            res = PM.remove(nm)
+            res = PM_S.remove(nm)
             if res > 1:
                 notgood.append(nm+': '+str(res))
                 logger.info('Pool %s not deleted.' % nm)
@@ -325,12 +351,13 @@ def wipe_pools(poolnames, usr):
     return good, notgood
 
 ######################################
-#### /{pool}  /{pool}/  GET  ####
+####    /{pool}    /{pool}/  GET  ####
 ######################################
 
 
-@ pools_api.route('/<string:pool>', methods=['GET'])
 @ pools_api.route('/<string:pool>/', methods=['GET'])
+@ pools_api.route('/<string:pool>', methods=['GET'])
+# @ auth.login_required(role=['read_only', 'read_write'])
 def get_pool(pool):
     """ Get information of the given pool.
 
@@ -340,7 +367,8 @@ def get_pool(pool):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug('Get pool info of ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Get pool info of ' + pool)
 
     result = get_pool_info(pool)
     return result
@@ -361,6 +389,10 @@ def get_pool_info(poolname, serialize_out=False):
 
         result = deserialize(result, int_key=True)
 
+        burl = request.base_url
+        if burl.endswith('/'):
+            burl = burl[:-1]
+
         # Add url to tags
         dt_display = {}
         for t, clses in result['dTags'].items():
@@ -368,11 +400,11 @@ def get_pool_info(poolname, serialize_out=False):
                 continue
             cdict = {}
             for cls, sns in clses.items():
-                sndict = dict((int(sn), request.base_url+cls+'/'+sn)
+                sndict = dict((int(sn), '/'.join((burl, cls, sn)))
                               for sn in sns)
                 cdict[cls] = sndict
             dt_display[t] = cdict
-        display = {'Tags with URLs': dt_display}
+        display = {'Tags': dt_display}
         # add urls to urns
         ty_display = {}
         rec_u = 0
@@ -387,16 +419,17 @@ def get_pool_info(poolname, serialize_out=False):
                 # str makes a list to show in ome line
                 base = {'tags': str(snd['tags']),
                         'meta': str(snd['meta']),
-                        'url': request.base_url + cl + '/' + str(sn)
+                        'url': '/'.join((burl, cl, str(sn)))
                         }
                 snd_url[sn] = base
             ty_display[cl] = snd_url
-        display['Datatypes with URLs'] = ty_display
+        display['DataTypes'] = ty_display
         msg = 'Getting pool %s information. %s.' % (poolname, mes)
-        _, count, _ = get_data_count(None, poolname)
+        _, count, _, _ = get_data_count(None, poolname)
         msg += '%d data items recorded. %d counted.' % (rec_u, count)
     else:
         code, result, msg = 404, FAILED, poolname + ' is not an exisiting Pool ID.'
+        display = {'result': result}
     return resp(code, display, msg, ts, False)
 
 
@@ -404,8 +437,8 @@ def get_pool_info(poolname, serialize_out=False):
 ####  {pooolid}/register PUT /unreg DELETE  ####
 ######################################
 
+@ pools_api.route('/<string:pool>/', methods=['PUT'])
 @ pools_api.route('/<string:pool>', methods=['PUT'])
-# @ pools_api.route('/<string:pool>/', methods=['PUT'])
 @ auth.login_required(role='read_write')
 def register(pool):
     """ Register the given pool.
@@ -418,9 +451,13 @@ def register(pool):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug('register pool ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('register pool ' + pool)
 
     code, thepool, msg = register_pool(pool, auth.current_user())
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('pmsGLB%x' % id(PM_S._GlobalPoolList))
+
     res = thepool if issubclass(thepool.__class__, str) else thepool._poolurl
     return resp(code, res, msg, ts)
 
@@ -430,19 +467,22 @@ def register_pool(pool, usr):
 
     :returns: code, pool object if successful, message
     """
+
+    logger = current_app.logger
     poolname = pool
     fullpoolpath = join(
         current_app.config['FULL_BASE_LOCAL_POOLPATH'], poolname)
     poolurl = current_app.config['POOLURL_BASE'] + poolname
-    makenew = usr and usr.role == 'read_write'
+    # read_write has precedence over read_only
+    makenew = usr and ('read_write' in usr.roles)
     try:
-        po = PM.getPool(poolname=poolname, poolurl=poolurl, makenew=makenew)
+        po = PM_S.getPool(poolname=poolname, poolurl=poolurl, makenew=makenew)
         return 200, po, 'register pool ' + poolname + ' OK.'
     except (ValueError, NotImplementedError, PoolNotFoundError) as e:
         code, result, msg = excp(
             e,
             msg='Unable to register pool: ' + poolname)
-        current_app.logger.error(msg)
+        logger.error(msg)
         return code, result, msg
 
 
@@ -456,7 +496,8 @@ def unregister(pool):
     """
     logger = current_app.logger
     ts = time.time()
-    logger.debug('Unregister pool ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('Unregister pool ' + pool)
 
     code, result, msg = unregister_pool(pool)
     return resp(code, result, msg, ts)
@@ -472,7 +513,7 @@ def unregister_pool(pool):
     poolname = pool
     current_app.logger.debug('UNREGISTER (DELETE) POOL' + poolname)
     try:
-        result = PM.remove(poolname)
+        result = PM_S.remove(poolname)
         if result == 1:
             result = '1'
             msg = 'Pool not registered or referenced: ' + poolname
@@ -502,6 +543,7 @@ def unregister_pool(pool):
 
 # @ pools_api.route('/<string:pool>/hk', methods=['GET'])
 @ pools_api.route('/<string:pool>/hk/', methods=['GET'])
+@ auth.login_required(role=['read_only', 'read_write'])
 def hk(pool):
     """ All kinds of pool housekeeping data.
 
@@ -511,7 +553,8 @@ def hk(pool):
 
     ts = time.time()
     pool = pool.strip('/')
-    logger.debug('get HK for ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('get HK for ' + pool)
 
     code, result, msg = load_HKdata([pool, 'hk'])
 
@@ -527,7 +570,7 @@ def load_HKdata(paths, serialize_out=True):
     # resourcetype = fullname(data)
 
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.readHK(serialize_out=serialize_out)
         msg = 'OK.'
         code = 200
@@ -537,13 +580,14 @@ def load_HKdata(paths, serialize_out=True):
     return code, result, msg
 
 
-######################################
+################################################
 ####  {pool}/api/        List webAPIs       ####
-######################################
+################################################
 
 
-# @ pools_api.route('/<string:pool>/api', methods=['GET'])
 @ pools_api.route('/<string:pool>/api/', methods=['GET'])
+@ pools_api.route('/<string:pool>/api', methods=['GET'])
+@ auth.login_required(role=['read_only', 'read_write'])
 def api_info(pool):
     """ A list of names of allowed API methods.
 
@@ -553,7 +597,8 @@ def api_info(pool):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug(f'get allowed API methods for {pool}')
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'get allowed API methods for {pool}')
 
     return resp(200, WebAPI, 'OK.', ts, serialize_out=False)
 
@@ -564,7 +609,7 @@ def api_info(pool):
 
 
 @ pools_api.route('/<string:pool>/wipe', methods=['PUT'])
-# @ pools_api.route('/<string:pool>/wipe/', methods=['PUT'])
+@ pools_api.route('/<string:pool>/wipe/', methods=['PUT'])
 @ auth.login_required(role='read_write')
 def wipe(pool):
     """ Removes all contents of the pool.
@@ -573,7 +618,8 @@ def wipe(pool):
     """
     ts = time.time()
     logger = current_app.logger
-    logger.debug(f'wipe ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'wipe ' + pool)
 
     good, bad = wipe_pools([pool])
     if bad:
@@ -593,7 +639,7 @@ def wipe(pool):
 
 
 @ pools_api.route('/<string:pool>/hk/<string:kind>', methods=['GET'])
-# @ pools_api.route('/<string:pool>/hk/<string:kind>/', methods=['GET'])
+@ auth.login_required(role=['read_only', 'read_write'])
 def hk_single(pool, kind):
     """ Returns the given kind of pool housekeeping data.
     """
@@ -602,7 +648,8 @@ def hk_single(pool, kind):
 
     ts = time.time()
     pool = pool.strip('/')
-    logger.debug(f'get {kind} HK for ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'get {kind} HK for ' + pool)
 
     code, result, msg = load_single_HKdata([pool, 'hk', kind])
 
@@ -621,7 +668,7 @@ def load_single_HKdata(paths, serialize_out=True):
     if hkname not in HKDBS:
         raise ValueError('Invalid HK type. Must be one of '+', '.join(HKDBS))
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.readHK(hkname, serialize_out=serialize_out)
         code, msg = 200, hkname + ' HK data returned OK'
     except Exception as e:
@@ -633,17 +680,9 @@ def load_single_HKdata(paths, serialize_out=True):
 ######################################
 
 
-# @ pools_api.route('/<string:pool>/count/<string:data_type>', methods=['GET'])
-def count_singleXXX(pool, data_type):
-    """ Returns the number of given type of data in the given pool.
-
-    :data_type:  (part of) dot-separated full class name of data items in pool.
-    """
-
-    return count_general(pool=pool, data_type=data_type, logger=current_app.logger)
-
-
+@ pools_api.route('/<string:pool>/count/', methods=['GET'])
 @ pools_api.route('/<string:pool>/count', methods=['GET'])
+@ pools_api.route('/<string:pool>/count/<string:data_type>/', methods=['GET'])
 @ pools_api.route('/<string:pool>/count/<string:data_type>', methods=['GET'])
 def count(pool, data_type=None):
     """ Returns the number of all types or of a given type of data in the given pool.
@@ -659,10 +698,11 @@ def count(pool, data_type=None):
     ts = time.time()
     poolname = pool.strip('/')
     poolurl = current_app.config['POOLURL_BASE'] + poolname
-    logger.debug(f'get {data_type} count for ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'get {data_type} count for ' + pool)
 
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.getCount(data_type)
         code, msg = 200, 'Recorded count of %s returned OK' % (
             data_type if data_type else 'all types')
@@ -672,8 +712,10 @@ def count(pool, data_type=None):
     return resp(code, result, msg, ts, serialize_out=False)
 
 
-@ pools_api.route('/<string:pool>/count/', methods=['GET'])
-@ pools_api.route('/<string:pool>/count/<string:data_type>/', methods=['GET'])
+@ pools_api.route('/<string:pool>/counted/', methods=['GET'])
+@ pools_api.route('/<string:pool>/counted', methods=['GET'])
+@ pools_api.route('/<string:pool>/counted/<string:data_type>/', methods=['GET'])
+@ pools_api.route('/<string:pool>/counted/<string:data_type>', methods=['GET'])
 def counted(pool, data_type=None):
     """ Counts and Returns the number of given type of data in the given pool.
 
@@ -684,9 +726,10 @@ def counted(pool, data_type=None):
 
     ts = time.time()
     pool = pool.strip('/')
-    logger.debug(f'count {data_type} type for ' + pool)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'count {data_type} type for ' + pool)
 
-    code, result, msg = get_data_count(data_type=data_type, pool_id=pool)
+    code, result, msg, size = get_data_count(data_type=data_type, pool_id=pool)
 
     return resp(code, result, msg, ts, serialize_out=False)
 
@@ -700,22 +743,27 @@ def get_data_count(data_type, pool_id):
     """
 
     logger = current_app.logger
-    logger.debug('### method %s data_type %s pool %s***' %
-                 (request.method, data_type, pool_id))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('### method %s data_type %s pool %s***' %
+                     (request.method, data_type, pool_id))
     res = 0
     nm = []
 
     path = join(current_app.config['FULL_BASE_LOCAL_POOLPATH'], pool_id)
+    size = 0
     if os.path.exists(path):
         for i in os.listdir(path):
+            size += os.path.getsize(join(path, i))
             if i[-1].isnumeric() and ((data_type is None) or (data_type in i)):
                 res = res+1
                 nm.append(i)
     else:
-        return 400, 'FAILED', f'Pool {pool} not found.'
+        return 400, 'FAILED', f'Pool {pool} not found.', 0
     s = str(nm)
-    logger.debug(str(data_type) + ' found '+s)
-    return 200, res, 'Counted %d %s files OK.' % (res, data_type if data_type else 'all types')
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(('All types' if data_type is None else data_type) +
+                     ' found ' + lls(s, 120))
+    return 200, res, 'Counted %d %s files OK.' % (res, data_type if data_type else 'all types'), size
 
 
 ######################################
@@ -723,8 +771,8 @@ def get_data_count(data_type, pool_id):
 ######################################
 
 
-@ pools_api.route('/<string:pool>/api/<string:method_args>', methods=['GET', 'POST'])
 @ pools_api.route('/<string:pool>/api/<string:method_args>/', methods=['GET', 'POST'])
+@ pools_api.route('/<string:pool>/api/<string:method_args>', methods=['GET', 'POST'])
 @ auth.login_required(role='read_write')
 def api(pool, method_args):
     """ Call api mathods on the running pool and returns the result.
@@ -734,14 +782,16 @@ def api(pool, method_args):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug('get API for %s ; %s.' % (pool, lls(method_args, 200)))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('get API for %s, %s(%d args).' %
+                     (pool, method_args[0], len(method_args)))
     if request.method == 'POST':
         # long args are sent with POST
         if request.data is None:
-            result, msg = '"FAILED"', 'No REquest data for command '+request.method
+            result, msg = '"FAILED"', 'No Request data for command '+request.method
             code = 400
             return resp(code, result, msg, ts, serialize_out=True)
-        data = str(request.data, encoding='ascii')
+        data = request.get_data(as_text=True)
         paths = [pool, 'api', method_args, data]
     else:
         paths = [pool, 'api', method_args]
@@ -757,7 +807,7 @@ def api(pool, method_args):
 def call_pool_Api(paths, serialize_out=False, posted=False):
     """ Call api mathods on the running pool and returns the result.
 
-    return: value if args is pool property; execution result if method. 
+    return: value if args is pool property; execution result if method.
     """
 
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
@@ -780,39 +830,60 @@ def call_pool_Api(paths, serialize_out=False, posted=False):
         # from the unquoted url extract the fist path segment.
         quoted_m_args = request.url.split(
             paths[0] + '/' + paths[1] + '/')[1].strip('/')
-        logger.debug('get API : %s' % lls(quoted_m_args, 1000))
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug('get API : %s' % lls(quoted_m_args, 1000))
         # get command positional arguments and keyword arguments
         code, m_args, kwds = deserialize_args(
             quoted_m_args, serialize_out=serialize_out)
 
     if code != 200:
         result, msg = m_args, kwds
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug(f'RT{code} {msg}')
         return 0, resp(422, result, msg, ts, serialize_out=False), 0
 
     method = m_args[0]
 
     if method not in WebAPI:
-        return 0, resp(400, FAILED,
-                       'Unknown web API method: %s.' % method,
+        code = 400
+        msg = 'Unknown web API method: %s.' % method
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug(f'RT{code} {msg}')
+        return 0, resp(code, FAILED,
+                       msg,
                        ts, serialize_out=False), 0
     args = m_args[1:] if len(m_args) > 1 else []
     kwdsexpr = [str(k)+'='+str(v) for k, v in kwds.items()]
     msg = '%s(%s)' % (method, ', '.join(
-        chain(map(str, args), kwdsexpr)))
-    logger.debug('WebAPI ' + lls(msg, 300))
-    # if args and args[0] == 'select':
-    #    __import__('pdb').set_trace()
+        chain(((getattr(x, 'where', str(x)))[:100] for x in args), kwdsexpr)))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('WebAPI ' + lls(msg, 300) +
+                     's %x' % id(PM_S._GlobalPoolList))
+
+    if method == 'isEmpty':
+        print('********', list(PM_S._GlobalPoolList))
 
     poolname = paths[0]
     poolurl = current_app.config['POOLURL_BASE'] + poolname
-    if not PM.isLoaded(poolname):
-        result = FAILED
+    if not PM_S.isLoaded(poolname):
         msg = 'Pool not found or not registered: ' + poolname
-        logger.error(msg)
-        return 0, resp(404, result, msg, ts, serialize_out=False), 0
+        if method in ('removeAll'):
+            result = 'OK'
+            code = 200
+            if logger.isEnabledFor(logging_DEBUG):
+                logger.debug(f'RT{code} {msg}')
+            return 0, resp(code, result, msg, ts, serialize_out=False), 0
+        else:
+            result = FAILED
+            if logger.isEnabledFor(logging_ERROR):
+                logger.error(msg)
+            code = 404
+            if logger.isEnabledFor(logging_DEBUG):
+                logger.debug(f'RT{code} {msg}')
+            return 0, resp(code, result, msg, ts, serialize_out=False), 0
 
     try:
-        poolobj = PM.getPool(poolname=poolname, poolurl=poolurl)
+        poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         res = getattr(poolobj, method)(*args, **kwds)
         result = res
         msg = msg + ' OK.'
@@ -821,12 +892,6 @@ def call_pool_Api(paths, serialize_out=False, posted=False):
         code, result, msg = excp(e, 422, serialize_out=serialize_out)
         logger.error(msg)
 
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug(f'RT{code} {msg}')
     return 0, resp(code, result, msg, ts, serialize_out=False), 0
-
-
-@pools_api.before_request
-def before_request_callback():
-    path = request.path
-    method = request.method
-
-    print("pools_api " + path + " >>>>>>[" + method + "]<<<<<<")
