@@ -5,10 +5,14 @@ from ..dataset.deserialize import deserialize, serialize_args
 from .poolmanager import PoolManager
 from .productref import ProductRef
 from .productpool import ProductPool
+from ..httppool.session import requests_retry_session
+
 from .dicthk import HKDBS
 from ..utils.common import trbk, lls, fullname
 from .urn import Urn, makeUrn
 
+import requests
+
 import os
 import builtins
 from itertools import chain
@@ -38,19 +42,19 @@ def toserver(self, method, *args, **kwds):
     # if method == 'select':
     #    __import__('pdb').set_trace()
 
-    #apipath = serialize_args(method, *args, not_quoted=self.not_quoted, **kwds)
-    apipath = serialize([args, kwds])
+    apipath = serialize_args(method, *args, not_quoted=self.not_quoted, **kwds)
     urn = 'urn:::0'  # makeUrn(self._poolname, typename, 0)
 
-    logger.debug("READ PRODUCT FROM REMOTE===> " + urn)
-    if 0:  # len(apipath) < 800:
+    logger.debug("toServer ===> " + urn)
+    if len(apipath) < 80:
         code, res, msg = read_from_server(
             urn, self._poolurl, apipath, auth=self.auth)
     else:
+        apipath = serialize([args, kwds])
         code, res, msg = post_to_server(apipath, urn, self._poolurl,
                                         contents=method + '__' + '/',
-                                        no_serial=True, auth=self.auth)
-
+                                        no_serial=True,
+                                        auth=self.auth, client=self.client)
     if issubclass(res.__class__, str) and 'FAILED' in res or code != 200:
         for line in chain(msg.split('.', 1)[:1], msg.split('\n')):
             excpt = line.split(':', 1)[0]
@@ -83,14 +87,34 @@ class HttpClientPool(ProductPool):
     """ the pool will save all products on a remote server.
     """
 
-    def __init__(self, auth=None, **kwds):
-        """ Initialize connection to the remote server. creates file structure if there isn't one. if there is, read and populate house-keeping records. create persistent files on server if not exist.
+    def __init__(self, auth=None, client=None, **kwds):
+        """Initialize connection to the remote server.
+
+        Sreates file structure if there isn't one. if there is, read
+        and populate house-keeping records. create persistent files on
+        server if not exist.
+
+        Parameters
+        ----------
+        auth : tuple, HTTPBasicAuth, or Authorization
+            Authorization for remote pool.
+        client : Request or Wrapper
+            Mainly used for testing with mock server.
+        **kwds :
+
+        Returns
+        -------
+
 
         """
+
         # print(__name__ + str(kwds))
         super().__init__(**kwds)
         self.not_quoted = True
         self.auth = auth
+        if client is None:
+            client = requests_retry_session()
+        self.client = client
 
     def setup(self):
         """ Sets up HttpPool interals.
@@ -111,6 +135,7 @@ class HttpClientPool(ProductPool):
         poolname = self._poolname
         logger.debug("READ HK FROM REMOTE===>poolurl: " + poolname)
         hk = {}
+        code = None
         try:
             code, r, msg = read_from_server(
                 None, self._poolurl, 'housekeeping')
@@ -118,7 +143,8 @@ class HttpClientPool(ProductPool):
                 for hkdata in HKDBS:
                     hk[hkdata] = r[hkdata]
         except Exception as e:
-            msg = 'Reading %s failed.%d ' % (poolname, code) + str(e) + trbk(e)
+            msg = 'Reading %s failed. %s ' % (
+                poolname, 'No code' if code is None else str(code)) + str(e) + trbk(e)
             r = 'FAILED'
 
         if r == 'FAILED':
@@ -150,12 +176,13 @@ class HttpClientPool(ProductPool):
             first = False
         sized += ']'
         res = save_to_server(sized, urn, self._poolurl,
-                             tag, no_serial=True, auth=self.auth)
+                             tag, no_serial=True,
+                             auth=self.auth, client=self.client)
         sv = deserialize(res.text)
         if sv['result'] == 'FAILED' or res.status_code != 200:
             logger.error('Save %d products to server failed.%d Message from %s: %s' % (
                 len(productlist), res.status_code, self._poolurl, sv['msg']))
-            raise Exception(sv['msg'])
+            raise RuntimeError(sv['msg'])
         else:
             urns = sv['result']
         logger.debug('Product written to remote server successful')
@@ -195,7 +222,8 @@ class HttpClientPool(ProductPool):
         urn = makeUrn(self._poolname, resourcetype, indexstr)
         logger.debug("READ PRODUCT FROM REMOTE===> " + urn)
         code, res, msg = read_from_server(
-            urn, self._poolurl, auth=self.auth)  # TODO: start,end
+            urn, self._poolurl,
+            auth=self.auth, client=self.client)  # TODO: start,end
         if res == 'FAILED' or code != 200:
             raise NameError('Loading ' + urn + ' failed:%d. ' % code + msg)
         return res
@@ -210,7 +238,8 @@ class HttpClientPool(ProductPool):
             if resourcetype is None or index is None:
                 raise ValueError()
             urn = makeUrn(self._poolname, resourcetype, index)
-        res, msg = delete_from_server(urn, self._poolurl)
+        res, msg = delete_from_server(
+            urn, self._poolurl, auth=self.auth, client=self.client)
         if res == 'FAILED':
             logger.error('Remove from server ' + self._poolname +
                          ' failed. Caused by: ' + msg)
@@ -222,7 +251,8 @@ class HttpClientPool(ProductPool):
         does the scheme-specific remove-all
         """
 
-        res, msg = delete_from_server(None, self._poolurl, 'pool')
+        res, msg = delete_from_server(
+            None, self._poolurl, 'pool', client=self.client)
         if res == 'FAILED':
             logger.error(msg)
             raise Exception(msg)
@@ -405,11 +435,18 @@ class HttpClientPool(ProductPool):
     @ toServer()
     def where(self, qw, prod='BaseProduct', urns=None):
         """
-        Restore the remote pool with the contents of the named tar file.
+        AbstractQuery convenience api with 'p' for the product.
         """
         raise NotImplementedError
 
-###
+    @ toServer()
+    def qm(self, qw, prod='BaseProduct', urns=None):
+        """
+        MetaQuery convenience api with 'm' for 'meta'..
+        """
+        raise NotImplementedError
+
+#
 
 
 def serialize_args1(*args, **kwds):
