@@ -130,7 +130,7 @@ class LocalPool(ManagedPool):
                 fo.seek(start)
                 js = fo.read(end - start)
         except Exception as e:
-            msg = 'Error in HK reading. exc: %s trbk: %s.' % (str(e), trbk(e))
+            msg = 'Error in HK reading. file: %s. exc: %s trbk: %s.' % (fp, str(e), trbk(e))
             logging.error(msg)
             raise NameError(msg)
         if 1:  # close:
@@ -318,6 +318,7 @@ class LocalPool(ManagedPool):
 
         index: int
         """
+
         fp0 = self.transformpath(self._poolname)
         fp = pathjoin(fp0, quote(resourcetype) + '_' + str(index))
         try:
