@@ -1,18 +1,17 @@
 
 # -*- coding: utf-8 -*-
-import pdb
+import logging
+from weakref import WeakValueDictionary, getweakrefcount
+import getpass
 
 from ..utils.getconfig import getConfig
 from ..utils.common import lls
 from .urn import parse_poolurl
-from ..pns.fdi_requests import put_on_server, delete_from_server
-from ..pal.productpool import ProductPool
+from ..pal.httppool import HttpPool
 
 from requests.exceptions import ConnectionError
+import requests
 
-import getpass
-from weakref import WeakValueDictionary, getweakrefcount, finalize, getweakrefs
-import logging
 # create logger
 logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
@@ -25,42 +24,103 @@ DEFAULT_POOL = 'fdi_pool_' + __name__ + getpass.getuser()
 Invalid_Pool_Names = ['pools', 'urn', 'URN', 'api']
 
 
-def remoteRegister(poolurl, auth=None):
-    logger.debug('Register %s on the server', poolurl)
-    if poolurl.endswith('/'):
-        poolurl = poolurl[:-1]
-    try:
-        res, msg = put_on_server(
-            'urn:::0', poolurl, 'register_pool', auth=auth)
-    except ConnectionError as e:
-        res, msg = 'FAILED', str(e)
-        logger.error(poolurl + ' ' + msg)
-        raise
-    if res == 'FAILED':
-        np = '<' + auth.username + ' ' + auth.password + \
-            '>' if auth else '<no authorization>'
-        raise RuntimeError(
-            'Registering ' + poolurl + ' failed with auth ' + np + ' , ' + msg)
-    return res, msg
+def remoteRegister(pool):
+    """ if registered a pool's auth and client will be used.
+
+    Note that a new http/csdb pool gets remoteRegistered before locally registered.
+
+    Parameter
+    ---------
+    pool : HttpClientPool, PublicClientPool
+        Pool object to be registered on remote server and have client/session set.
+    auth : object
+        Authorization object for the client. If given will substitute that of pool, if pool has auth.
+    client : flask.requests (testing), or requests.Session
+        The client. If given will substitute that of pool, if pool is given
+
+    """
+    # pool object
+    poolo = None
+    from ..pal import httpclientpool, publicclientpool
+
+    if issubclass(pool.__class__, httpclientpool.HttpClientPool):
+
+        # HttpClientPool. Init the remote pool. If exists, load.d
+        poolurl = pool._poolurl
+        poolo = pool
+
+        logger.debug('Register %s on the server', poolurl)
+        if poolurl.endswith('/'):
+            poolurl = poolurl[:-1]
+        from ..pns.fdi_requests import put_on_server
+        try:
+            res, msg = put_on_server(
+                'urn:::0', poolurl, 'register_pool', auth=poolo.auth, client=poolo.client)
+        except ConnectionError as e:
+            res, msg = 'FAILED', str(e)
+            logger.error(poolurl + ' ' + msg)
+            raise
+        if res == 'FAILED':
+            np = '<' + poolo.auth.username + ' ' + poolo.auth.password + \
+                '>' if poolo.auth else '<no authorization>'
+            raise RuntimeError(
+                'Registering ' + poolurl + ' failed with auth ' + np + ' , ' + msg)
+        return res, msg
+    elif issubclass(pool.__class__, publicclientpool.PublicClientPool):
+
+        # register csdb pool. If existing, load. IF not exists, create and initialize sn.
+        status, poolres = pool.createPool2()
+
+        if status:
+            pool.getToken()
+            pool.client.headers.update({'X-AUTH-TOKEN': pool.token})
+            if poolres['code'] == 1:
+                # exists
+                pinfo = pool.getPoolInfo()
+            res = 'OK'
+        elif not status:
+            msg = poolres['msg']
+            logger.error(
+                'Registering ' + poolurl + ' failed with auth ' + 'np' + ' , ' + msg)
+            res = 'FAILED'
+        return res, poolres['msg']
+    else:
+        return
 
 
-def remoteUnregister(poolurl, auth=None):
+def remoteUnregister(poolurl, auth=None, client=None):
     """ this method does not reference pool object. """
+
     if not poolurl.lower().startswith('http'):
         logger.warning('Ignored: %s not for a remote pool.' % poolurl)
         return 1
     logger.debug('unregister %s on the server', poolurl)
+
+    # check if poolurl has been registered
+    for pool, poolo in PoolManager._GlobalPoolList.items():
+        if issubclass(poolo.__class__, HttpPool):
+            continue
+        if poolurl == poolo._poolurl:
+            if client is None:
+                client = poolo.client
+            if auth is None:
+                auth = poolo.auth
+            break
+    else:
+        raise NameError(
+            f'Remote Unregistering failed. {poolurl} not registered or not suitable.')
+    from ..pns.fdi_requests import delete_from_server
     #url = api_baseurl + post_poolid
     #x = requests.delete(url, auth=HTTPBasicAuth(auth_user, auth_pass))
     #o = deserialize(x.text)
     urn = 'urn:::0'
     try:
         res, msg = delete_from_server(
-            urn, poolurl, 'unregister_pool', auth=auth)
+            urn, poolurl, 'unregister_pool', auth=auth, client=client)
     except ConnectionError as e:
         res, msg = 'FAILED', str(e)
     if res == 'FAILED':
-        logger.warning('Ignored: Unregisterinf ' +
+        logger.warning('Ignored: Unregistering ' +
                        poolurl + ' failed.  ' + msg)
         code = 2
     else:
@@ -74,46 +134,70 @@ class PoolManager(object):
 
 This is done by calling the getPool() method, which will return an existing pool or create a new one if necessary.
     """
-    # Global centralized dict that returns singleton -- the same -- pool for the same ID.
+
     _GlobalPoolList = WeakValueDictionary()
+    """ Global centralized dict that returns singleton -- the same -- pool for the same ID."""
 
     # maps scheme to default place/poolpath
-    # pc['node']['host']+':'+str(pc['node']['port'])+pc['baseurl']
-    p = getConfig(name='').strip('/').split('://')[1]
+    # pc['host']+':'+str(pc['port'])+pc['baseurl']
+    p = getConfig('poolurl:').strip('/').split('://')[1]
     PlacePaths = {
         'file': pc['base_local_poolpath'],
         'mem': '/',
         'http': p,
         'https': p,
-        'server': pc['server_poolpath'],
+        'server': pc['server_local_poolpath'],
+        'csdb': pc['cloud_api_version']
     }
     del p
 
     @classmethod
-    def getPool(cls, poolname=None, poolurl=None, pool=None, makenew=True, auth=None, **kwds):
+    def getPool(cls, poolname=None, poolurl=None, pool=None, makenew=True, auth=None, client=None, **kwds):
         """ returns an instance of pool according to name or path of the pool.
 
-        Returns the pool object if the pool is registered. Creates the pool if it does not already exist. the same poolname-path always get the same pool. Http pools will be registered on the sserver side.
+        Returns the pool object if the pool is registered. Creates the pool if it does not already exist. the same poolname-path always get the same pool. Http pools will be registered on the server side.
 
 Pools registered are kept as long as the last reference remains. When the last is gone the pool gets :meth;`removed` d.
 
-        :poolname: name of the pool.
-        :poolurl: if given the poolpath, scheme, place will be derived from it. if not given for making a new pool (i.e. when poolname is not a registered pool name.. 
-If poolname is missing it is derived from poolurl; if poolurl is also absent, ValueError will be raised.
-        :makenew: when the pool does not exist, make a new one (````True```; default) or throws `PoolNotFoundError` (```False```).
-        :kwds: passed to pool instanciation arg-list.
-        :Returns: the pool object.
+        Parameter
+        ---------
+        poolname : str
+            name of the pool.
+        poolurl : str
+            If given the poolpath, scheme, place will be derived from it. if not given for making a new pool (i.e. when poolname is not a registered pool name. If poolname is missing it is derived from poolurl; if poolurl is also absent, ValueError will be raised.
+        pool: ProductPool
+            If `auth` and `client` are given they will substitute those of  `pool`. If `pool` is not given, those will need to be given.
+        makenew : bool
+            When the pool does not exist, make a new one (````True```; default) or throws `PoolNotFoundError` (```False```).
+        auth : str
+            For `remoteRegister`.
+        client : default is `None`.
+            For `remoteRegister`.
+        kwds  : dict
+            Passed to pool instanciation arg-list.
+
+        Returns
+        -------
+        ProductPool:
+            The pool object.
         """
         # logger.debug('GPL ' + str(id(cls._GlobalPoolList)) +
         #             str(cls._GlobalPoolList) + ' PConf ' + str(cls.PlacePaths))
-
         if pool:
             if poolname:
                 raise ValueError(
                     'Pool name %s and pool object cannot be both given.' % poolname)
-            poolname, poolurl, p = pool._poolname, pool._poolurl, pool
-            if poolurl.lower().startswith('http'):
-                res, msg = remoteRegister(poolurl, p.auth)
+            poolname, poolurl = pool._poolname, pool._poolurl
+            if cls.isLoaded(poolname):
+                return cls._GlobalPoolList[poolname]
+            if poolurl.lower()[:4] in ('http', 'csdb'):
+                if auth is not None and getattr(pool, 'auth', None) is None:
+                    pool.auth = auth
+                if client is not None and getattr(pool, 'client', None) is None:
+                    from ..httppool.session import requests_retry_session
+                    pool.client = requests_retry_session()
+                res, msg = remoteRegister(pool)
+            p = pool
         else:
             # quick decisions can be made knowing poolname only
             if poolname == DEFAULT_MEM_POOL:
@@ -127,11 +211,13 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
                     return cls._GlobalPoolList[poolname]
 
             # get poolname and scheme
+            if not poolurl:
+                poolurl = getConfig('poolurl:'+poolname)
             if poolurl:
                 pp, schm, pl, pn, un, pw = parse_poolurl(poolurl)
             else:
                 raise ValueError(
-                    'A new pool %s cannot be created without a pool url.' % poolname)
+                    'A new pool %s cannot be created without a pool url. Maybe the pool needs to be registered?' % poolname)
             if poolname:
                 if pn != poolname:
                     raise ValueError(
@@ -156,15 +242,25 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
                 from . import httppool
                 p = httppool.HttpPool(
                     poolname=poolname, poolurl=poolurl, **kwds)
-            elif schm in ('http', 'https'):
-                from . import httpclientpool
-                p = httpclientpool.HttpClientPool(
-                    poolname=poolname, poolurl=poolurl, auth=auth, **kwds)
-                res, msg = remoteRegister(poolurl, auth=p.auth)
-            elif schm == 'csdb':
-                from . import publicclientpool
-                p = publicclientpool.PublicClientPool(poolurl=poolurl)
-                # res, msg = remoteRegister(poolurl, auth=p.auth)
+            elif schm in ('http', 'https', 'csdb'):
+                if schm == 'csdb':
+                    from . import publicclientpool
+                    pooltype = publicclientpool.PublicClientPool
+                    purl = pc['scheme'] + poolurl[4:]
+                else:
+                    from . import httpclientpool
+                    pooltype = httpclientpool.HttpClientPool
+                    purl = poolurl
+                if auth is None:
+                    from requests.auth import HTTPBasicAuth
+                    auth = HTTPBasicAuth(pc['username'],
+                                         pc['password'])
+                if client is None:
+                    from ..httppool.session import requests_retry_session
+                    client = requests_retry_session()
+                p = pooltype(poolname=poolname, poolurl=purl,
+                             auth=auth, client=client, **kwds)
+                res, msg = remoteRegister(p)
             else:
                 raise NotImplementedError(schm + ':// is not supported')
         #print(getweakrefs(p), id(p), '////')
@@ -197,21 +293,22 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
             return 0
 
     @ classmethod
-    def removeAll(cls):
+    def removeAll(cls, ignore_error=False):
         """ deletes all pools from the pool list, pools not wiped
         """
         nl = list(cls._GlobalPoolList)
         for pool in nl:
-            cls.remove(pool)
+            cls.remove(pool, ignore_error=ignore_error)
 
     @ classmethod
     def save(cls, poolname, poolobj):
         """
         """
         cls._GlobalPoolList[poolname] = poolobj
+        poolobj.setPoolManager(cls)
 
     @ classmethod
-    def remove(cls, poolname):
+    def remove(cls, poolname, ignore_error=False):
         """ Remove from list and unregister remote pools.
 
         returns 0 for successful removal, ``1`` for poolname not registered or referenced, still attempted to remove. ``> 1`` for the number of weakrefs the pool still have, and removing failed.
@@ -225,8 +322,9 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
             # this is the only reference. unregister remote first.
             thepool = cls._GlobalPoolList[poolname]
             poolurl = thepool._poolurl
-            if poolurl.lower().startswith('http'):
-                code = remoteUnregister(poolurl, thepool.auth)
+            from ..pal.httpclientpool import HttpClientPool
+            if issubclass(thepool.__class__, HttpClientPool):
+                code = remoteUnregister(poolurl)
             else:
                 code = 0
         elif nwr > 1:
@@ -236,9 +334,13 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
             # nwr <=  0
             code = 1
         try:
-            del cls._GlobalPoolList[poolname]
-        except Exception as e:
-            logger.info("Ignored: "+str(e))
+            pool = cls._GlobalPoolList.pop(poolname)
+            pool.setPoolManager(None)
+        except KeyError as e:
+            if ignore_error:
+                logger.info("Ignored: "+str(e))
+            else:
+                raise
         return code
 
     @ classmethod
@@ -269,20 +371,22 @@ If poolname is missing it is derived from poolurl; if poolurl is also absent, Va
         """
         return self._GlobalPoolList.items()
 
-    def __setitem__(self, *args, **kwargs):
+    def __setitem__(self, poolname, poolobj):
         """ sets value at key.
         """
-        self._GlobalPoolList.__setitem__(*args, **kwargs)
+        self._GlobalPoolList.__setitem__(poolname, poolobj)
+        poolobj.setPoolManager(None, self.__class__)
 
     def __getitem__(self, *args, **kwargs):
         """ returns value at key.
         """
         return self._GlobalPoolList.__getitem__(*args, **kwargs)
 
-    def __delitem__(self, *args, **kwargs):
+    def __delitem__(self, poolname):
         """ removes value and its key.
         """
-        self._GlobalPoolList.__delitem__(*args, **kwargs)
+        self._GlobalPoolList[poolname].setPoolManager(None)
+        self._GlobalPoolList.__delitem__(poolname)
 
     def __len__(self, *args, **kwargs):
         """ size of data
