@@ -1,26 +1,28 @@
 # -*- coding: utf-8 -*-
-from ..dataset.odict import ODict
+
+from ..dataset.baseproduct import BaseProduct
 from ..dataset.classes import Classes
 from ..dataset.product import Product
 from ..dataset.serializable import serialize
-from ..dataset.deserialize import deserialize
+from ..dataset.deserialize import deserialize, Class_Look_Up
 from .urn import Urn, parseUrn, parse_poolurl, makeUrn
 from .versionable import Versionable
 from .taggable import Taggable
-from .dicthk import DictHk
+from . import dicthk
 from .definable import Definable
 from ..utils.common import pathjoin, fullname, lls, trbk
 from .productref import ProductRef
 from .query import AbstractQuery, MetaQuery, StorageQuery
 
+from collections import OrderedDict, ChainMap
+from functools import lru_cache
 import logging
 import filelock
 import getpass
+import functools
 import os
 import sys
 import builtins
-from collections import OrderedDict, ChainMap
-from functools import lru_cache
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
@@ -33,9 +35,6 @@ else:
 logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
-Class_Look_Up = ChainMap(Classes.mapping, globals(), vars(builtins))
-
-
 Lock_Path_Base = '/tmp/fdi_locks_' + getpass.getuser()
 # lock time-out
 locktout = 10
@@ -82,6 +81,7 @@ When implementing a ProductPool, the following rules need to be applied:
         self.setPoolurl(poolurl)
         # self._pathurl = pr.netloc + pr.path
         # self._pathurl = None
+        self._poolmanager = None
 
     class ParametersIncommpleteError(Exception):
         pass
@@ -164,19 +164,31 @@ When implementing a ProductPool, the following rules need to be applied:
         through visitor pattern."""
         visitor.visit(self)
 
+    def getPoolManager(self):
+        """
+        """
+        return self._poolmanager
+
+    def setPoolManager(self, pm):
+        """
+        """
+        self._poolmanager = pm
+
     def dereference(self, ref):
         """
         Decrement the reference count of a ProductRef.
+
+        XXX TODO
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def exists(self, urn):
         """
         Determines the existence of a product with specified URN.
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def getDefinition(self):
         """
@@ -207,13 +219,13 @@ When implementing a ProductPool, the following rules need to be applied:
         Returns all Product classes found in this pool.
         mh: returns an iterator.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def getReferenceCount(self, ref):
         """
         Returns the reference count of a ProductRef.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def getScheme(self):
         """
@@ -238,12 +250,12 @@ When implementing a ProductPool, the following rules need to be applied:
         Determines if the pool is empty.
         """
 
-        raise(NotImplementedError)
+        raise NotImplementedError
 
     def schematicSave(self, products, tag=None, geturnobjs=False, serialize_in=True, serialize_out=False, **kwds):
         """ to be implemented by subclasses to do the scheme-specific saving
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def saveProduct(self, product, tag=None, geturnobjs=False, serialize_in=True, serialize_out=False, **kwds):
         """
@@ -281,13 +293,13 @@ When implementing a ProductPool, the following rules need to be applied:
         Loads the descriptors belonging to specified URN.
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def schematicLoad(self, resourcetype, index,
                       start=None, end=None, serialize_out=False):
         """ to be implemented by subclasses to do the scheme-specific loading
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def loadProduct(self, urn, serialize_out=False):
         """
@@ -297,8 +309,8 @@ When implementing a ProductPool, the following rules need to be applied:
         """
         poolname, resource, index = parseUrn(urn)
         if poolname != self._poolname:
-            raise(ValueError('wrong pool: ' + poolname +
-                             ' . This is ' + self._poolname))
+            raise (ValueError('wrong pool: ' + poolname +
+                              ' . This is ' + self._poolname))
         ret = self.schematicLoad(
             resourcetype=resource, index=index, serialize_out=serialize_out)
 
@@ -321,7 +333,7 @@ When implementing a ProductPool, the following rules need to be applied:
         Loads the meta-data belonging to the product of specified URN.
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     @property
     def count(self):
@@ -340,65 +352,104 @@ When implementing a ProductPool, the following rules need to be applied:
         Return the number of URNs for the product type.
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def reference(self, ref):
         """
         Increment the reference count of a ProductRef.
         """
 
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def schematicRemove(self, urn=None, resourcetype=None, index=None):
         """ to be implemented by subclasses to do the scheme-specific removing
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
-    def remove(self, urn=None, resourcetype=None, index=None):
+    def remove(self, urn=None, resourcetype=None, index=None,
+               ignore_error=False):
         """
         Removes a Product belonging to specified URN or a pair of data type and serial number.
         """
+        self.ignore_error_when_delete = ignore_error
         res = self.schematicRemove(urn, resourcetype=resourcetype, index=index)
         return res
 
     def schematicWipe(self):
         """ to be implemented by subclasses to do the scheme-specific wiping.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
-    def removeAll(self):
+    def removeAll(self, ignore_error=False):
         """
         Remove all pool data (self, products) and all pool meta data (self, descriptors, indices, etc.).
         """
-
+        self.ignore_error_when_delete = ignore_error
         return self.schematicWipe()
 
     def saveDescriptors(self,  urn,  desc):
         """
         Save/Update descriptors in pool.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
-    def schematicSelect(self,  query, results=None):
+    def schematicSelect(self,  query, previous=None):
         """
         to be implemented by subclasses to do the scheme-specific querying.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def select(self,  query, variable='m', ptype=Product,
-               results=None):
-        """
-        Returns a list of references to products that match the specified query.
+               previous=None):
+        """Returns a list of references to products that match the specified query.
+
+        Parameters
+        ----------
+        query : str
+            the 'where' query string to make a query object.
+        variable : str
+            name of the dummy variable in the query string.
+            if `variable` is 'm', query goes via `MetaQuery(ptype, query)` ; else by `AbstractQuery(ptype, variable, query)` .
+        ptype : class
+            The class object whose instances are to be queried. Or
+            fragment of the name of such classes.
+        previous : list or str
+            of urns, possibly from previous search. or a string of comma-separated urns, e.g. `'urn:a:foo:12,urn:b:bar:9'`
+
+        Returns
+        -------
+        list
+            of found URNs.
+
         """
+        if issubclass(previous.__class__, str):
+            previous = previous.split(',')
         if issubclass(query.__class__, StorageQuery):
-            res = self.schematicSelect(query, results)
-        elif variable == 'm':
-            res = self.schematicSelect(MetaQuery(ptype, where=query), results)
+            res = self.schematicSelect(query, previous)
+            return res
+        if issubclass(ptype.__class__, str):
+            for cn, cls in Classes.mapping.items():
+                if ptype in cn and issubclass(cls, BaseProduct):
+                    break
+            else:
+                raise (ValueError(ptype + ' is not a product type.'))
+            ptype = cls
+        if variable == 'm':
+            res = self.schematicSelect(MetaQuery(ptype, where=query), previous)
         else:
             res = self.schematicSelect(AbstractQuery(
-                ptype, where=query, variable=variable), results)
+                ptype, where=query, variable=variable), previous)
         return res
 
+    def qm(self, qw, prod='BaseProduct', urns=None):
+        """ short-hand method for `select(qw, variable'm', ptype=prod, previous=urns`.
+
+        example:
+        ..code:
+        curl http://foo.edu:23456/data/pool/api/qm__m["age"]>66 and m["name"]=="Bob"'
+        """
+        return self.select(qw, variable='m', ptype=prod, previous=urns)
+
     def __repr__(self):
         co = ', '.join(str(k) + '=' + lls(v, 40)
                        for k, v in self.__getstate__().items())
@@ -422,12 +473,69 @@ class PoolNotFoundError(Exception):
     pass
 
 
+def _eval(code='', m='', **kwds):
+    """ evaluate compiled code with given local vars. """
+    try:
+        res = eval(code)
+    except NameError:
+        res = False
+        logger.debug("Evaluation error: %s. Traceback %s" %
+                     (str(e), trbk(e)))
+
+    return res
+
+
+def populate_pool2(tags, pn, dTypes=None, dTags=None):
+    """
+    tags : list
+        The tags in a list.
+    pn : str
+        The product name / datatype / class name of the data item, new or existing.
+
+    Returns
+    -------
+    tuple
+        dTypes and dTags with updates, and the index/serial number
+    """
+
+    # new ###
+    if dTypes is None:
+        dTypes = {}
+    if dTags is None:
+        dTags = {}
+
+    if pn in dTypes:
+        int_sn = dTypes[pn]['currentSN'] + 1
+    else:
+        int_sn = 0
+        dTypes[pn] = {
+            'currentSN': int_sn,
+            'sn': {}
+        }
+
+    snd = dTypes[pn]['sn']
+    _sn = str(int_sn)
+    if int_sn not in snd:
+        snd[int_sn] = {
+            'tags': [],
+            'meta': []
+        }
+
+    dTypes[pn]['currentSN'] = int_sn
+
+    # /new #####
+    for t in tags:
+        dicthk.add_tag_datatype_sn(t, pn, int_sn, dTypes, dTags)
+
+    return dTypes, dTags, _sn
+
+
 # Do not include leading or trailing whitespace as they are not guarantteed.
 MetaData_Json_Start = '{"_ATTR_meta":'
 MetaData_Json_End = '"_STID": "MetaData"}'
 
 
-class ManagedPool(ProductPool, DictHk):
+class ManagedPool(ProductPool, dicthk.DictHk):
     """ A ProductPool that manages its internal house keeping. """
 
     def __init__(self, **kwds):
@@ -503,7 +611,11 @@ class ManagedPool(ProductPool, DictHk):
         # new ###
         poolname, dt, sn = parseUrn(urn, int_index=True)
         # assert self._urns[ref.urn]['refcnt'] == self._dType[dt]['sn'][sn]['refcnt']
-        self._dType[dt]['sn'][sn]['refcnt'] -= 1
+        r = self._dType[dt]['sn'][sn]
+        if r['refcnt'] == 0:
+            raise ValueError('Cannot deref below 0.')
+        else:
+            r['refcnt'] -= 1
         # /new ###
         # self._urns[ref.urn]['refcnt'] -= 1
 
@@ -527,6 +639,7 @@ class ManagedPool(ProductPool, DictHk):
         """
         # new ###
         assert list(self._classes.keys()) == list(self._dTypes.keys())
+
         return self._classes.keys()
 
     def getCount(self, typename=None):
@@ -547,7 +660,7 @@ class ManagedPool(ProductPool, DictHk):
     def doSave(self, resourcetype, index, data, tags=None, serialize_in=True, **kwds):
         """ to be implemented by subclasses to do the action of saving
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def getReferenceCount(self, ref):
         """
@@ -644,33 +757,39 @@ class ManagedPool(ProductPool, DictHk):
                                    meta: [$start, $end]
                                    refcnt: $count
                example::
-                    foo.bar.Bar:
+                    'foo.bar.Bar':
                             'currentSN': 1
                             0:
-                               tags: ['cat', 'white']
-                               meta: [123, 456]
-                               refcnt: 0
+                               'tags': ['cat', 'white']
+                               'meta': [123, 456]
+                               'refcnt': 0
                             1:
-                               tags: ['dog', 'white']
-                               meta: [321, 765]
-                               refcnt: 0
-                    foo.baz.Baz:
+                               'tags': ['dog', 'white']
+                               'meta': [321, 765]
+                               'refcnt': 0
+                    'foo.baz.Baz':
                             'currentSN': 34
                             34:
-                               tags: ['tree', 'green']
-                               meta: [100, 654]
-                               refcnt: 1
+                               'tags': ['tree', 'green']
+                               'meta': [100, 654]
+                               'refcnt': 1
      `dTags` differs from `tag` 1. uses dType:[sn], instead of urn, so there is no poolname anywhere, 2. simplify by removing second level dict::
                      $tag_name0:
                            $class_name1:[$sn1, $sn2...]
                            $class_name2:[$sn3, ...]
-              exxample::
+               example::
                      'cat': { 'foo.bar.Bar':[0] }
                      'white': { 'foo.bar.Bar'; [0, 1] }
                      'dog': ...
 
         :res: list of result.
         :serialize_out: if True returns contents in serialized form.
+
+        Return
+        ------
+        `list` of the following:
+        `ProductRef`. `Urn` if `geturnobjs` is set. if`serialze_out` is set for `ProductRef` no product metadata is stored in the returned instance.
+        The result is also stored in the `re` parameter.
         """
         if serialize_in:
             pn = fullname(prd)
@@ -683,7 +802,6 @@ class ManagedPool(ProductPool, DictHk):
             pn = fullname(cls)
         with filelock.FileLock(self.lockpath('w')), \
                 filelock.FileLock(self.lockpath('r')):
-
             # some new ####
             self._classes, self._tags, self._urns, \
                 self._dTypes, self._dTags = tuple(
@@ -704,7 +822,7 @@ class ManagedPool(ProductPool, DictHk):
                 urn = makeUrn(poolname=self._poolname, typename=pn, index=sn)
 
                 if urn not in u:
-                    u[urn] = dict(tags=[])
+                    u[urn] = {'tags': []}
 
             # new+old ###
             if tag is None:
@@ -717,31 +835,46 @@ class ManagedPool(ProductPool, DictHk):
                 raise TypeError('Bad type for tag: %s.' %
                                 tag.__class__.__name__)
             # new ####
-            dTypes, dTags = self._dTypes, self._dTags
+            self._dTypes, self._dTags, _sn = populate_pool2(tags, pn,
+                                                            self._dTypes,
+                                                            self._dTags)
 
-            if pn in dTypes:
-                _sn = dTypes[pn]['currentSN'] + 1
-            else:
-                _sn = 0
-                dTypes[pn] = {
-                    'currentSN': _sn,
-                    'sn': {}
-                }
+            if 0:
+                if tag is None:
+                    tags = []
+                elif issubclass(tag.__class__, str):
+                    tags = [tag]
+                elif issubclass(tag.__class__, list):
+                    tags = tag
+                else:
+                    raise TypeError('Bad type for tag: %s.' %
+                                    tag.__class__.__name__)
+                # new ####
+                dTypes, dTags = self._dTypes, self._dTags
 
-            snd = dTypes[pn]['sn']
-            if _sn not in snd:
-                snd[_sn] = {
-                    'tags': [],
-                    'meta': []
-                }
+                if pn in dTypes:
+                    _sn = dTypes[pn]['currentSN'] + 1
+                else:
+                    _sn = 0
+                    dTypes[pn] = {
+                        'currentSN': _sn,
+                        'sn': {}
+                    }
 
-            dTypes[pn]['currentSN'] = _sn
+                snd = dTypes[pn]['sn']
+                if _sn not in snd:
+                    snd[_sn] = {
+                        'tags': [],
+                        'meta': []
+                    }
+
+                dTypes[pn]['currentSN'] = _sn
+                urn = makeUrn(poolname=self._poolname, typename=pn, index=_sn)
+                # /new #####
+
+                for t in tags:
+                    self.setTag(t, urn)
             urn = makeUrn(poolname=self._poolname, typename=pn, index=_sn)
-            # /new #####
-
-            for t in tags:
-                self.setTag(t, urn)
-
             try:
                 # save prod and HK
                 self.doSave(resourcetype=pn,
@@ -767,7 +900,8 @@ class ManagedPool(ProductPool, DictHk):
             else:
                 res.append(Urn(urn, poolurl=self._poolurl))
         else:
-            rf = ProductRef(urn=Urn(urn, poolurl=self._poolurl))
+            rf = ProductRef(urn=Urn(urn, poolurl=self._poolurl),
+                            poolmanager=self._poolmanager)
             if serialize_out:
                 # return without meta
                 res.append(rf)
@@ -778,8 +912,15 @@ class ManagedPool(ProductPool, DictHk):
 
     def schematicSave(self, products, tag=None, geturnobjs=False, serialize_in=True, serialize_out=False, **kwds):
         """ do the scheme-specific saving.
+        :product:  product or '[ size1, prd, size2, prd2, ...]'.
+        :serialize_out: if True returns contents in serialized form.
+        :serialize_in: if set, product input is seriaized.
 
-            :serialize_out: if True returns contents in serialized form.
+        Return
+        ------
+        `ProductRef`. `Urn` if `geturnobjs` is set. If `serialze_out` is set `str` of serialized form of `ProductRef` or `URN`.
+
+        `list` of the above of input is a list.
         """
         res = []
 
@@ -791,6 +932,7 @@ class ManagedPool(ProductPool, DictHk):
                              serialize_in, serialize_out, res, kwds)
             else:
                 for prd in products:
+                    # result is in res
                     self.saveOne(prd, tag, geturnobjs,
                                  serialize_in, serialize_out, res, kwds)
         else:
@@ -827,7 +969,7 @@ class ManagedPool(ProductPool, DictHk):
     def doLoad(self, resourcetype, index, start=None, end=None, serialize_out=False):
         """ to be implemented by subclasses to do the action of loading
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def schematicLoad(self, resourcetype, index, start=None, end=None,
                       serialize_out=False):
@@ -843,7 +985,7 @@ class ManagedPool(ProductPool, DictHk):
     def doRemove(self, resourcetype, index):
         """ to be implemented by subclasses to do the action of reemoving
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def schematicRemove(self, urn=None, resourcetype=None, index=None):
         """ do the scheme-specific removing
@@ -901,7 +1043,7 @@ class ManagedPool(ProductPool, DictHk):
     def doWipe(self):
         """ to be implemented by subclasses to do the action of wiping.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
     def schematicWipe(self):
         """ do the scheme-specific wiping
@@ -942,11 +1084,11 @@ class ManagedPool(ProductPool, DictHk):
 
         if reflist:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 for ref in reflist:
                     refmet = ref.getMeta()
                     m = refmet if refmet else self.getMetaByUrn(ref.urn)
-                    if eval(code):
+                    if _eval(code=code, m=m):
                         ret.append(ref)
                 return ret
             else:
@@ -958,21 +1100,23 @@ class ManagedPool(ProductPool, DictHk):
                 return ret
         elif urnlist:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 for urn in urnlist:
                     m = self.getMetaByUrn(urn)
-                    if eval(code):
-                        ret.append(ProductRef(urn=urn, meta=m))
+                    if _eval(code=code, m=m):
+                        ret.append(ProductRef(urn=urn, meta=m,
+                                   poolmanager=self._poolmanager))
                 return ret
             else:
                 for urn in urnlist:
                     m = self.getMetaByUrn(urn)
                     if qw(m):
-                        ret.append(ProductRef(urn=urn, meta=m))
+                        ret.append(ProductRef(urn=urn, meta=m,
+                                   poolmanager=self._poolmanager))
                 return ret
         elif snlist or datatypes:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 if snlist:
                     datatypes = {typename: snlist}
                 for cls in datatypes:
@@ -981,8 +1125,9 @@ class ManagedPool(ProductPool, DictHk):
                         urn = makeUrn(poolname=self._poolname,
                                       typename=typename, index=n)
                         m = self.getMetaByUrn(urn)
-                        if eval(code):
-                            ret.append(ProductRef(urn=urn, meta=m))
+                        if _eval(code=code, m=m):
+                            ret.append(ProductRef(urn=urn, meta=m,
+                                       poolmanager=self._poolmanager))
                 return ret
             else:
                 if snlist:
@@ -994,10 +1139,11 @@ class ManagedPool(ProductPool, DictHk):
                                       typename=typename, index=n)
                         m = self.getMetaByUrn(urn)
                         if qw(m):
-                            ret.append(ProductRef(urn=urn, meta=m))
+                            ret.append(ProductRef(urn=urn, meta=m,
+                                       poolmanager=self._poolmanager))
                 return ret
         else:
-            raise('Must give a list of ProductRef or urn or sn')
+            raise ('Must give a list of ProductRef or urn or sn')
 
     def prod_filter(self, q, cls=None, reflist=None, urnlist=None, snlist=None, datatypes=None):
         """ returns filtered collection using the query.
@@ -1023,10 +1169,10 @@ class ManagedPool(ProductPool, DictHk):
 
         if reflist:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 for ref in reflist:
                     glbs[var] = pref.getProduct()
-                    if eval(code):
+                    if _eval(code=code, m=m):
                         ret.append(ref)
                 if savevar != 'not in glbs':
                     glbs[var] = savevar
@@ -1041,18 +1187,18 @@ class ManagedPool(ProductPool, DictHk):
                 return ret
         elif urnlist:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 for urn in urnlist:
-                    pref = ProductRef(urn=urn)
+                    pref = ProductRef(urn=urn, poolmanager=self._poolmanager)
                     glbs[var] = pref.getProduct()
-                    if eval(code):
+                    if _eval(code=code):
                         ret.append(pref)
                 if savevar != 'not in glbs':
                     glbs[var] = savevar
                 return ret
             else:
                 for urn in urnlist:
-                    pref = ProductRef(urn=urn)
+                    pref = ProductRef(urn=urn, poolmanager=self._poolmanager)
                     glbs[var] = pref.getProduct()
                     if qw(glbs[var]):
                         ret.append(pref)
@@ -1061,7 +1207,7 @@ class ManagedPool(ProductPool, DictHk):
                 return ret
         elif snlist or datatypes:
             if isinstance(qw, str):
-                code = compile(qw, 'py', 'eval')
+                code = compile(qw, 'qw.py', 'eval')
                 if snlist:
                     datatypes = {cls.__name__: snlist}
                 for typename in datatypes:
@@ -1069,9 +1215,10 @@ class ManagedPool(ProductPool, DictHk):
                     cls = Class_Look_Up[typename.rsplit('.', 1)[-1]]
                     for n in snlist:
                         urno = Urn(cls=cls, poolname=self._poolname, index=n)
-                        pref = ProductRef(urn=urno)
+                        pref = ProductRef(
+                            urn=urno, poolmanager=self._poolmanager)
                         glbs[var] = pref.getProduct()
-                        if eval(code):
+                        if _eval(code=code):
                             ret.append(pref)
                     if savevar != 'not in glbs':
                         glbs[var] = savevar
@@ -1084,7 +1231,8 @@ class ManagedPool(ProductPool, DictHk):
                     cls = glbs[typename]
                     for n in snlist:
                         urno = Urn(cls=cls, poolname=self._poolname, index=n)
-                        pref = ProductRef(urn=urno)
+                        pref = ProductRef(
+                            urn=urno, poolmanager=self._poolmanager)
                         glbs[var] = pref.getProduct()
                         if qw(glbs[var]):
                             ret.append(pref)
@@ -1092,7 +1240,7 @@ class ManagedPool(ProductPool, DictHk):
                         glbs[var] = savevar
                 return ret
         else:
-            raise('Must give a list of ProductRef or urn or sn')
+            raise ('Must give a list of ProductRef or urn or sn')
 
     def where(self, qw, prod='BaseProduct', urns=None):
         q = AbstractQuery(prod, 'p', qw)
@@ -1109,13 +1257,13 @@ class ManagedPool(ProductPool, DictHk):
         #### assert [r.urn for r in res] == [r.urn for r in res2]
         return [r.urn for r in res2]
 
-    def doSelect(self, query, results=None):
+    def doSelect(self, query, previous=None):
         """
         to be implemented by subclasses to do the action of querying.
         """
-        raise(NotImplementedError)
+        raise (NotImplementedError)
 
-    def schematicSelect(self,  query, results=None):
+    def schematicSelect(self,  query, previous=None):
         """
         do the scheme-specific querying.
         """
@@ -1127,8 +1275,8 @@ class ManagedPool(ProductPool, DictHk):
         t, v, w, a = query.getType(), query.getVariable(
         ), query.getWhere(), query.retrieveAllVersions()
         ret = []
-        if results:
-            this = (x for x in results if x.urnobj.getPoolId()
+        if previous:
+            this = (x for x in previous if x.urnobj.getPoolId()
                     == self._poolname)
             if is_MetaQ:
                 ret += self.meta_filter(q=query, reflist=this)
