@@ -22,15 +22,41 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
     """ A lightweight reference to a product that is stored in a ProductPool or in memory.
     """
 
-    def __init__(self, urn=None, poolname=None, product=None, meta=None, **kwds):
-        """ Urn can be the string or URNobject. 
+    def __init__(self, urn=None, poolname=None, product=None, meta=None, poolmanager=None, **kwds):
+        """Urn can be the string or URNobject.
+
+        Parameters
+        ----------
+        urn : str
+        poolname : str
+             If given overrides the pool name in urn, and causes
+             metadata to be loaded from pool, unless this prodref
+             points to a mempool.
+        product : subclass of BaseProduct
+            A productref created from a single product will result in
+            a memory pool urn, and the metadata won't be loaded.
+        meta : Metadata
+            If meta is given, it will be used instead of that from
+            poolname.
+        poolmanager : class
+            subclass od PoolManager
+        **kwds :
+
+        Returns
+        -------
+        ProductRef
+            Reference of a product.
 
-        Poolname if given overrides the pool name in urn, and causes metadata to be loaded from pool, unless this prodref points to a mempool.
-        If meta is given, it will be used instead of that from poolname.
-        A productref created from a single product will result in a memory pool urn, and the metadata won't be loaded.
         """
 
         urnobj = None
+
+        if poolmanager:
+            # poolservers have their own PoolManagers
+            self._poolmanager = poolmanager
+        else:
+            from .poolmanager import PoolManager
+            self._poolmanager = PoolManager
         super(ProductRef, self).__init__(**kwds)
         if issubclass(urn.__class__, str):
             urnobj = Urn(urn)
@@ -44,10 +70,12 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
             urnobj = None
 
         if product is not None:
-            from .poolmanager import PoolManager, DEFAULT_MEM_POOL
+            from .poolmanager import DEFAULT_MEM_POOL
             from . import productstorage
-            pool = PoolManager.getPool(poolurl='mem:///' + DEFAULT_MEM_POOL)
-            st = productstorage.ProductStorage(pool)
+            pool = self._poolmanager.getPool(
+                poolurl='mem:///' + DEFAULT_MEM_POOL)
+            st = productstorage.ProductStorage(
+                pool, poolmanager=self._poolmanager)
             urnobj = st.save(product, geturnobjs=True)
             # a lone product passed to prodref will be stored to mempool
 
@@ -76,31 +104,38 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
         the product
         """
 
-        if self._poolname is None:
-            return None
         if hasattr(self, '_product') and self._product is not None:
             return self._product
-        from .poolmanager import PoolManager
-        p = PoolManager.getPool(self._poolname).loadProduct(self.getUrn())
+
+        poolname = self.getPoolname()
+        if poolname is None:
+            raise ValueError('ProductRef needs a poolname to get product.')
+        p = self._poolmanager.getPool(poolname).loadProduct(self.getUrn())
         if issubclass(p.__class__, Context):
             self._product = p
         return p
 
     def getPoolname(self):
         """ Returns the name of the product pool associated.
+
+        If not set, poolname from `getUrnObj().getPoolname()` is used and set to `self`.
         """
+        if self._poolname is None:
+            self._poolname = self.getUrnObj().getPoolname()
         return self._poolname
 
     def getStorage(self):
         """ Returns the product storage associated.
         """
-        raise NotImplementedError
+        st = productstorage.ProductStorage(self._poolname,
+                                           poolmanager=self._poolmanager)
+        self._storage = st
         return self._storage
 
     def setStorage(self, storage):
         """ Sets the product storage associated.
         """
-        raise NotImplementedError
+
         self._storage = storage
         # if hasattr(self, '_urn') and self._urn:
         #    self._meta = self._storage.getMeta(self._urn)
@@ -153,12 +188,12 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
 
         Parameters:
         -----------
-        urnobj: Urn
-        a URN object.
-        poolname: str
-        if given overrides the pool name in urn, and causes metadata to be loaded from pool.
+        urnobj : Urn
+            a URN object.
+        poolname : str
+            if given overrides the pool name in urn, and causes metadata to be loaded from pool.
         meta: MetaData
-        If  is given, it will be used instead of that from poolname.
+            If  is given, it will be used instead of that from poolname.
         """
         if urnobj is not None:
             uc = urnobj.__class__
@@ -174,7 +209,7 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
             if poolname is None:
                 poolname = urnobj.pool
             else:
-                pool = PoolManager.getPool(poolname)
+                pool = self._poolmanager.getPool(poolname)
             self._meta = (meta if meta else pool.meta(
                 urnobj.urn)) if loadmeta else None
             self._poolname = poolname
@@ -338,6 +373,7 @@ class ProductRef(MetaDataHolder, DeepEqual, Serializable, Comparable):
         return s
 
     string = toString
+    txt = toString
 
     __str__ = toString
 
