@@ -22,23 +22,38 @@ class ProductStorage(object):
 
     """
 
-    def __init__(self, pool=None, poolurl=None, **kwds):
+    def __init__(self, pool=None, poolurl=None, poolmanager=None,
+                 **kwds):
         """ Gets the storage "control pannel" for pool with specifed name.
 
-        pool: if is a string will be taken as a poolname. if is a pool object will be registered with its name,
-        poolurl: is sent to the PoolManager with poolname to get the pool object.
+        `auth` with `client`, can be given here to be passed to `PoolManager.getPool`.
+        `client` how to call remote api if poolurl indicates a remote pool. Default is `None` for using the configured host/port/credentials. If doing a mocked server test, this needs to be set.
+        :pool: can be given as the only pramemter. If `auth` and `client` are given they will substitute those of  `pool`. If `pool` is not given, those will need to be given.
+        :poolname: if is a string will be taken as a poolname. if is a pool object will be registered with its name,
+        :poolurl: is sent to the PoolManager with poolname to get the pool object.
+
         """
         if issubclass(pool.__class__, str) and ':' in pool:
             raise TypeError(
                 'First argument must be a poolname or a pool object, not ' + str(pool))
         super(ProductStorage, self).__init__()
+
+        if poolmanager:
+            # poolservers have their own PoolManagers
+            self.PM = poolmanager
+        else:
+            from .poolmanager import PoolManager
+            self.PM = PoolManager
+
         self._pools = ODict()  # dict of poolname - poolobj pairs
         self.register(pool=pool, poolurl=poolurl, **kwds)
 
-    def register(self,  poolname=None, poolurl=None, pool=None, **kwds):
+    def register(self,  poolname=None, poolurl=None, pool=None,
+                 **kwds):
         """ Registers the given pools to the storage.
 
-
+        :client: passed to `PoolManager.getPool`.
+        :auth: passed to `PoolManager.getPool`.
         """
 
         if issubclass(pool.__class__, str) and poolname is None:
@@ -46,7 +61,7 @@ class ProductStorage(object):
         with filelock.FileLock(makeLockpath('ProdStorage', 'w')), \
                 filelock.FileLock(makeLockpath('ProdStorage', 'r')):
             if pool and issubclass(pool.__class__, ProductPool):
-                _p = PoolManager.getPool(pool=pool, **kwds)
+                _p = self.PM.getPool(pool=pool, **kwds)
             elif poolurl is None and poolname is None:
                 # quietly return for no-arg construction case
                 return
@@ -57,14 +72,14 @@ class ProductStorage(object):
                 if poolurl is not None and not issubclass(poolurl.__class__, str):
                     raise TypeError('Poolurl must be a string, not ' +
                                     poolurl.__class__.__name__)
-                _p = PoolManager.getPool(
-                    poolname=poolname, poolurl=poolurl, **kwds)
+                _p = self.PM.getPool(poolname=poolname, poolurl=poolurl,
+                                     **kwds)
             self._pools[_p._poolname] = _p
 
         logger.debug('registered pool %s -> %s.' %
                      (str(pool), str(self._pools)))
 
-    def unregister(self, pool=None, **kwds):
+    def unregister(self, pool=None, ignore_error=False, **kwds):
         """ Unregisters the given pools to the storage.
         """
 
@@ -73,9 +88,10 @@ class ProductStorage(object):
                 poolname = pool.getId()
             else:
                 poolname = pool
-            if PoolManager.isLoaded(poolname):
+            if self.PM.isLoaded(poolname):
                 # remove frpm pool manager
-                res = PoolManager.remove(poolname)  # TODO i dentify self
+                # TODO i dentify self
+                res = self.PM.remove(poolname, ignore_error=ignore_error)
                 # do this after del above
                 del self._pools[poolname]
                 logger.debug('unregistered pool %s -> %s.' %
@@ -84,8 +100,8 @@ class ProductStorage(object):
                 logger.info('Pool %s is not registered.' % poolname)
         return
 
-    def unregisterAll(self):
-        PoolManager.removeAll()
+    def unregisterAll(self, ignore_error=False):
+        self.PM.removeAll(ignore_error=ignore_error)
         self._pools.clear()
 
     def load(self, urnortag):
@@ -141,10 +157,11 @@ class ProductStorage(object):
         elif poolname not in self._pools:
             self.register(poolname)
 
-        desc = [x.description for x in product] if issubclass(
-            product.__class__, list) else product.description
-        logger.debug('saving product:' + str(desc) +
-                     ' to pool ' + str(poolname) + ' with tag ' + str(tag))
+        if logger.getEffectiveLevel() <= logging.DEBUG:
+            desc = [x.description for x in product] if issubclass(
+                product.__class__, list) else product.description
+            logger.debug('saving product:' + str(desc) +
+                         ' to pool ' + str(poolname) + ' with tag ' + str(tag))
 
         try:
             ret = self._pools[poolname].saveProduct(
@@ -153,16 +170,21 @@ class ProductStorage(object):
         except Exception as e:
             logger.error('unable to save to the writable pool.')
             raise
+        from fdi.pal.productref import ProductRef
+        if issubclass(ret.__class__, list):
+            if ret and issubclass(ret[0].__class__, ProductRef):
+                for r in ret:
+                    r.setStorage(self)
         return ret
 
-    def remove(self, urn):
+    def remove(self, urn, ignore_error=False):
         """ removes product of urn from the writeable pool
         """
         poolname = self.getWritablePool()
         logger.debug('removing product:' + str(urn) +
                      ' from pool ' + str(poolname))
         try:
-            self._pools[poolname].remove(urn)
+            self._pools[poolname].remove(urn, ignore_error=ignore_error)
         except Exception as e:
             logger.error('unable to remove from the writable pool.')
             raise e
@@ -181,11 +203,12 @@ class ProductStorage(object):
     def getPools(self):
         """  Returns the set of ProductPools registered.
         mh: in a list of poolnames
+        XXX TODO: getPoolnames
         """
         return list(self._pools.keys())
 
     def getPool(self, poolname):
-        """ mh: returns the pool object from poolname
+        """ mh: returns the pool object by poolname from this storage
         """
         if poolname not in self._pools:
             msg = 'pool ' + poolname + ' not found'
@@ -193,10 +216,14 @@ class ProductStorage(object):
             raise NameError(msg)
         return self._pools[poolname]
 
-    def getWritablePool(self):
+    def getWritablePool(self, obj=False):
         """ returns the poolname of the first pool, which is the only writeable pool.
+
+        :obj: (bool) return the pool objject instead of the name.
         """
-        return self.getPools()[0]
+        l = list(self._pools.items())
+
+        return l[0][1] if obj else l[0][0]
 
     def getAllTags(self):
         """ Get all tags defined in the writable pool.
@@ -230,22 +257,48 @@ class ProductStorage(object):
 
         return self._pools[self.getWritablePool()].getUrn(tag)
 
-    def wipePool(self):
+    def wipePool(self, ignore_error=False):
         """ Clear all data and meta data of the writable pool.
         """
 
-        list(self._pools.values())[0].removeAll()
+        val = self._pools.values
+        list(val())[0].removeAll(ignore_error=ignore_error)
 
-    def select(self, query, previous=None):
+    def isEmpty(self):
+        """ Returns whether all pools are empty or there is no pool. """
+
+        return len(self._pools) == 0 or all(p.isEmpty() for p in self._pools.values())
+
+    def select(self, query, variable=None, ptype=None, previous=None):
         """ Returns a list of URNs to products that match the specified query.
 
-        Parameters:
-        query - the query object
-        previous - results to be refined
-        Returns:
+        Parameters
+        ----------
+        query : the query object, or str
+            The Query instances or
+             the 'where' query string to make a query object.
+        variable : str
+            name of the dummy variable in the query string.
+            if `variable` is 'm', query goes via `MetaQuery(ptype, query)` ; else by `AbstractQuery(ptype, variable, query)` .
+        ptype : class
+            The class object whose instances are to be queried. Or
+            fragment of the name of such classes.
+        previous : list or str
+            of urns, possibly from previous search. or a string of comma-separated urns, e.g. `'urn:a:foo:12,urn:b:bar:9'`
+
+        Returns
+        -------
         the set of return eferences to products matching the supplied query.
         """
         ret = []
+        if issubclass(previous.__class__, str):
+            previous = previous.split(',')
+        if issubclass(query.__class__, str) and ptype and variable:
+            # search all registered pools
+            for poolnm, pool in self._pools.items():
+                ret += pool.select(query, variable, ptype, previous)
+            return ret
+
         # search all registered pools
         for poolnm, pool in self._pools.items():
             ret += pool.select(query, previous)
