@@ -3,8 +3,9 @@ from .comparable import Comparable
 from ..dataset.serializable import Serializable
 from ..dataset.odict import ODict
 from ..dataset.eq import DeepEqual
-from ..dataset.classes import Classes
+from ..dataset.classes import Class_Look_Up
 from ..utils.common import fullname
+
 import sys
 import os
 import builtins
@@ -26,6 +27,11 @@ else:
     from urlparse import urlparse
 
 
+def is_urn(u):
+    sp = u.split(':')
+    return len(sp) == 4 and sp[0].lower() and sp[3].isdigit()
+
+
 def makeUrn(poolname, typename, index):
     """ assembles a URN with infos of the pool, the resource type, and the index
 
@@ -132,7 +138,6 @@ URNs are used to to identify data be cause URNs are location agnostic. Storage P
 
         poolname, resourcetype, index = parseUrn(urn)
 
-        Class_Look_Up = ChainMap(Classes.mapping, globals(), vars(builtins))
         cls = Class_Look_Up[resourcetype.split('.')[-1]]
 
         self._poolname = poolname
@@ -224,6 +229,7 @@ URNs are used to to identify data be cause URNs are location agnostic. Storage P
             )
 
     string = toString
+    txt = toString
 
 
 def parseUrn(urn, int_index=True):
@@ -243,7 +249,7 @@ def parseUrn(urn, int_index=True):
     ------
     :poolname: NAme of the pool
     :resourceclass: type of resource/products
-    :index: serial number of resourceclass in the pool.
+    :index: (int) serial number of resourceclass in the pool.
 
     """
 
@@ -373,7 +379,7 @@ class UrnUtils():
     def getClass(urn):
         """ Get the class contained in a URN. """
         pn, prod, sn = parseUrn(urn)
-        return Classes.mapping[prod.rsplit('.', 1)[1]]
+        return Class_Look_Up[prod.rsplit('.', 1)[1]]
 
     @ staticmethod
     def getClassName(urn):
