@@ -11,6 +11,7 @@ import traceback
 import pprint
 import textwrap
 import copy
+import fnmatch
 import os
 from pathlib import Path
 import pwd
@@ -29,6 +30,11 @@ else:
 logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
+logging_ERROR = logging.ERROR
+logging_WARNING = logging.WARNING
+logging_INFO = logging.INFO
+logging_DEBUG = logging.DEBUG
+
 
 def str2md5(string):
     return hashlib.md5(string.encode('utf8')).hexdigest()
@@ -39,7 +45,7 @@ def trbk(e):
     """
     ls = [x for x in traceback.extract_tb(e.__traceback__).format()] if hasattr(
         e, '__traceback__') else ['']
-    return ' '.join(ls) + ' ' + \
+    return '\n'.join(ls) + ' ' + \
         (e.child_traceback if hasattr(e, 'child_traceback') else '')
 
 
@@ -48,10 +54,10 @@ def trbk2(e):
     return ''.join(tb.stack.format())
 
 
-def bstr(x, length=0, tostr=True, quote="'", level=0,
-         width=0, heavy=True, yaml=False, html=False,
+def bstr(x, length=0, tostr=True, quote="'", yaml=False, html=False,
          **kwds):
     """ returns the best string representation.
+
     if the object is a string, return single-quoted; if has toString(), use it; else returns str(). Length limited by lls(lls)
     """
 
@@ -63,9 +69,7 @@ def bstr(x, length=0, tostr=True, quote="'", level=0,
         r = quote + x + quote
     elif tostr and hasattr(x, 'toString') and not issubclass(x.__class__, type):
         # has toString()
-        r = x.toString(level=level,
-                       width=width, heavy=heavy,
-                       **kwds)
+        r = x.toString(**kwds)
     elif issubclass(x.__class__, (bytes, bytearray, memoryview)):
         r = x.hex()
     else:
@@ -88,91 +92,6 @@ def lls(s, length=80):
         return '%s...%s' % (st[:l], st[3 + l - length:])
 
 
-""" https://stackoverflow.com/a/2718268
-LHan = [[0x2E80, 0x2E99],    # Han # So  [26] CJK RADICAL REPEAT, CJK RADICAL RAP
-        # Han # So  [89] CJK RADICAL CHOKE, CJK RADICAL C-SIMPLIFIED TURTLE
-        [0x2E9B, 0x2EF3],
-        [0x2F00, 0x2FD5],    # Han # So [214] KANGXI RADICAL ONE, KANGXI RADICAL FLUTE
-        0x3005,              # Han # Lm       IDEOGRAPHIC ITERATION MARK
-        0x3007,              # Han # Nl       IDEOGRAPHIC NUMBER ZERO
-        # Han # Nl   [9] HANGZHOU NUMERAL ONE, HANGZHOU NUMERAL NINE
-        [0x3021, 0x3029],
-        # Han # Nl   [3] HANGZHOU NUMERAL TEN, HANGZHOU NUMERAL THIRTY
-        [0x3038, 0x303A],
-        0x303B,              # Han # Lm       VERTICAL IDEOGRAPHIC ITERATION MARK
-        # Han # Lo [6582] CJK UNIFIED IDEOGRAPH-3400, CJK UNIFIED IDEOGRAPH-4DB5
-        [0x3400, 0x4DB5],
-        # Han # Lo [20932] CJK UNIFIED IDEOGRAPH-4E00, CJK UNIFIED IDEOGRAPH-9FC3
-        [0x4E00, 0x9FC3],
-        # Han # Lo [302] CJK COMPATIBILITY IDEOGRAPH-F900, CJK COMPATIBILITY IDEOGRAPH-FA2D
-        [0xF900, 0xFA2D],
-        # Han # Lo  [59] CJK COMPATIBILITY IDEOGRAPH-FA30, CJK COMPATIBILITY IDEOGRAPH-FA6A
-        [0xFA30, 0xFA6A],
-        # Han # Lo [106] CJK COMPATIBILITY IDEOGRAPH-FA70, CJK COMPATIBILITY IDEOGRAPH-FAD9
-        [0xFA70, 0xFAD9],
-        # Han # Lo [42711] CJK UNIFIED IDEOGRAPH-20000, CJK UNIFIED IDEOGRAPH-2A6D6
-        [0x20000, 0x2A6D6],
-        [0x2F800, 0x2FA1D]]  # Han # Lo [542] CJK COMPATIBILITY IDEOGRAPH-2F800, CJK COMPATIBILITY IDEOGRAPH-2FA1D
-"""
-
-
-@lru_cache(maxsize=128)
-def wcw(char):
-    # cached width function
-    from ..dataset.metadata import wcwidth
-    return wcwidth.wcwidth(char)
-
-
-def wcw_wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
-    line = []
-    for s in st.splitlines():
-        lens = len(s)
-        # starting index for current line based on the last line
-        lasti = 0
-        # display length starting from the beginning of the last line.
-        l = 0
-        for i, c in enumerate(s):
-            w = wcw(c)
-            l0 = l
-            if w == -1:
-                # change unprintable
-                # ref https://wcwidth.readthedocs.io/en/latest/api.html
-                c = unprintable
-                w = wcw(c)
-                l += w
-            else:
-                l += w
-            # print(i, c, l, lasti, s)
-            if l == width:
-                line.append(c)
-                line.append(linebreak)
-                lasti, l = i+1, 0
-            elif l > width:
-                if width < 2:
-                    # print wide characters even they are too wide for width==1
-                    line.append(c)
-                    line.append(linebreak)
-                    lasti = i+1
-                    l = 0
-                else:
-                    # set line pointer to this char
-                    if fill:
-                        line.append((width-l0) * fill)
-                    line.append(linebreak)
-                    line.append(c)
-                    lasti = i
-                    l = w
-            else:
-                line.append(c)
-        if len(line) == 0 or line[-1] != '\n':
-            if fill:
-                line.append((width-l) * fill)
-            line.append(linebreak)
-        # print('*****', line)
-    end = len(linebreak)
-    return ''.join(line[:-end])
-
-
 def wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
     """ generates a string comtaining width-limited strings separated with '\n'.
 
@@ -180,27 +99,47 @@ def wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
     Removes trailing line-breaks.
 
     :st: input string. If not a string, ```str(st)``` is used.
-    :width: if > 0  returns the str with `linebreak` inserted every width chars max. Or else return the input ``st``. Default width is 15. A CJK characters occupies 2 in widths.
+    :width: if > 0  returns the str with `linebreak` inserted every width chars max. Default width is 15. A CJK characters occupies 2 in widths.
     :linebreak: line-break character. default `\n`
     :unprintable: substitute unprintable characters with this, only active if wide or unprintable characters are found. default is '#'.
     """
     if not issubclass(st.__class__, str):
         st = str(st)
-    if width <= 0 or len(st) == 0:
+    if len(st) == 0:
         return st
+
+    # wrap text.
+    if width > 0:
+        lines = []
+        for s in st.splitlines():
+            s = textwrap.wrap(s, width=width, replace_whitespace=False,
+                              drop_whitespace=False)
+            lines.extend(s)
+        return linebreak.join(lines)
+    return linebreak.join(st.splitlines())
+
+    # wrap text. This is obsolete for tabulate since tabulate 0.8.10
     if len(st.encode('utf8')) == len(st) and fill is None:
         lines = []
-        [lines.extend(textwrap.wrap(s, width=width, replace_whitespace=False,
-                                    drop_whitespace=False))
-         for s in st.splitlines()]
+        for s in st.splitlines():
+            if width > 0:
+                s = textwrap.wrap(s, width=width, replace_whitespace=False,
+                                  drop_whitespace=False)
+                lines.extend(s)
+            else:
+                lines.append(s)
+            lines.append(s)
         return linebreak.join(lines)
     else:
         # string has CJK characters
-        return wcw_wls(st, width=width, fill=fill,
-                       linebreak=linebreak, unprintable=unprintable)
+        return linebreak.join(st.splitlines())
+
+        # TODO: fix width breaking for CJK
+        # return wcw_wls(st, width=width, fill=fill,
+        #               linebreak=linebreak, unprintable=unprintable)
 
 
-def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0, **kwds):
+def mstr(obj, level=0, excpt=None, indent=4, depth=0, **kwds):
     """ Makes a presentation string at a detail level.
 
     'tablefmt' is needed to be passed in recursive calls under some conditions it is used.
@@ -396,7 +335,7 @@ def attrstr(p, v, missingval='', ftime=False, state=True, width=1, **kwds):
                 # binary masked. validity is a list of tuple/lists
                 # validity is (val, state, mask height, mask width)
                 sep = '\n' if width else ', '
-                vs = sep.join(r[1] if r[1] == 'Invalid' else'%s (%s)' %
+                vs = sep.join(r[1] if r[1] == 'Invalid' else '%s (%s)' %
                               (r[1], format(r[0], '#0%db' % (r[3]+2))) for r in validity)
     return vs
 
@@ -414,11 +353,11 @@ def attrstr1(p, v, missingval='', ftime=False, state=True, width=1, **kwds):
 
     ts = getattr(p, '_type') if hasattr(p, '_type') else missingval
     if ts is None:
-        ts = 'None'
+        ts = missingval
     if hasattr(p, v):
         val = getattr(p, v)
         if val is None:
-            return 'None'
+            return missingval
         val_cls = val.__class__
         # from ..dataset.finetime import FineTime
         # if issubclass(val_cls, FineTime):
@@ -474,13 +413,13 @@ def exprstrs(param, v='_value', extra=False, **kwds):
     """ Generates a set of strings for param.toString().
 
     :param: Parameter or xDstaset.
-    :extra: Whether to include less often used attributes suc as ```fits_keyword```.
+    :extra: Whether to include less often used attributes such as ```fits_keyword```.
     """
     if issubclass(param.__class__, dataset.metadata.Parameter):
         extra_attrs = copy.copy(param._all_attrs)
     elif issubclass(param.__class__, (dataset.arraydataset.ArrayDataset,
                                       dataset.tabledataset.TableDataset,
-                                      dataset.unstructureddataset.UnstructuredDataset)):
+                                      dataset.unstructureddataset.UnstructuredDataset)):  # if v['default'] else '')
         extra_attrs = dict((n, v['default'])
                            for n, v in param.zInfo['metadata'].items())
     else:
@@ -652,37 +591,40 @@ def findShape(data, element_seq=(str)):
                     d.__class__, Mapping) else d[0]
             except (TypeError, IndexError, KeyError) as e:
                 d = None
-    return tuple(shape)
+    return shape
 
 
-def find_all_files(datadir, verbose=False, include=None, exclude=None, not_if=None):
+def find_all_files(datadir, verbose=False, include=None, exclude=None, not_if=None, absdir=False):
     """ returns a list of names of all files in `datadir`.
 
-    :name: of starting directory
-    :include: only if a file name has any of these sub-strings.
+    :name: of starting directory or a list of file name strings to filter.
+    :include: only if a file name has any of these sub-strings. format is as if used in `glob(include)`.
     :exclude: only if a file name has not any of these sub-strings. Empty strings are removed.
-    :not_if: a function that returns true if given a name of unwanted file. default is None, ```os.is_dir```, which excludes directories.
+    :not_if: a function that returns true if given a name of unwanted file. default is None, (which excludes directories when `datadir` is a string, and disabled if `datadir` is a `list`.
+    :absdir: Set to True to return absolute_paths.
     """
 
     allf = []
-    if not_if is None:
-        not_if = os.path.isdir
-    # def ok(f, inc, exc, verbose=False):
-    #     if inc and all(i not in f for i in inc):
-    #         return False
-    #     if exc and any(e in f for e in exc):
-    #         return False
-    #     return True
+
     if not include:
         include = '*'
-    inc = Path(datadir).glob(include)
-    #li = list(inc)
+
+    isadir = issubclass(datadir.__class__, str)
+    if isadir:
+        if not_if is None:
+            not_if = os.path.isdir
+        inc = Path(datadir).glob(include)
+        inc = list(str(f) for f in inc)
+    else:
+        inc = fnmatch.filter(datadir, include)
+
     #print("find", len(inc))
 
     if exclude is None:
         exclude = []
-    allf = list(str(f) for f in inc if not any(
-        e in f.name for e in exclude if e != '') and not (not_if(f)))
+    ab = os.path.abspath if absdir else lambda x: x
+    allf = list(ab(f) for f in inc if not any(
+        e in f for e in exclude if e != '') and not (isadir and not_if(f)))
 
     # for root, dirs, files in os.walk(datadir):
     #     if verbose:
@@ -694,3 +636,91 @@ def find_all_files(datadir, verbose=False, include=None, exclude=None, not_if=No
     if verbose:
         print('Find %d files total.' % len(allf))
     return allf
+
+########### old code grave yard ############
+
+
+""" https://stackoverflow.com/a/2718268
+LHan = [[0x2E80, 0x2E99],    # Han # So  [26] CJK RADICAL REPEAT, CJK RADICAL RAP
+        # Han # So  [89] CJK RADICAL CHOKE, CJK RADICAL C-SIMPLIFIED TURTLE
+        [0x2E9B, 0x2EF3],
+        [0x2F00, 0x2FD5],    # Han # So [214] KANGXI RADICAL ONE, KANGXI RADICAL FLUTE
+        0x3005,              # Han # Lm       IDEOGRAPHIC ITERATION MARK
+        0x3007,              # Han # Nl       IDEOGRAPHIC NUMBER ZERO
+        # Han # Nl   [9] HANGZHOU NUMERAL ONE, HANGZHOU NUMERAL NINE
+        [0x3021, 0x3029],
+        # Han # Nl   [3] HANGZHOU NUMERAL TEN, HANGZHOU NUMERAL THIRTY
+        [0x3038, 0x303A],
+        0x303B,              # Han # Lm       VERTICAL IDEOGRAPHIC ITERATION MARK
+        # Han # Lo [6582] CJK UNIFIED IDEOGRAPH-3400, CJK UNIFIED IDEOGRAPH-4DB5
+        [0x3400, 0x4DB5],
+        # Han # Lo [20932] CJK UNIFIED IDEOGRAPH-4E00, CJK UNIFIED IDEOGRAPH-9FC3
+        [0x4E00, 0x9FC3],
+        # Han # Lo [302] CJK COMPATIBILITY IDEOGRAPH-F900, CJK COMPATIBILITY IDEOGRAPH-FA2D
+        [0xF900, 0xFA2D],
+        # Han # Lo  [59] CJK COMPATIBILITY IDEOGRAPH-FA30, CJK COMPATIBILITY IDEOGRAPH-FA6A
+        [0xFA30, 0xFA6A],
+        # Han # Lo [106] CJK COMPATIBILITY IDEOGRAPH-FA70, CJK COMPATIBILITY IDEOGRAPH-FAD9
+        [0xFA70, 0xFAD9],
+        # Han # Lo [42711] CJK UNIFIED IDEOGRAPH-20000, CJK UNIFIED IDEOGRAPH-2A6D6
+        [0x20000, 0x2A6D6],
+        [0x2F800, 0x2FA1D]]  # Han # Lo [542] CJK COMPATIBILITY IDEOGRAPH-2F800, CJK COMPATIBILITY IDEOGRAPH-2FA1D
+"""
+
+
+# @lru_cache(maxsize=128)
+# def wcw(char):
+#     # cached width function
+#     from ..dataset.metadata import wcwidth
+#     return wcwidth.wcwidth(char)
+
+
+# def wcw_wls(st, width=15, fill=None, linebreak='\n', unprintable='#'):
+#     # for CJK this is obsolete with the CJK code in `wls` for tabulate
+#     line = []
+#     for s in st.splitlines():
+#         lens = len(s)
+#         # starting index for current line based on the last line
+#         lasti = 0
+#         # display length starting from the beginning of the last line.
+#         l = 0
+#         for i, c in enumerate(s):
+#             w = wcw(c)
+#             l0 = l
+#             if w == -1:
+#                 # change unprintable
+#                 # ref https://wcwidth.readthedocs.io/en/latest/api.html
+#                 c = unprintable
+#                 w = wcw(c)
+#                 l += w
+#             else:
+#                 l += w
+#             # print(i, c, l, lasti, s)
+#             if l == width:
+#                 line.append(c)
+#                 line.append(linebreak)
+#                 lasti, l = i+1, 0
+#             elif l > width:
+#                 if width < 2:
+#                     # print wide characters even they are too wide for width==1
+#                     line.append(c)
+#                     line.append(linebreak)
+#                     lasti = i+1
+#                     l = 0
+#                 else:
+#                     # set line pointer to this char
+#                     if fill:
+#                         line.append((width-l0) * fill)
+#                     line.append(linebreak)
+#                     line.append(c)
+#                     lasti = i
+#                     l = w
+#             else:
+#                 line.append(c)
+#         if len(line) == 0 or line[-1] != '\n':
+#             if fill:
+#                 line.append((width-l) * fill)
+#             line.append(linebreak)
+#         # print('*****', line)
+#     end = len(linebreak)
+#     return ''.join(line[:-end])
