@@ -4,7 +4,7 @@
 # This test is to be run on the same machine where the http pool server is running.
 #################
 
-
+from fdi.httppool.model.user import LOGIN_TMPLT
 from fdi.dataset.unstructureddataset import UnstructuredDataset
 from test_dataset import bookstore, simple_ex, complex_ex, do_jsonPath
 from fdi.dataset.testproducts import get_demo_product
@@ -14,20 +14,24 @@ from fdi.dataset.product import Product
 from fdi.pal.httpclientpool import HttpClientPool
 from fdi.pal.poolmanager import PoolManager
 from fdi.pal.productstorage import ProductStorage
-from fdi.pal.productpool import Lock_Path_Base
+from fdi.pal.productpool import Lock_Path_Base, makeLockpath
 from fdi.utils.common import lls, trbk, fullname
 from fdi.utils.fetch import fetch
+from fdi.pns.fdi_requests import safe_client
 from fdi.pns.jsonio import auth_headers
 
 import pytest
 import filelock
+
 import sys
+import copy
+import concurrent.futures
+import requests.models
 from urllib.request import pathname2url
 from urllib.error import URLError
 
-from flask import current_app
+from flask import current_app, g, request
 from requests.auth import HTTPBasicAuth
-from requests.models import Response as rmResponse  # mock server
 from flask.wrappers import Response as fwResponse  # live server
 # import requests
 import random
@@ -36,13 +40,8 @@ import requests
 import pytest
 from pprint import pprint
 import time
-import getpass
 from collections.abc import Mapping
 
-import asyncio
-import aiohttp
-
-from fdi.pns.jsonio import getJsonObj, postJsonObj, putJsonObj, commonheaders
 from fdi.utils.options import opt
 
 
@@ -66,16 +65,6 @@ logger.setLevel(logging.INFO)
 logger.debug('logging level %d' % (logger.getEffectiveLevel()))
 
 
-if 0:
-    @pytest.fixture(scope="module")
-    def runserver():
-        from fdi.pns.httppool_server import app
-        app.run(host='127.0.0.1', port=5000,
-                threaded=False, debug=verbose, processes=5)
-
-        return smtplib.SMTP("smtp.gmail.com", 587, timeout=5)
-
-
 # last time/lastUpdate
 lupd = 0
 
@@ -96,10 +85,11 @@ if 0:
     sys.exit()
 
 
-@pytest.fixture(scope="module")
-def project_app(pc):
-    from fdi.httppool import create_app
-    return create_app(config_object=pc, level=logger.getEffectiveLevel())
+def make_auth(userpass):
+    return userpass
+    # for mock server.
+    # return
+    #   "basic", {"username": userpass[0], "password": userpass[1]})
 
 
 def issane(o):
@@ -147,7 +137,7 @@ def getPayload(aResponse, ignore_error=True):
         return x, aResponse.status_code
 
 
-def check_response(o, code=200, failed_case=False, excluded=None):
+def check_response(o, code=200, failed_case=False, excluded=None, login=False, ordered=True):
     """ Generic checking.
 
     :o: deserialized response data or text.
@@ -164,12 +154,21 @@ def check_response(o, code=200, failed_case=False, excluded=None):
         issubclass(oc, str) and 'Bad string to decode as JSON' not in o or \
         issubclass(oc, bytes) and not o.startswith(b'<!DOC') or \
         someone, o
-    if not failed_case:
+    if failed_case is None:
+        return True
+    elif not failed_case:
         if not someone:
             # properly formated
-            assert 'FAILED' != o['result'], o['result']
+            if login:
+                if issubclass(oc, bytes) and o.startswith(b'<!doctype html>'):
+                    assert b'password' in o
+                    return True
+                else:
+                    return o
+            assert 'FAILED' != o['result'], o['msg']
             assert code == 200, str(o)
-            assert o['time'] > lupd
+            if ordered:
+                assert o['time'] > lupd
             lupd = o['time']
             return True
     else:
@@ -192,62 +191,88 @@ def test_clear_local_server(local_pools_dir):
     assert not os.path.exists(ppath)
 
 
+def test_svcl(server, client):
+    aburl, headers = server
+    cl = client
+    print(cl)
+    print(aburl)
+
+
 def test_root(server, client):
     aburl, headers = server
     url = aburl + '/'
-    x = client.get(url)
+    x = safe_client(client.get, url)
     o, code = getPayload(x)
     check_response(o, code=code)
     c0 = o['result']  # a list
     # no slash
     url = aburl
-    x = client.get(url)
+    x = safe_client(client.get, url)
     o, code = getPayload(x)  # a dict of urls
     if check_response(o, code=code, excluded=['Redirecting']):
         c = o['result']
         assert set(c0) == set(c)
     # /
-    url = aburl + '/pools'
-    x = client.get(url, headers=headers)
+    url = aburl + '/pools/'
+    x = safe_client(client.get, url, headers=headers)
     o, code = getPayload(x)
     check_response(o, code=code)
     c_pools = o['result']  # a dict
     assert set(iter(c_pools)) <= set(iter(c0))
 
 
-def make_pools(name, aburl, clnt, auth, n=1):
-    """ generate n pools """
+def test_wipe_all_pools_on_server(server, tmp_local_remote_pools, local_pools_dir, client, auth):
 
-    lst = []
-    for i in range(n):
-        poolid = name + str(i)
-        pool = PoolManager.getPool(poolid, aburl + '/'+poolid)
-        lst.append(pool)
-        data = serialize(Product('lone prod in '+poolid))
-        url = aburl + '/' + poolid + '/'
-        x = clnt.post(url, auth=auth, data=data)
-        o, code = getPayload(x)
-        check_response(o, code=code, failed_case=False)
-    return lst[0] if n == 1 else lst
+    npools = tmp_local_remote_pools
+    n = len(npools)
+    pool, prd, ref, tag = npools[0]
+    # ======== wipe all pools =====
+    logger.info('Wipe all pools on the server')
+
+    # register all pools and get count
+    aburl, headers = server
+    url = aburl + '/' + 'pools/register_all'
+    x = safe_client(client.post, url, auth=auth)
+    o, code = getPayload(x)
+    check_response(o, code=code, failed_case=False)
+    regd = o['result']
+
+    # make an unregistered pool by copying an existing pool
+    p = os.path.join(local_pools_dir, pool.getId())
+    os.system('cp -rf %s %s_copy' % (p, p))
+    assert os.path.exists(p+'_copy')
+
+    assert len(get_files_in_local_dir('', local_pools_dir)) >= n+1
+
+    # wipe all pools
+    url = aburl + '/' + 'pools/wipe_all'
+    x = safe_client(client.delete, url, auth=auth)
+    o, code = getPayload(x)
+    check_response(o, code=code, failed_case=False)
+
+    files = get_files_in_local_dir('', local_pools_dir)
+    assert len(files) == 0, 'Wipe_all_pools failed: ' + \
+        o['msg'] + 'Files ' + str(files)
 
 
-def test_wipe_all_pools_on_server(server, local_pools_dir, client, userpass):
+def est_wipe_all_pools_on_server(server, local_pools_dir, client, auth):
     aburl, headers = server
     post_poolid = test_poolid
-    auth = HTTPBasicAuth(*userpass)
     # ======== wipe all pools =====
     logger.info('Wipe all pools on the server')
 
     # register all pools and get count
     url = aburl + '/' + 'pools/register_all'
-    x = client.put(url, auth=auth)
+    x = safe_client(client.post, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code, failed_case=False)
     regd = o['result']
 
     # make some pools
     n = 5
-    lst = make_pools(post_poolid, aburl, client, auth, n)
+    import random
+    pn = post_poolid+str(random.randint(0, 100))
+    lst = make_pools(pn, aburl, client, auth, n)
     # make an unregistered pool by copying an existing pool
     p = os.path.join(local_pools_dir, lst[0].getId())
     os.system('cp -rf %s %s_copy' % (p, p))
@@ -257,7 +282,7 @@ def test_wipe_all_pools_on_server(server, local_pools_dir, client, userpass):
 
     # wipe all pools
     url = aburl + '/' + 'pools/wipe_all'
-    x = client.delete(url, auth=auth)
+    x = safe_client(client.delete, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code, failed_case=False)
 
@@ -270,14 +295,14 @@ def test_new_user_read_write(new_user_read_write, pc):
     """
     GIVEN a User model
     WHEN a new User is created
-    THEN check the username, hashed_password, authenticated, and role fields are defined correctly
+    THEN check the username, hashed_password, authenticated, and roles fields are defined correctly
     https://www.patricksoftwareblog.com/testing-a-flask-application-using-pytest/
     """
     new_user, headers = new_user_read_write
-    assert new_user.username == pc['node']['username']
-    assert new_user.hashed_password != pc['node']['password']
+    assert new_user.username == pc['username']
+    assert new_user.hashed_password != pc['password']
     assert not new_user.authenticated
-    assert new_user.role == 'read_write'
+    assert new_user.roles == ('read_write',)
     logger.debug('Done.')
 
 
@@ -285,32 +310,56 @@ def test_new_user_read_only(new_user_read_only, pc):
     """
     GIVEN a User model
     WHEN a new User is created
-    THEN check the username, hashed_password, authenticated, and role fields are defined correctly
+    THEN check the username, hashed_password, authenticated, and roles fields are defined correctly
     """
     new_user, headers = new_user_read_only
-    assert new_user.username == pc['node']['ro_username']
+    assert new_user.username == pc['ro_user']
     assert not new_user.hashed_password.startswith('o')
     assert not new_user.authenticated
-    assert new_user.role == 'read_only'
+    assert new_user.roles == ('read_only',)
     logger.debug('Done.')
 
 
 def getapis(server_ro, client):
     aburl, headers = server_ro
-    x = client.get(aburl.rsplit('/', 1)[0]+'/apispec_1.json', headers=headers)
-    return x.json()
+    x = safe_client(client.get, aburl.rsplit('/', 1)
+                    [0]+'/apispec_1.json', headers=headers)
+    return deserialize(x.text if type(x) == requests.models.Response else x.data)
 
 
-def test_unauthorizedread_write(server, server_ro, client):
+def test_unauthorizedread_write(server, server_ro, client, tmp_local_remote_pools):
     aburl, headers = server
     roaburl, roheaders = server_ro
-    poolid = test_poolid
+    pool, prod, ref, tag = tmp_local_remote_pools[0]
+    poolid = pool.poolname
     # generate a unauthorized user header
-    uheaders = auth_headers('k', 'hu8')
-    x = client.get(aburl+'/pools', headers=uheaders)
-    assert x.status_code == 401
-    o, code = getPayload(x, ignore_error=True)
-    assert o == 'Unauthorized Access'
+    uheaders = auth_headers('k', 'hu8', headers=headers)
+    if uheaders['server_type'] == 'mock':
+        # we need to run before_requet, so we call full_dispatch_request
+        # ref https://flask.palletsprojects.com/en/2.2.x/testing/#accessing-and-modifying-the-session
+        with current_app.test_request_context(aburl+'/pools/', method='GET', headers=uheaders):
+            # now we have request set
+            with current_app.test_client() as client:
+                # now we have client set
+                with client.session_transaction() as session:
+                    # now we have session set
+                    user_id = session.get('user_id')
+                    hdrs = str(request.headers)
+                    logger.debug('*** session %x user_id = "%s"' %
+                                 (id(session), str(user_id))+hdrs)
+                    x = current_app.full_dispatch_request()
+    else:
+        x = safe_client(client.get, aburl+'/pools/', headers=uheaders)
+    if LOGIN_TMPLT:
+        # In order to use the login page, the return code has to be 200
+        assert x.status_code == 200
+        o, code = getPayload(x, ignore_error=False)
+        assert str(o).startswith('<!doctype html>')
+    else:
+        # If json is to be used, the return code shoul be 401 to trigger browser login prompt
+        assert x.status_code == 401
+        o, code = getPayload(x, ignore_error=True)
+        assert o in ('Unauthorized Access', b'Unauthorized Access')
 
     # These needs read_write
     paths = getapis(server_ro, client)['paths']
@@ -320,14 +369,21 @@ def test_unauthorizedread_write(server, server_ro, client):
             if meth == 'post':
                 print(meth, aburl+api, '""')
                 # unknown user
-                x = client.post(aburl+api, headers=uheaders, data='')
+                x = safe_client(client.post, aburl+api,
+                                headers=uheaders, data='')
                 assert x.status_code == 200 if p == '/pool/{method_args}' else 401
                 # read_only
-                x = client.post(roaburl+api, headers=roheaders, data='')
-                assert x.status_code == 200 if p == '/user/login' \
-                    else 401 if p == '/user/logout' else 403
+                x = safe_client(client.post, roaburl+api,
+                                headers=roheaders, data='')
+                if LOGIN_TMPLT:
+                    # In order to use the login page, the return code has to be 200
+                    assert x.status_code == 200 if p == '/login' \
+                        else 401 if p == '/logout' else 403
+                assert x.status_code == 200 if p == '/login' \
+                    else 401 if p == '/logout' else 403
                 # read_write
-                x = client.post(roaburl+api, headers=headers, data='')
+                x = safe_client(client.post, aburl+api,
+                                headers=headers, data='')
                 assert x.status_code == 401 if p == '/user/logout' else 200
 
     logger.debug('Done.')
@@ -335,11 +391,11 @@ def test_unauthorizedread_write(server, server_ro, client):
 
 def test_authorizedread_write(server, new_user_read_write, client):
     aburl, headers = server
-    x = client.get(aburl+'/pools', headers=headers)
+    x = safe_client(client.get, aburl+'/pools/', headers=headers)
     assert x.status_code == 200
     # with pytest.raises(URLError):
     o, code = getPayload(x)
-    check_response(o, code=code)
+    check_response(o, code=code, login=True)
     pools = o['result']
     assert isinstance(pools, list)
 
@@ -357,7 +413,7 @@ def clear_server_local_pools_dir(poolid, local_pools_dir):
         # x.creator = 'test'
         # data = serialize(x)
         # url = aburl + '/' + test_poolid + '/fdi.dataset.product.Product/0'
-        # x = requests.post(url, auth=HTTPBasicAuth(*userpass), data=data)
+        # x = requests.post(url, auth=make_auth(userpass), data=data)
 
 
 def get_files_in_local_dir(poolid, local_pools_dir):
@@ -373,8 +429,8 @@ def get_files_in_local_dir(poolid, local_pools_dir):
 
 def empty_pool(post_poolid, aburl, auth, clnt):
     path = post_poolid + '/api/removeAll'
-    url = aburl + '/' + path
-    x = clnt.get(url, auth=HTTPBasicAuth(*auth))
+    url = aburl + '/' + path + '/'  # trailing / is needed by mock server
+    x = clnt.get(url, auth=auth)
     o, code = getPayload(x)
     # ignore "FAILED" so non-exisiting target will not cause a failed case.
     check_response(o, code=code, failed_case=None)
@@ -391,7 +447,7 @@ def populate_pool(poolid, aburl, auth, clnt):
         x.creator = i
         data = serialize(x)
         url = aburl + '/' + poolid + '/'
-        x = clnt.post(url, auth=HTTPBasicAuth(*auth), data=data)
+        x = clnt.post(url, auth=auth, data=data)
         # print(len(data))
         o, code = getPayload(x)
         check_response(o, code=code)
@@ -400,17 +456,7 @@ def populate_pool(poolid, aburl, auth, clnt):
     return creators, instruments, urns
 
 
-@ pytest.fixture(scope='function')
-def thepool(server, client, userpass):
-    aburl, headers = server
-    auth = HTTPBasicAuth(*userpass)
-    # register
-    pool = 1  # make_pools(test_poolid, aburl, client, auth, n=1)
-    yield pool, test_poolid
-    del pool
-
-
-def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
+def test_CRUD_product(local_pools_dir, server, auth, client):
     ''' test saving, read, delete products API, products will be saved at /data/pool_id
     '''
 
@@ -418,17 +464,19 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     aburl, headers = server
 
     post_poolid = test_poolid
-    auth = HTTPBasicAuth(*userpass)
+    # auth = HTTPBasicAuth(*userpass) # not working with mock server
+    # auth = tuple(userpasss.values())
+
     # register
-    pool = make_pools(test_poolid, aburl, client, auth, n=1)
-    empty_pool(post_poolid, aburl, userpass, client)
+    #pool = make_pools(test_poolid, aburl, client, auth, n=1)
+    empty_pool(post_poolid, aburl, auth, client)
 
     files = [f for f in get_files_in_local_dir(
         post_poolid, local_pools_dir) if f[-1].isnumeric()]
     origin_prod = len(files)
 
     creators, instruments, urns = populate_pool(
-        post_poolid, aburl, userpass, client)
+        post_poolid, aburl, auth, client)
 
     files1 = [f for f in get_files_in_local_dir(
         post_poolid, local_pools_dir) if f[-1].isnumeric()]
@@ -440,13 +488,12 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     newfiles = set(files1) - set(files)
     us = set(u.split(':', 2)[2].replace(':', '_') for u in urns)
     assert newfiles == us, str(newfiles) + str(us)
-
     # ==========
     logger.info('read product')
 
     u = random.choice(urns)
     url = aburl + '/' + u  # [4:].replace(':', '/')
-    x = client.get(url, auth=auth)
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     assert o['result'].creator == creators[urns.index(u)], 'Creator not match'
@@ -456,11 +503,13 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     '''
     logger.info('read hk')
     hkpath = '/hk'
-    url = aburl + '/' + post_poolid + hkpath
-    x = client.get(url, auth=auth)
+    url = aburl + '/' + post_poolid + hkpath + \
+        '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o1, c1 = getPayload(x)
-    url2 = aburl + '/' + post_poolid + '/api/readHK'
-    x2 = client.get(url2, auth=auth)
+    url2 = aburl + '/' + post_poolid + '/api/readHK' + \
+        '/'  # trailing / is needed by mock server
+    x2 = safe_client(client.get, url2, auth=auth)
     o2, c2 = getPayload(x2)
     for o, c in [(o1, c1), (o2, c2)]:
         check_response(o, code=c)
@@ -482,7 +531,7 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     logger.info('read dTypes')
     hkpath = '/hk/dTypes'
     url = aburl + '/' + post_poolid + hkpath
-    x = client.get(url, auth=auth)
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     assert list(o['result'][prodt]['sn'])[-l:] == inds
@@ -493,8 +542,9 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     logger.info('check count')
     num = len(o['result'][prodt]['sn'])
     apipath = '/api/getCount__' + prodt
-    url = aburl + '/' + post_poolid + apipath
-    x = client.get(url, auth=auth)
+    url = aburl + '/' + post_poolid + apipath + \
+        '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     assert o['result'] == num
@@ -502,7 +552,8 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     logger.info('read dTags')
     hkpath = '/hk/dTags'
     url = aburl + '/' + post_poolid + hkpath
-    x = client.get(url, auth=auth)
+    # '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     assert len(o['result']) == 0
@@ -518,7 +569,7 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     # poolname following 'urn' immediately
     url = aburl + '/urn' + post_poolid + '/fdi.dataset.product.Product/' + index
 
-    x = client.delete(url, auth=auth)
+    x = safe_client(client.delete, url, auth=auth)
 
     o, code = getPayload(x)
     check_response(o, code=code)
@@ -538,7 +589,7 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     url2 = aburl + '/urn' + ':' + post_poolid + \
         '/fdi.dataset.product.Product/' + index2
 
-    x = client.delete(url2, auth=auth)
+    x = safe_client(client.delete, url2, auth=auth)
 
     o, code = getPayload(x)
     check_response(o, code=code)
@@ -559,16 +610,18 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     assert len(files) != 0, 'Pool is already empty: ' + post_poolid
 
     # wipe the pool on the server
-    url = aburl + '/' + post_poolid + '/api/removeAll'
-    x = client.get(url, auth=auth)
+    url = aburl + '/' + post_poolid + '/api/removeAll' + \
+        '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
 
     files = get_files_in_local_dir(post_poolid, local_pools_dir)
     assert len(files) == 0, 'Wipe pool failed: ' + o['msg']
 
-    url = aburl + '/' + post_poolid + '/api/isEmpty'
-    x = client.get(url, auth=auth)
+    url = aburl + '/' + post_poolid + '/api/isEmpty' + \
+        '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     assert o['result'] == True
@@ -576,23 +629,24 @@ def test_CRUD_product(local_pools_dir, server, userpass, client, thepool):
     # ========
     logger.info('unregister a pool on the server')
     url = aburl + '/' + post_poolid
-    x = client.delete(url, auth=auth)
+    x = safe_client(client.delete, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
 
     # this should fail as pool is unregistered on the server
-    url = aburl + '/' + post_poolid + '/api/isEmpty'
-    x = client.get(url, auth=auth)
+    url = aburl + '/' + post_poolid + '/api/isEmpty' + \
+        '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=x.status_code, failed_case=True)
 
 
-def test_data_path(server, userpass, client):
+def test_data_path(server, auth, client):
 
     aburl, headers = server
-    auth = HTTPBasicAuth(*userpass)
-    # empty_pool(post_poolid,aburl,userpass)
-    pstore = ProductStorage(test_poolid, aburl + '/'+test_poolid)
+    # empty_pool(post_poolid,aburl,auth)
+    pstore = ProductStorage(test_poolid, aburl + '/' +
+                            test_poolid, auth=auth, client=client)
     pool = PoolManager.getPool(test_poolid)
 
     url0 = aburl + '/' + test_poolid + '/'
@@ -602,7 +656,7 @@ def test_data_path(server, userpass, client):
     data = serialize(p)
     # print(len(data))
     url1 = url0
-    x = client.post(url1, auth=auth, data=data)
+    x = safe_client(client.post, url1, auth=auth, data=data)
     o, code = getPayload(x)
     check_response(o, code=code)
     urn = o['result']
@@ -617,7 +671,7 @@ def test_data_path(server, userpass, client):
     pcls = urn.split(':')[2].replace(':', '/')
     urlapi = url0 + pcls
     # 'http://127.0.0.1:5000/fdi/v0.10/fdi_serv.test_httppool/fdi.dataset.product.Product'
-    x = client.get(urlapi, auth=auth)
+    x = safe_client(client.get, urlapi, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     c = o['result']
@@ -629,7 +683,7 @@ def test_data_path(server, userpass, client):
     # make url w/  urn
     # url2       = 'http://127.0.0.1:5000/fdi/v0.10/fdi_serv.test_httppool/fdi.dataset.product.Product/0/measurements/Time_Energy_Pos/Energy/data'
     url2 = aburl + urn.replace(':', '/')[3:] + '/' + pth
-    x = client.get(url2, auth=auth)
+    x = safe_client(client.get, url2, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     c = o['result']
@@ -641,7 +695,7 @@ def test_data_path(server, userpass, client):
     urlp = url0 + pt
     # http://127.0.0.1:5000/fdi/v0.10/fdi_serv.test_httppool/fdi.dataset.product.Product/0/measurements/Time_Energy_Pos/Energy/data
     url3 = urlp + '/' + pth
-    x = client.get(url3, auth=auth)
+    x = safe_client(client.get, url3, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     c2 = o['result']
@@ -656,7 +710,7 @@ def test_data_path(server, userpass, client):
             "measurements/calibration/unit",
     ]:
         url = urlp + '/' + pth
-        x = client.get(url, auth=auth)
+        x = safe_client(client.get, url, auth=auth)
         o, code = getPayload(x)
         check_response(o, code=code)
         c = o['result']
@@ -666,7 +720,7 @@ def test_data_path(server, userpass, client):
 
     # pt = fdi.dataset.product.Product/0
     url = url0 + pt + '/'
-    x = client.get(url, auth=auth)
+    x = safe_client(client.get, url, auth=auth)
     o, code = getPayload(x)
     check_response(o, code=code)
     c = o['result']
@@ -676,17 +730,17 @@ def test_data_path(server, userpass, client):
 
     # 'http://127.0.0.1:5000/fdi/v0.10/fdi_serv.test_httppool/string/fdi.dataset.product.Product/0'
     url = url0 + pt + '/toString'
-    x = client.get(url, auth=auth)
+    x = safe_client(client.get, url, auth=auth)
     assert x.headers['Content-Type'] == 'text/plain'
     o, c = getPayload(x)
-    assert 'UNKNOWN' in o
+    assert ('UNKNOWN' if isinstance(o, str) else b'UNKNOWN') in o
 
 
 def test_get_pools(local_pools_dir, server, client):
 
     aburl, headers = server
-    url = aburl + '/'+'pools'
-    x = client.get(url, headers=headers)
+    url = aburl + '/'+'pools' + '/'  # trailing / is needed by mock server
+    x = safe_client(client.get, url, headers=headers)
     o, code = getPayload(x)
     check_response(o, code=code)
     c = o['result']
@@ -694,6 +748,134 @@ def test_get_pools(local_pools_dir, server, client):
     assert set(c) == set(get_files_in_local_dir('', local_pools_dir))
 
 
+def lock_pool2(poolid, sec, local_pools_dir):
+    ''' Lock a pool from reading and return a fake response
+    '''
+    ppath = os.path.join(local_pools_dir, poolid)
+    # lock to prevent writing
+    lock = Lock_Path_Base + ppath.replace('/', '_') + '.read'
+    xx = makeLockpath(poolid, 'r')
+    logger.info('Keeping files %s locked for %f sec' % (lock, sec))
+    t0 = str(time.time())
+    with filelock.FileLock(lock):
+        time.sleep(sec)
+    fakeres = '{"result": "DONE", "msg": "This is a fake responses", "time": ' + t0 + '}'
+    return deserialize(fakeres)
+
+
+def read_product2(poolid, server, auth, client):
+
+    aburl, headers = server
+
+    # trying to read
+    if 1:
+        prodpath = '/'+prodt+'/0'
+        # trailing / is needed by mock server
+        url = aburl + '/' + poolid + prodpath  # + '/'
+    else:
+        hkpath = '/hk/dTypes'
+        url = aburl + '/' + poolid + hkpath + '/'  # trailing / is needed by mock server
+    logger.debug('Reading a locked file http...%s' % (url[-5:]))
+    x = safe_client(client.get, url, auth=auth)
+    r = x.text if type(x) == requests.models.Response else x.data
+    logger.info("read %f http...%s  %s" % (time.time(), url[-5:], lls(r, 26)))
+    o, code = getPayload(x)
+    o['msg'] = r
+    return o
+
+
+def pararun(n, sleep2, *args):
+    res, futs = {}, {}
+    with concurrent.futures.ThreadPoolExecutor(max_workers=n) as executor:
+        for i in range(n):
+            time.sleep(sleep2)
+            tt = time.time()
+            readfut = executor.submit(*args)
+            futs[tt] = readfut
+        for creadfut in concurrent.futures.as_completed(futs.values()):
+            # __import__('pdb').set_trace()
+
+            readres = creadfut.result()
+            tt = time.time()
+            # logger.info('PPPPPP %f %s FFFF %s' % (readres['time'], readres['result'].description,st$
+            #readres['msg'] = tt
+            res[tt] = readres
+    return res
+
+# https://github.com/pallets/flask/issues/4375#issuecomment-990102774
+
+
+def Xest_lock_file2(server, userpass, local_pools_dir, client):
+    ''' Test if a pool is locked, others can not manipulate this pool anymore before it's released
+    '''
+    logger.info('%f Test read a locked file, it will return DONE.' %
+                time.time())
+    aburl, headers = server
+    poolid = test_poolid
+    # init server
+    populate_pool(poolid, aburl, userpass, client)
+    # hkpath = '/hk/dTypes'
+    # url = aburl + '/' + poolid + hkpath
+    # x = safe_client(client.get, url, auth=make_auth(userpass))
+    locreadres, futs = {}, {}
+    sleep1, sleep2 = 1.0, 0.12
+
+    with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
+        futs[time.time()] = executor.submit(
+            lock_pool2, poolid, sleep1, local_pools_dir)
+        logger.info('%f lock submitted. %s. start reading ...' %
+                    (time.time(), str(futs)))
+
+        futs[time.time()] = executor.submit(pararun, 10, sleep2,
+                                            read_product2, poolid,
+                                            server, userpass, client)
+        for locreadfut in concurrent.futures.as_completed(futs.values()):
+            locreadres[time.time()] = locreadfut.result()
+
+    logger.debug(list(locreadres))
+    # __import__('pdb').set_trace()
+    for t, r in locreadres.items():
+        if r.get('msg', '') == 'This is a fake responses':
+            # lock process
+            result = r
+            tl = t
+        else:
+            res = r
+            tr = t
+    logger.debug('res ' + str(res)[:99])
+
+    #assert result['result'] == 'DONE'
+    v = list(res.values())
+    assert issubclass(v[0].__class__, Mapping)
+    assert 'result' in v[-1]
+    assert v[0]['result'].description == 'desc 0'
+    assert not issubclass(v[-1]['result'].__class__, str)
+    print(len(res), v[-1]['time']-result['time'])
+    assert v[-1]['time']-result['time'] > sleep1
+    assert len(v) > int(sleep1/sleep2) - 1
+
+    """try:
+        loop = asyncio.get_running_loop()
+        # 1. Run in the default loop's executor:
+        #__import__('pdb').set_trace()
+        result = await loop.run_in_executor(None, lock_pool2, poolid, sleep1, local_pools_dir)
+        for i in range(10):
+                readres = read_product2(poolid, server, userpass, client)
+                logger.info('PPPPPP %d %d' % (i,i))
+                res.append(readres)
+                if 'FAIL' in readres['result']:
+                     time.sleep(sleep2)
+                else:
+                     break
+        # task=functool.partial(read_product,poolid, server, userpass, client))
+        # print('custom thread pool', result, res)
+    except Exception as e:
+        logger.error('unable to start thread ' + str(e) + trbk(e))
+        raise
+    """
+
+
+"""
 async def lock_pool(poolid, sec, local_pools_dir):
     ''' Lock a pool from reading and return a fake response
     '''
@@ -709,26 +891,33 @@ async def lock_pool(poolid, sec, local_pools_dir):
     return deserialize(fakeres)
 
 
-async def read_product(poolid, server, userpass):
+async def read_product(poolid, server, userpass, client):
 
     aburl, headers = server
+    if headers['server_type'] == 'live':
+        auth = aiohttp.BasicAuth(*userpass)
+    else:
+        auth = make_auth(userpass)
     # trying to read
     if 1:
         prodpath = '/'+prodt+'/0'
-        url = aburl + '/' + poolid + prodpath
+        # trailing / is needed by mock server
+        url = aburl + '/' + poolid + prodpath + '/'
     else:
         hkpath = '/hk/dTypes'
-        url = aburl + '/' + poolid + hkpath
+        url = aburl + '/' + poolid + hkpath + '/'  # trailing / is needed by mock server
     logger.debug('Reading a locked file '+url)
     async with aiohttp.ClientSession() as session:
-        async with session.get(url, auth=aiohttp.BasicAuth(*userpass)) as res:
+        async with session.get(url, auth=auth) as res:
             x = await res.text()
             o = deserialize(x)
     logger.debug("@@@@@@@locked file read: " + lls(x, 200))
     return o
 
+# https://github.com/pallets/flask/issues/4375#issuecomment-990102774
 
-def test_lock_file(server, userpass, local_pools_dir, client):
+
+def XXtest_lock_file(server, userpass, local_pools_dir, client):
     ''' Test if a pool is locked, others can not manipulate this pool anymore before it's released
     '''
     logger.info('Test read a locked file, it will return FAILED')
@@ -738,12 +927,12 @@ def test_lock_file(server, userpass, local_pools_dir, client):
     populate_pool(poolid, aburl, userpass, client)
     # hkpath = '/hk/dTypes'
     # url = aburl + '/' + poolid + hkpath
-    # x = client.get(url, auth=HTTPBasicAuth(*userpass))
+    # x = safe_client(client.get, url, auth=make_auth(userpass))
 
     try:
         loop = asyncio.get_event_loop()
         tasks = [asyncio.ensure_future(
-            lock_pool(poolid, 2, local_pools_dir)), asyncio.ensure_future(read_product(poolid, server, userpass))]
+            lock_pool(poolid, 2, local_pools_dir)), asyncio.ensure_future(read_product(poolid, server, userpass, client))]
         taskres = loop.run_until_complete(asyncio.wait(tasks))
         loop.close()
     except Exception as e:
@@ -751,11 +940,13 @@ def test_lock_file(server, userpass, local_pools_dir, client):
         raise
     res = [f.result() for f in [x for x in taskres][0]]
     logger.debug('res ' + lls(res[0], 200) + '************' + lls(res[1], 200))
-    if issubclass(res[0].__class__, Mapping) and 'result' in res[0] and issubclass(res[0]['result'].__class__, str):
+    if issubclass(res[0].__class__, Mapping) and 'result' in res[0] \
+       and issubclass(res[0]['result'].__class__, str):
         r1, r2 = res[0], res[1]
     else:
         r2, r1 = res[0], res[1]
     check_response(r1, code=400, failed_case=True)
+    """
 
 
 def test_read_non_exists_pool(server, userpass, client):
@@ -766,7 +957,7 @@ def test_read_non_exists_pool(server, userpass, client):
     wrong_poolid = 'nonexist_' + __name__.replace('.', '_')
     prodpath = '/' + prodt + '/0'
     url = aburl + '/' + wrong_poolid + prodpath
-    x = client.get(url, auth=HTTPBasicAuth(*userpass))
+    x = safe_client(client.get, url, auth=make_auth(userpass))
     o, code = getPayload(x)
     check_response(o, code=400, failed_case=True)
 
@@ -801,11 +992,11 @@ def xtest_webapi_jsonPath(server, userpass, client):
 
         def jsonPath(self, *args, **kwds):
             urlargs = serialize_args(*args, not_quoted=False, **kwds)
-            urlargs = serialize_args(
-                urlargs, not_quoted=False)
+            urlargs = serialize_args(urlargs, not_quoted=False)
             url = self.purl + '/jsonPath__' + urlargs
             nonlocal userpass
-            x = client.get(url, auth=HTTPBasicAuth(*userpass))
+            auth = make_auth(userpass)
+            x = safe_client(client.get, url, auth=auth)
             o, code = getPayload(x)
             check_response(o, code=code)
             return o['result']
@@ -815,7 +1006,7 @@ def xtest_webapi_jsonPath(server, userpass, client):
 
 if __name__ == '__main__':
     now = time.time()
-    node, verbose = opt(pc['node'])
+    node, verbose = opt(pc)
     if verbose:
         logger.setLevel(logging.DEBUG)
     else:
