@@ -1,23 +1,5 @@
 # -*- coding: utf-8 -*-
 
-import os.path as op
-import datetime
-import fractions
-import decimal
-import traceback
-from pprint import pprint
-import copy
-import json
-import sys
-import threading
-import functools
-import time
-import locale
-import array
-from math import sqrt
-from datetime import timezone
-import pytest
-
 from fdi.dataset.annotatable import Annotatable
 from fdi.dataset.copyable import Copyable
 from fdi.dataset.odict import ODict
@@ -53,12 +35,32 @@ from fdi.dataset.product import Product
 from fdi.dataset.browseproduct import BrowseProduct
 from fdi.dataset.readonlydict import ReadOnlyDict
 from fdi.dataset.unstructureddataset import UnstructuredDataset
-from fdi.dataset.testproducts import SP, get_demo_product
+from fdi.dataset.testproducts import get_demo_product, DemoProduct
 from fdi.utils.checkjson import checkjson
 from fdi.utils.loadfiles import loadMedia
 from fdi.utils.ydump import ydump
 
 from jsonpath_ng.parser import JsonPathParserError
+import networkx as nx
+import pydot
+
+import os.path as op
+import datetime
+import fractions
+import decimal
+import traceback
+from pprint import pprint
+import copy
+import json
+import sys
+import threading
+import functools
+import time
+import locale
+import array
+from math import sqrt
+from datetime import timezone
+import pytest
 
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
@@ -68,8 +70,6 @@ else:
 
 locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
 
-Classes.updateMapping()
-
 # make format output in /tmp/outputs.py
 mk_outputs = 0
 output_write = 'tests/outputs.py'
@@ -88,7 +88,7 @@ else:
 
     # This is to be able to test w/ or w/o installing the package
     # https://docs.python-guide.org/writing/structure/
-    #from pycontext import fdi
+    # from pycontext import fdi
     import fdi
     if not mk_outputs:
         from outputs import nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
@@ -184,15 +184,23 @@ def test_serialization():
     v = None
     checkjson(v)
     v = Ellipsis
+    checkjson(v)
+    v = datetime.datetime(2022, 2, 3, 4, 5, 6, 765432,
+                          tzinfo=datetime.timezone.utc)
+
     checkjson(v)
     v = b'\xde\xad\xbe\xef'
     # checkjson(v)
     v = array.array('d', [1.2, 42])
     checkjson(v)
+    v = array.array('H', b'123\xef')
+    checkjson(v)
+    v = bytearray(b'1234\xef')
+    checkjson(v)
     v = [1.2, 'ww']
     checkjson(v)
     v = Product
-    checkjson(v)
+    checkjson(v, 1)
     # escape
     assert deserialize('{ "_STID":"set"}').__class__ == set
     assert deserialize('{ "_STID":"0set"}').__class__ == dict
@@ -428,7 +436,7 @@ def test_EventType():
 test123 = 0
 
 
-@pytest.fixture()
+@ pytest.fixture()
 def mocksndrlsnr():
     global test123
 
@@ -502,7 +510,7 @@ def test_MqttRelay_mqtt(mocksndrlsnr):
                           port=None, username=None, passwd=None,
                           client_id='foo', callback=None, qos=1,
                           userdata=None, clean_session=None,)
-    # mqhost = v.mq._host
+    # mq_host = v.mq._host
     mf = MockFileWatcher()
     mf.addListener(v)
     w = MqttRelaySender(topics="test.mq.bounce2", host=None,
@@ -534,6 +542,9 @@ def test_MqttRelay_mqtt(mocksndrlsnr):
         print(w.last_msg)
         assert test123 == "['foo'] changed."
         assert w.last_msg == ['foo']
+    # to avoid this client from messing up later tests
+    v.mq.disconnect()
+    w.mq.disconnect()
 
 
 def test_datatypes():
@@ -629,8 +640,8 @@ def test_Parameter_init():
     assert v.valid is None
     # incompatible type
     a2 = DataWrapper()
-    with pytest.raises(TypeError):
-        v = Parameter(a2)
+    v = Parameter(a2)
+    assert a2 == v.value
     # also only one positional argument, but with a keyword arg
     a1 = 'a test parameter'
     a2 = FineTime1(8765)
@@ -740,6 +751,25 @@ def test_Parameter_valid():
     assert v.validate('') == ('', 'empty')
     assert v.toString(alist=True)[0] == 'Invalid (Right)'
 
+    if 0:
+        # vector. all element must pass the same test to gether
+        a1 = 'a test parameter with vector in value'
+        a2 = [1, 2, 3, 4]
+        a3 = 'list'
+        a4 = [0, 0, 0, 0]
+        a5 = {(0, 4): 'ok'}
+        v = Parameter(description=a1, value=a2, typ_=a3, default=a4, valid=a5)
+        assert v.isValid() == True
+        v.valid = {(0, 3): 'ok'}
+        assert v.isValid() == False
+        v.valid = {(0, 3): 'ok', 4: 'also'}
+        assert v.isValid() == False
+        v = Parameter(value=['b', 'aa', 'd'], typ_='list',
+                      valid={('a', 'c'): 'ok'})
+        assert v.isValid() == False
+        del v.value[2]
+        assert v.isValid()
+
 
 def test_Parameter_features():
     # test equivalence of v.setXxxx(a) and v.xxx = a
@@ -762,7 +792,7 @@ def test_Parameter_features():
         with pytest.raises(TypeError):
             v = Parameter(a2, a1, a4)
     else:  # smart
-        # not recommendedxs
+        # not recommended
         v = Parameter(a2, a1, a4)
         # v = Parameter(typ_=a4)
         # v.value = a2
@@ -821,7 +851,7 @@ def test_Parameter_features():
     # CHANNGED parameter equality behavior
     assert v.equals(v1)
     assert v.__eq__(v1)
-    assert v.__eq__ == v1.__eq__
+    #assert v.__eq__ == v1.__eq__
     assert v == v1
     v1.value = -4
 
@@ -995,37 +1025,6 @@ def test_NumericParameter():
     checkgeneral(v)
 
 
-def test_NumericParameter():
-    v = NumericParameter()
-    assert v.description == 'UNKNOWN'
-    assert v.value is None
-    assert v.unit is None
-    assert v.type == ''
-    assert v.default is None
-    assert v.valid is None
-    assert v.typecode is None
-    h = v.hash()
-
-    a1 = 'a test NumericParameter'
-    a2 = 100.234
-    a3 = 'second'
-    a4 = 'float'
-    a5 = 0
-    a6 = ''
-    a7 = 'f'
-    v = NumericParameter(description=a1, value=a2, unit=a3,
-                         typ_=a4, default=a5, valid=a6, typecode=a7)
-    assert v.description == a1
-    assert v.value == a2
-    assert v.unit == a3
-    assert v.type == a4
-    assert v.default == a5
-    assert v.valid is None
-    assert v.typecode == a7
-
-    checkjson(v)
-
-
 def test_BooleanParameter():
     v = BooleanParameter()
     assert v.description == 'UNKNOWN'
@@ -1067,6 +1066,7 @@ def test_DateParameter():
     a5 = 9
     a6 = ''
     a7 = '%f'
+    # value can take integer as TAI.
     v = DateParameter(description=a1, value=a2,
                       default=a5, valid=a6)
     assert v.description == a1
@@ -1100,8 +1100,10 @@ def test_DateParameter():
     assert v.value.isoutc() == '2019-02-19T01:02:03.457000'
     assert str(
         v) == 'DateParameter(finetime: 2019\n1929229360457000 <>, "UNKNOWN", default= None, valid= None tcode=%Y-%m-%dT%H:%M:%S.%f)'
-    with pytest.raises(TypeError):
-        DateParameter(3.3)
+    # 1972-1-1 emit warning
+    assert DateParameter(63072000.0).value.isoutc(
+    ) == '1972-01-01T00:00:00.000000'
+    DateParameter(63043200.0 - 0.0000001)
     vv = copy.copy(v)
     checkjson(v)
 
@@ -1215,7 +1217,7 @@ def test_MetaData():
 
     # toString wth extra attributes
     ts = 'MetaData.toString with extra'
-    v = Product(ts)
+    v = DemoProduct(ts)
     v = v.meta
     ts += '\n'
     ts += v.toString(extra=True)
@@ -1406,11 +1408,16 @@ def do_ArrayDataset_init(atype):
     assert v.typecode == 'UNKNOWN'
     assert v.meta['description'].value == 'UNKNOWN'
     # from DRM
-    a1 = atype([1, 4.4, 5.4E3])      # an array of data
+    if issubclass(atype([]).__class__, (bytes, bytearray)):
+        a1 = atype([1, 44, 0xff])      # an array of data
+        a4 = 'integer'              # type
+        a6 = 'H'                  # typecode
+    else:
+        a1 = atype([1, 4.4, 5.4E3])      # an array of data
+        a4 = 'float'              # type
+        a6 = 'f'                  # typecode
     a2 = 'ev'                 # unit
     a3 = 'three energy vals'  # description
-    a4 = 'float'              # type
-    a6 = 'f'                  # typecode
     a7 = (8, 9)
 
     v = ArrayDataset(data=a1, unit=a2, description=a3,
@@ -1419,18 +1426,18 @@ def do_ArrayDataset_init(atype):
     assert v.unit == a2
     assert v.description == a3
     assert v.typecode == a6
-    assert v.shape == (len(a1),)
+    assert v.shape == [len(a1)]
 
     ashape = [[[1], [2]], [[3], [4]], [[5], [6]]]
     v2 = ArrayDataset()
     # setting data also sets shape
     v2.data = ashape
     # setting dataa also v.updateShape()
-    assert v2.shape == (3, 2, 1)
+    assert v2.shape == [3, 2, 1]
     v2.pop()
-    assert v2.shape == (2, 2, 1)
+    assert v2.shape == [2, 2, 1]
     v2 = ArrayDataset(data=['poi'])
-    assert v2.shape == (1,)
+    assert v2.shape == [1]
 
     # omit the parameter names when instantiating, the orders are data, unit, description
     v2 = ArrayDataset(a1)
@@ -1462,9 +1469,12 @@ def check_MDP(x):
 def do_ArrayDataset_func(atype):
 
     # test data and unit
-    a1 = atype([1, 4.4, 5.4E3])      # an array of data
+    if issubclass(atype([]).__class__, (bytes, bytearray)):
+        a1 = atype([1, 44, 0xde])      # an array of data
+    else:
+        a1 = atype([1, 4.4, 5.4E3])      # an array of data
     a2 = 'ev'                 # unit
-    a3 = atype([34, 9999])
+    a3 = atype([34, 99])
     a4 = 'm'
     a5 = 'new description'
     v = ArrayDataset(data=a1)
@@ -1474,10 +1484,13 @@ def do_ArrayDataset_func(atype):
     v.unit = a4
     assert v.data == a3
     assert v.unit == a4
-    assert v.data[1] == 9999
+    assert v.data[1] == 99
 
     # test equality
-    a1 = atype([1, 4.4, 5.4E3])
+    if issubclass(atype([]).__class__, (bytes, bytearray)):
+        a1 = atype([1, 44, 0xde])      # an array of data
+    else:
+        a1 = atype([1, 4.4, 5.4E3])      # an array of data
     a2 = 'ev'
     a3 = 'three energy vals'
     a6 = None
@@ -1488,7 +1501,10 @@ def do_ArrayDataset_func(atype):
     v.meta[a4] = a5
 
     # b[1-5] and v1 have the  same contents as a[1-5] and v
-    b1 = atype([1, 4.4, 5.4E3])
+    if issubclass(atype([]).__class__, (bytes, bytearray)):
+        b1 = atype([1, 44, 0xde])      # an array of data
+    else:
+        b1 = atype([1, 4.4, 5.4E3])      # an array of data
     b2 = ''.join(a2)
     b3 = ''.join(a3)
     v1 = ArrayDataset(data=b1, unit=b2, description=b3)
@@ -1540,27 +1556,38 @@ def do_ArrayDataset_func(atype):
     assert y[0] == 3
     # removal
     r0 = x[2]
-    x.remove(r0)
-    assert x[4] == 6  # x=[1,2,4,5,6]
-    r0 = x[0]
-    assert r0 == x.pop(0)
-    assert x[0] == 2
+    if issubclass(atype([]).__class__, (bytes)):
+        with pytest.raises(AttributeError):
+            x.remove(r0)
+        # iteration
+        i = []
+        for m in v:
+            i.append(m)
+    else:
+        x.remove(r0)
+        assert x[4] == 6  # x=[1,2,4,5,6]
+        r0 = x[0]
+        assert r0 == x.pop(0)
+        assert x[0] == 2
 
-    # iteration
-    i = atype([])
-    for m in v:
-        i.append(m)
-    assert i == a1
+        # iteration
+        i = atype([])
+        for m in v:
+            i.append(m)
+    assert list(i) == list(a1)
 
     try:
         d = atype([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
     except TypeError:
-        d = atype([2.3, 4e3, -9.9])
+        if issubclass(atype([]).__class__, (bytes, bytearray)):
+            d = atype(b'1s\xde')
+        else:
+            d = atype([2.3, 4e3, -9.9])
         x = ArrayDataset(data=d)
         for n, p in standardtestmeta().items():
             x.meta[n] = p
         ts = x.toString()
-        # print(atype, ts)
+        #print('***', atype, ts)
 
         checkjson(x)
         checkgeneral(x)
@@ -1603,7 +1630,7 @@ def do_ArrayDataset_func(atype):
     if mk_outputs:
         print(ts[i:])
     else:
-        assert ts[i:-132] == nds2
+        assert ts[i:-131] == nds2
     ts += '\n\nlevel 1\n'
     ts += x.toString(1)
     ts += '\n\nlevel 2, repr\n'
@@ -1636,6 +1663,26 @@ def test_ArrayDataset_func():
     do_ArrayDataset_func(atype)
 
 
+def test_bytes_init():
+    atype = bytes
+    do_ArrayDataset_init(atype)
+
+
+def test_bytes_func():
+    atype = bytes
+    do_ArrayDataset_func(atype)
+
+
+def test_bytearray_init():
+    atype = bytearray
+    do_ArrayDataset_init(atype)
+
+
+def test_bytearray_func():
+    atype = bytearray
+    do_ArrayDataset_func(atype)
+
+
 def test_ArrayDataset_array_init():
     atype = functools.partial(array.array, 'f')
     do_ArrayDataset_init(atype)
@@ -1691,13 +1738,13 @@ def test_TableDataset_init():
         assert v.getColumnName(0) == 'col1'
         t = a1['col2'].data[1]  # 43.2
         assert v.getValueAt(rowIndex=1, columnIndex=1) == t
-        assert v.shape == (2, 3)
+        assert v.shape == [2, 3]
 
     # 2: add columns one by one
     v2 = TableDataset()
     v2['col1'] = Column(data=copy.deepcopy(d1), unit='eV')
     v2['col2'] = Column(data=copy.deepcopy(d2), unit='cnt')
-    assert v2.shape == (2, 3)
+    assert v2.shape == [2, 3]
 
     # print('DDD ', deepcmp(v, v2, verbose=0))
 
@@ -1860,7 +1907,7 @@ def test_TableDataset_func_column():
     assert w3.list == []
 
 
-def test_TableDataset_func():
+def test_TableDataset_func0():
     u = TableDataset()
     c1 = Column([1, 4], 'sec')
     u.addColumn('col3', c1)
@@ -1948,29 +1995,37 @@ def test_TableDataset_func():
     ts += 'an empty level 2: \n'
     ts += TableDataset().toString(level=2)
     ts += '\n\n'
-    for n, c in {'group1.val': a10['col1'], 'group1.err': a10['col2'],
-                 'no-group.val': a10['col1'],
-                 'group2.val': a10['col1'], 'group2.err': a10['col2'],
-                 'group2.seq': a10['col2'],
-                 'group2.wgt': Column(data=['', 0.32, -9876543210], unit='g'),
-                 }.items():
-        v.addColumn(n, c)
+    for i, n in enumerate([
+        'no-group1.val',
+        'group1.val1',
+        'group1.val2',
+        '.val',
+        'grou.p2.val1',
+        'grou.p2.',
+        'grou.p2.val3',
+        'group3_val1',
+        'group3.val2',
+        'group3/val3',
+    ]):
+        v.addColumn(n, Column(data=[n, 123], unit='g'+str(i)),)
     from fdi.utils.tree import tree
-    print('\n'.join(tree(v, style='ascii')))
+    #print('\n'.join(tree(v, style='ascii')))
     from fdi.utils.jsonpath import jsonPath, flatten_compact
-    print(list(str(x.full_path)
-               for x in jsonPath(v, '$..*', val='context')))
-    print('\n'.join(flatten_compact([v]).keys()))
+    # print(list(str(x.full_path)
+    #           for x in jsonPath(v, '$..*', val='context')))
+    # print('\n'.join(flatten_compact([v]).keys()))
 
     ts += 'grouped column names'
-    ts += v.string(0, 'grid', 'rst', 'simple')
-    ts += v.string(0, 'grid', 'rst', 'rst')
-    ts += v.string(0, 'grid', 'rst', 'grid')
-    ts += v.string(0, 'grid', 'rst', 'fancy_grid')
-    ts += v.string(0, 'grid', 'rst', 'plain')
-    ts += v.string(0, 'grid', 'rst', 'orgtbl')
-    ts += v.string(0, 'grid', 'rst', 'psql')
-    ts += v.string(0, 'unsafehtml', 'unsafehtml', 'unsafehtml')
+    ts += v.string(0, False, None, 'grid', 'rst', 'simple')
+    ts += v.string(0, False, None, 'simple_grid', 'rst', 'rst')
+    ts += v.string(0, False, None, 'rounded_grid', 'rst', 'outline')
+    ts += v.string(0, False, None, 'grid', 'rst', 'rounded_outline')
+    ts += v.string(0, False, None, 'grid', 'rst', 'fancy_outline')
+    ts += v.string(0, False, None, 'fancy_grid', 'rst', 'fancy_grid')
+    ts += v.string(0, False, None, 'grid', 'rst', 'plain')
+    ts += v.string(0, False, None, 'grid', 'rst', 'orgtbl')
+    ts += v.string(0, False, None, 'grid', 'rst', 'psql')
+    ts += v.string(0, False, None, 'unsafehtml', 'unsafehtml', 'unsafehtml')
     ts += '\n\n'
     if mk_outputs:
         print(ts)
@@ -2265,7 +2320,7 @@ def test_UnstrcturedDataset():
     checkgeneral(u)
 
 
-# https://goessner.net/articles/JsonPath/
+# https://goessner.articles/JsonPath/
 bookstore = """{ "store": {
     "book": [
       { "category": "reference",
@@ -2768,35 +2823,136 @@ def test_FineTimes_toString():
         assert ts == out_FineTime
 
 
-def test_History():
+def test_History(tmp_local_storage, tmp_prods):
     v = History()
-    with pytest.raises(TypeError):
+    with pytest.raises(AttributeError):
         v.add_input(args=[(2, 3)])
 
     v = History()
-    v.add_input(args=[1, 2.33, 'asd', True, [4, 5], array.array('f', [6, 7])],
-                keywords=dict(a='b', c=[11]),
+    dt = datetime.datetime(
+        2019, 2, 19, 1, 2, 3, 456789, tzinfo=timezone.utc)
+    v.add_input(args={'id': 1, 'width': 2.33, 'name': 'asd',
+                      'ok': True, 'speed': [4, 5],
+                      'scores': array.array('f', [6, 7]),
+                      'and': None, 'a': 'b', 'c': [11],
+                      'dt': dt
+                      },
                 info=dict(c='d')
                 )
-    args, kwds = v.get_args()
-    assert args[-1] == array.array('f', [6, 7])
-    assert kwds.rowCount == 2
+    bik = v.builtin_keys
+    len0 = len(bik)
+    args = v.get_args_info()
+    assert args['scores'] == array.array('f', [6, 7])
+    assert len(v.meta) == len0 + 10
+    assert isinstance(v.meta['dt'].value, datetime.datetime)
+
     assert v.meta['c'].value == 'd'
     assert 'e' not in v.meta
     # add more
-    v.add_input(args=[{9: 10}],
+    v.add_input(args={'more': {9: 10}},
                 keywords=dict(d=FineTime(123456)),
                 info=dict(e='f')
                 )
-    assert args[-1][9] == 10
-    assert v['kw_args'].rowCount == 3
+    assert v.get_args_info()['more'][9] == 10
+    assert len(v.meta) == len0 + 13
     assert v.meta['e'].value == 'f'
+    assert v.rowCount == 0
+    # add urn
+    ps = tmp_local_storage
+    p0_, p11_, p12_, p121, p122, p1221, p12211 = tmp_prods
+    p0 = copy.deepcopy(p11_)
+    p11 = copy.deepcopy(p11_)
+    p12 = copy.deepcopy(p12_)
+    urn11 = ps.save(p11).urn
+    urn12 = ps.save(p12).urn
+    v = p0.history
+    v.add_input(refs={'p1-1': urn11, 'p1-2': urn12})
+    assert v.rowCount == 2
+    th = v.getTaskHistory(use_name=False, verbose=0)
+    assert len(th.nodes) == 3
+    assert len(th.edges) == 2
+    assert len(list(th.pred['root'])) == 2
+    # urn ends with '2'
+    assert th[f'"{urn11}"'] == th[f'"{urn12}"'] == {'root': {}}
+    checkjson(v)
+    checkgeneral(v)
+
+    # build a bigger chain
+    urn12211 = ps.save(p12211).urn
+    p1221.history.add_input(refs={'p1-2-2-1-1': urn12211})
+    assert p1221.history.rowCount == 1
+    assert p12211.history.rowCount == 0
+
+    urn1221 = ps.save(p1221).urn
+    p122.history.add_input(refs={'p1-2-2-1': urn1221})
+    assert p121.history.rowCount == 0
+    assert p122.history.rowCount == 1
+
+    urn121 = ps.save(p121).urn
+    urn122 = ps.save(p122).urn
+    # get fresh p0, p11, p12
+    p11 = copy.deepcopy(p11_)
+    p12 = copy.deepcopy(p12_)
+    p11.description = p11.description + '_new'
+    p12.description = p12.description + '_new'
+    p12.history.add_input(refs={'p1-2-1': urn121, 'p1-2-2': urn122})
+    assert p11.history.rowCount == 0
+    assert p12.history.rowCount == 2
+
+    urn11new = ps.save(p11).urn
+    urn12new = ps.save(p12).urn
+    p0 = copy.deepcopy(p11_)
+    v = p0.history
+    v.add_input(refs={'p1-1': urn11new, 'p1-2': urn12new})
+    assert v.rowCount == 2
+    th = v.getTaskHistory(use_name=False, verbose=0)
+    assert len(th.nodes) == 7
+    assert len(th.adj) == 7
+    assert len(list(th.pred['root'])) == 2
+
+    h = p0.history.getTaskHistory(verbose=0)
+    if 1:
+        assert len(p11.history['name']) == 0
+        assert p12.history['name'][0] == 'p1-2-1'
+        assert p12.history['name'][1] == 'p1-2-2'
+        assert p122.history['name'][0] == 'p1-2-2-1'
+        assert p1221.history['name'][0] == 'p1-2-2-1-1'
+        assert nx.is_directed_acyclic_graph(h)
+        # __import__('pdb').set_trace()
+    assert len(h.adj) == 7
+    print(f'Graph {h}')
     # print(v.string())
     # print(kwds.string())
     # print(v.tree())
-    checkjson(v)
+    v = p12
+    checkjson(v, 0)
     checkgeneral(v)
 
+    #pos = nx.nx_agraph.graphviz_layout(h)
+
+    pdot = nx.drawing.nx_pydot.to_pydot(h)
+    print(pdot.to_string())
+    fmt = 'png'
+    fig = pdot.create(format=fmt)
+    # print(fig)
+    with open('/tmp/G.'+fmt, 'wb') as f:
+        f.write(fig)
+    pdot.write_png("/tmp/D.png")
+
+    if 0:
+        try:
+            import matplotlib.pyplot as plt
+        except ImportError:
+            pass
+        else:
+            plt.tight_layout()
+            nx.draw_networkx(g, arrows=True)
+            plt.savefig("/tmp/g.png", format="PNG")
+            plt.savefig("/tmp/g.svg", format="SVG")
+            # tell matplotlib you're done with the plot: https://stackoverflow.com/questions/741877/how-do-i-tell-matplotlib-that-i-am-done-with-a-plot
+            # plt.clf()
+            # plt.show()
+
 
 def test_BaseProduct():
     """ """
@@ -3000,28 +3156,6 @@ def test_BrowseProduct():
     assert v.getDefault().data.startswith(b'GIF')
 
 
-def test_SubProduct():
-
-    y = SP()
-
-    # register it in Classes.mapping so deserializer knows how to instanciate.
-    Classes.mapping.update({'SP': SP})
-
-    check_Product(SP)
-
-    from fdi.pal.context import MapContext
-
-    class SSP(SP, MapContext):
-        def __init__(self, **kwds):
-            super().__init__(**kwds)
-
-    x = SSP()
-    x.instrument = 'ff'
-    assert x.instrument == 'ff'
-    x.rr = 'r'
-    assert x.rr == 'r'
-
-
 def est_yaml2python():
     import pkg_resources
     v = {'a': 1, 'b': 'foo', 'c': 4.5, 'd': FineTime1(0), 'e': Vector((7, 8, 9))
