@@ -24,8 +24,11 @@ from fdi.dataset.testproducts import TP
 from fdi.utils.checkjson import checkjson
 from fdi.pns.fdi_requests import save_to_server, read_from_server, delete_from_server
 
+# from flask import request as
+import requests
 from requests.auth import HTTPBasicAuth
 from requests.exceptions import ConnectionError
+from urllib3.exceptions import NewConnectionError
 import itertools
 import random
 import timeit
@@ -37,7 +40,6 @@ import json
 import shutil
 import getpass
 import os
-import requests
 import weakref
 import gc
 from os import path as op
@@ -55,8 +57,6 @@ else:
 Test_Pool_Name = __name__.replace('.', '_')
 defaultpoolPath = '/tmp/fditest'
 
-Classes.updateMapping()
-
 if __name__ == '__main__' and __package__ == 'tests':
     # run by python -m tests.test_dataset
 
@@ -278,7 +278,7 @@ def cleanup(poolurl=None, poolname=None):
         try:
             p = PoolManager.getPool(pname, poolurl)
             p.removeAll()
-        except RuntimeError:
+        except (RuntimeError, ValueError):
             pass
 
         PoolManager.remove(pname)
@@ -301,6 +301,9 @@ def test_PoolManager():
     assert PoolManager.size() == 1
     assert defaultpoolName in PoolManager.getMap()
     # print('GlobalPoolList#: ' + str(id(pm.getMap())) + str(pm))
+    PoolManager.remove('not_exists', ignore_error=True)
+    with pytest.raises(KeyError):
+        PoolManager.remove('not_exists')
     PoolManager.removeAll()
     assert PoolManager.size() == 0
     assert weakref.getweakrefcount(pool) == 0
@@ -342,10 +345,11 @@ def test_PoolManager():
     assert pm.remove(defaultpoolName) == 0
 
     # http pool gets registered
-    with pytest.raises(ConnectionError):
+    with pytest.raises(NewConnectionError):
         ph = pm.getPool(poolurl='http://h.edu/foo')
     assert not PoolManager.isLoaded('foo')
-    assert PoolManager.remove('foo') == 1
+    with pytest.raises(KeyError):
+        assert PoolManager.remove('foo') == 1
 
 
 def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args, csdb=None, **kwds):
@@ -392,7 +396,7 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args, csdb
         count = int(x.json()['result'])
         assert count == expected_cnt
         # counting files. Append a '/'
-        cpath = poolname + '/' + 'count/' + prodname + '/'
+        cpath = poolname + '/' + 'counted/' + prodname + '/'
         api_baseurl = scheme + '://' + place + poolpath + '/'
         url = api_baseurl + cpath
         x = requests.get(url, auth=auth)
@@ -410,7 +414,7 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args, csdb
         assert count >= expected_cnt
 
         # count all with couting
-        cpath = poolname + '/' + 'count' + '/'
+        cpath = poolname + '/' + 'counted' + '/'
         api_baseurl = scheme + '://' + place + poolpath + '/'
         url = api_baseurl + cpath
         x = requests.get(url, auth=auth)
@@ -431,8 +435,8 @@ def checkdbcount(expected_cnt, poolurl, prodname, currentSN, usrpsw, *args, csdb
 
     elif scheme in ['csdb']:
         test_pool = PoolManager.getPool(poolname)
-        #a, b = csdb
-        #assert test_pool == csdb
+        # a, b = csdb
+        # assert test_pool == csdb
         pinfo = test_pool.getPoolInfo()
         count = test_pool.getCount(prodname)
         assert count == expected_cnt
@@ -541,6 +545,8 @@ def test_ProductStorage_init():
         p1 = ps.getPools()[0]
     # with poolurl the pool is constructed
     ps = ProductStorage(poolurl=defaultpoolurl)
+    # There is no product
+    assert ps.isEmpty()
     # get the pool object
     pspool = ps.getPool(defaultpoolname)
     assert len(pspool.getProductClasses()) == 0
@@ -561,23 +567,21 @@ def test_ProductStorage_init():
     assert ps2.getPools()[1] == newpoolname
 
     # multiple storages pointing to the same pool will get exception
-    try:
-        ps2 = ProductStorage()
-    except Exception as e:
-        pass
-    else:
-        pass  # assert False, 'exception expected'
+    # with pytest.raises(TypeError):
+    ps2 = ProductStorage()
 
 
-def getCurrSnCount(csdb, prodname):
-    pinfo = csdb if issubclass(csdb.__class__, dict) else \
-        csdb.getPoolInfo()
+def getCurrSnCount(csdb_c, prodname):
+    pinfo = csdb_c if issubclass(csdb_c.__class__, dict) else \
+        csdb_c.getPoolInfo()
+    init_sn, init_count = 0, 0
     for pool in pinfo.values():
-        for cl in pool['_classes']:
-            if cl['productTypeName'] == prodname:
-                init_sn = cl['currentSn']
-                init_count = len(cl['sn']) - 1
-                break
+        poolclass = pool['_classes']
+        if prodname in poolclass:
+            init_sn = poolclass[prodname]['currentSn']
+            # XXX was the issue xaused " - 1" fixed?
+            init_count = len(poolclass[prodname]['sn'])
+            break
     return init_sn, init_count, pinfo
 
 
@@ -597,19 +601,22 @@ def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
         csdb = PoolManager.getPool(thepoolname)
         init_sn, init_count, pinfo = getCurrSnCount(csdb, pcq)
         init_sn_m, init_count_m, _ = getCurrSnCount(pinfo, mcq)
-        print('1...', init_count, init_sn, init_count_m, init_sn_m)
+        logger.debug('1...', init_count, init_sn, init_count_m, init_sn_m)
 
     # save
     ref = ps.save(x)
     s0, s1 = tuple(ref.urn.rsplit(':', 1))
     assert s0 == 'urn:' + thepoolname + ':' + pcq
-    #ref_m = ps.save(y)
+    # not saving thie one
+    # ref_m = ps.save(y)
     init_count, init_count_m = 0, 0
-    is_csdb = thepoolurl.startswith('csdb://')
     if is_csdb:
+        __import__("pdb").set_trace()
+
         currentsn2, count2, pinfo2 = getCurrSnCount(csdb, pcq)
-        currentsn_m2, count_m2, _ = getCurrSnCount(pinfo2, mcq)
-        print('2...', count2, currentsn2, count_m2, currentsn_m2)
+        # m is unchanged
+        currentsn_m2, count_m2, pinfo_m2 = getCurrSnCount(pinfo2, mcq)
+        logger.debug('2...', count2, currentsn2, count_m2, currentsn_m2)
 
         assert count2 == init_count + 1
         assert currentsn2 == init_sn + 1
@@ -628,7 +635,7 @@ def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
     # ps has 1 prod
     assert init_sn == int(s1)
     checkdbcount(init_count+1, thepoolurl, pcq, init_sn + 0, *args)
-    #checkdbcount(init_count_m+0, thepoolurl, mcq, init_sn_m + 0, *args)
+    # checkdbcount(init_count_m+0, thepoolurl, mcq, init_sn_m + 0, *args)
 
     # save more
     # one by one
@@ -692,8 +699,10 @@ def check_prodStorage_func_for_pool(thepoolname, thepoolurl, *args):
                  pcq, init_sn+n-1, *args)
     checkdbcount(init_count_m+1, thepoolurl, mcq, init_sn_m+0, *args)
 
+    # check isEmpty
+
     # report lru_cache info
-    print('****** %s ' % str(type(pspool)), pspool.getCacheInfo())
+    print('***HTTPClient Pool cache*** %s ' % str(pspool.getCacheInfo()))
 
     # clean up a pool
     ps.wipePool()
@@ -728,11 +737,10 @@ def test_ProdStorage_func_http(server, userpass):
     cleanup(thepoolurl, thepoolname)
     # First test registering with local pstor will also register on server
     pool = HttpClientPool(poolurl=thepoolurl)
-    # not exist
-    with pytest.raises(RuntimeError):
-        assert pool.isEmpty()
+
     # register
     pstore = ProductStorage(pool=pool)
+
     assert pool.isEmpty()
     pstore.unregister(thepoolname)
     # not exist
@@ -741,19 +749,18 @@ def test_ProdStorage_func_http(server, userpass):
     check_prodStorage_func_for_pool(thepoolname, thepoolurl, userpass)
 
 
-def test_ProdStorage_func_server(local_pools_dir):
+def test_ProdStorage_func_server():
     # httppool , the http server-side pool
     thepoolname = 'server'+Test_Pool_Name
-    thepoolurl = 'server://' + local_pools_dir + '/' + thepoolname
+    thepoolurl = 'server://' + '/tmp/fditest' + '/' + thepoolname
 
     cleanup(thepoolurl, thepoolname)
     check_prodStorage_func_for_pool(thepoolname, thepoolurl, None)
 
 
 def test_ProdStorage_func_csdb(csdb):
-    test_pool, url = csdb  # csdb:///csdb_test_pool
-    thepoolname = 'csdb_test_pool'
-    thepoolurl = 'csdb:///' + thepoolname
+    thepool, thepoolurl, pstore = csdb  # csdb:///csdb_test_pool
+    thepoolname = thepool.poolname
 
     cleanup(thepoolurl, thepoolname)
     check_prodStorage_func_for_pool(thepoolname, thepoolurl, None)
@@ -790,9 +797,9 @@ def test_LocalPool():
     ps2 = ProductStorage(pool=cpn, poolurl=cpu)
     # two ProdStorage instances have the same DB
     p2 = ps2.getPool(ps2.getPools()[0])
-    #assert deepcmp(p1._urns, p2._urns) is None
-    #assert deepcmp(p1._tags, p2._tags) is None
-    #assert deepcmp(p1._classes, p2._classes) is None
+    # assert deepcmp(p1._urns, p2._urns) is None
+    # assert deepcmp(p1._tags, p2._tags) is None
+    # assert deepcmp(p1._classes, p2._classes) is None
     assert deepcmp(p1._dTypes, p2._dTypes) is None
     assert deepcmp(p1._dTags, p2._dTags) is None
 
@@ -829,16 +836,17 @@ def backup_restore(ps):
     assert tar == tar2
     # restore
     lst = p2.restore(tar2)
-    # print(lst)
+
     # two pools are the same
     assert deepcmp(hk1, p2.readHK()) is None
 
 
-def mkStorage(thepoolname, thepoolurl, db=None):
+def mkStorage(thepoolname, thepoolurl, pstore=None):
     """ returns pool object and productStorage """
 
     cleanup(thepoolurl, thepoolname)
-    pstore = ProductStorage(thepoolname, thepoolurl)
+    if not pstore:
+        pstore = ProductStorage(thepoolname, thepoolurl)
     thepoolpath, tsc, tpl, pn, un, pw = parse_poolurl(thepoolurl, thepoolname)
     if tsc in ['file', 'server']:
         assert op.exists(transpath(thepoolname, thepoolpath))
@@ -911,9 +919,9 @@ def doquery(poolpath, newpoolpath):
 
     # query with a specific parameter in all products' metadata, which is the variable 'm' in the query expression, i.e. ``m = product.meta; ...``
     m = 2
+
     q = MetaQuery(TP, 'm["description"].value == "%s"' % rec1[m]['a0'])
     res = pstore.select(q)
-    assert len(res) == 1, str(res)
 
     def chk(r, c):
         p = r.product
@@ -924,6 +932,10 @@ def doquery(poolpath, newpoolpath):
 
     chk(res[0], rec1[m])
 
+    # gracefully handle none-exisiting key
+    # q = MetaQuery(TP, 'm["not_exists"].value == "%s"' % rec1[m]['a0'])
+    # res = pstore.select(q)
+
     # query with a parent class and a specific parameter
     m = 3
     q = MetaQuery(BaseProduct, 'm["creator"].value == "%s"' % rec1[m]['a1'])
@@ -948,6 +960,11 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[1])
     chk(res[1], rec1[2])
 
+    qstr = 'm["extra"] > 5000 and m["extra"] <= 5002'
+    pqv = pstore.select(qstr, variable='m', ptype=BaseProduct)
+    pqs = pstore.select(qstr, variable='m', ptype='BaseProduct')
+    assert res == pqv == pqs
+
     # two classes
     q = MetaQuery(BaseProduct, 'm["extra"] > 5000 and m["extra"] < 5004')
     res = pstore.select(q)
@@ -994,6 +1011,11 @@ def doquery(poolpath, newpoolpath):
     chk(res[2], rec1[5])
     chk(res[3], rec1[6])
 
+    qstr = 'True'
+    pqv = pstore.select(qstr, variable='m', ptype=Context)
+    pqs = pstore.select(qstr, variable='m', ptype='Context')
+    assert res == pqv == pqs
+
     # all 'time' < 5006. will cause KeyError because some Contex data do not have 'time'
     q = MetaQuery(Context, 'm["time"] < 5006')
     with pytest.raises(KeyError):
@@ -1026,6 +1048,11 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[3])
     chk(res[1], rec1[4])
 
+    qstr = '"n 1" in m["creator"].value'
+    pq1 = thepool.select(qstr, ptype='BaseProduct')
+    pq2 = newpool.qm(qstr, 'BaseProduct')
+    assert res == pq1 + pq2
+
     if not newpoolpath.startswith('http'):
         # same as above but query is a function
         def t(m):
@@ -1047,6 +1074,15 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[3])
     chk(res[1], rec1[4])
 
+    qstr = '"n 1" in p.creator'
+    pqv = pstore.select(qstr, variable='p', ptype=BaseProduct)
+    pqs = pstore.select(qstr, variable='p', ptype='BaseProduct')
+    assert res == pqv == pqs
+    # pool
+    pq1 = thepool.select(qstr, 'p', 'BaseProduct')
+    pq2 = newpool.select(qstr, 'p', 'BaseProduct')
+    assert res == pq1 + pq2
+
     q = '"n 1" in p.creator'
     resu = thepool.where(q)
     res = [ProductRef(u) for u in resu]
@@ -1063,7 +1099,7 @@ def doquery(poolpath, newpoolpath):
     chk(res[2], rec1[3])
 
     # report lru_cache info
-    print('****** %s ' % str(type(thepool)), thepool.getCacheInfo())
+    print('***cache*** %s ' % str(thepool.getCacheInfo()))
 
 
 def test_query_local_mem():
@@ -1296,14 +1332,15 @@ def test_realistic_http(server, demo_product):
     pass
 
 
-def test_realistic_csdb(csdb, demo_product):
-    test_pool, url = csdb  # csdb:///csdb_test_pool
-    poolname = 'test_csdb'
-    poolurl = 'csdb:///' + poolname
+def test_realistic_csdb(csdb, urlcsdb, csdb_client, demo_product):
+    urlupload, urldelete, urllist, client = csdb_client
+    test_pool, poolurl, pstore = csdb  # csdb:///csdb_test_pool
+    poolname = test_pool
+
     cleanup(poolurl, poolname)
     # remove existing pools in memory
     # clean up possible garbage of previous runs. use class method to avoid reading pool hk info during ProdStorage initialization.
-    thepool, pstore = mkStorage(poolname, poolurl, test_pool)
+    thepool, pstore = mkStorage(poolname, poolurl, pstore)
 
     p1 = Product(description='p1')
     p2 = Product(description='p2')
