@@ -16,18 +16,18 @@ from fdi.utils.options import opt
 from fdi.utils.fetch import fetch
 from fdi.utils.tree import tree
 from fdi.utils.loadfiles import loadMedia
-from fdi.utils.validator import getValidator, validateJson
-
+from fdi.utils.getconfig import getConfig, Config_Look_Up, Config_NameSpace
 import traceback
+import importlib
 import copy
 from datetime import timezone, timedelta, datetime
 import sys
 import os
+import getopt
 import json
 import hashlib
 import os.path as op
 import pytest
-import numpy as np
 from astropy.io import fits
 
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
@@ -434,6 +434,9 @@ def test_wls():
     assert wls('1\n2345', 2) == '1\n23\n45'
     assert wls('1\n\n2345', 2) == '1\n23\n45'
     assert wls('12345\n\n', 2) == '12\n34\n5'
+
+
+def obsolete_wls():
     assert wls('格式版本', 4) == '格式\n版本'
     assert wls('格式版本', 8) == '格式版本'
     assert wls('格式版本。', 10) == '格式版本。'
@@ -500,7 +503,7 @@ def test_opt(caplog):
     assert out[0]['result'] == 1
 
     # unplanned option and '--help' get exception and exits
-    with pytest.raises(SystemExit):
+    with pytest.raises(getopt.GetoptError):
         out = opt(options, ['exe', '--helpme', '--name=Awk', '-y'])
     assert 'option -y not recognized' in caplog.text
 
@@ -511,56 +514,175 @@ def test_opt(caplog):
 
 
 def check_conf(cfp, typ, getConfig):
+    """ plant a config file and show that it works. """
+
     cfn = typ + 'local.py'
-    cfp = op.expanduser(cfp)
-    filec = op.join(cfp, cfn)
-    os.system('rm -f ' + filec)
-    conf = 'import os; %sconfig={"jk":98, "m":os.path.abspath(__file__)}' % typ
+    if cfp:
+        cfp = op.expanduser(cfp)
+        filec = op.join(cfp, cfn)
+    else:
+        # see test_getConfig_init
+        return
+    try:
+        os.unlink(filec)
+    except:
+        pass
+    for v in ('NO_PRE',):
+        os.environ.pop(v, '')
+    conf = 'import os; %sconfig={"jk":98, "m":os.path.abspath(__file__)}; config={"no_pre":4}' % typ
     with open(filec, 'w') as f:
         f.write(conf)
     # check conf file directory
     w = getConfig(conf=typ, force=True)
     assert w['jk'] == 98
+    assert w['no_pre'] == 4
     pfile = w['m']
+    assert getConfig('jk', conf=typ) == 98
     assert pfile.startswith(cfp)
-    os.system('rm -f ' + filec)
+    # XXX TODO: NOT WORKING
+    # conf file changed afer loading
+    if 0:
+        try:
+            pass  # os.unlink(filec)
+        except:
+            pass
+        conf = 'import os; %sconfig={"jk":33, "m":os.path.abspath(__file__)}' % typ
+        with open(filec, 'w') as f:
+            f.write(conf)
+        # check conf file directory
+        w = getConfig(conf=typ)  # cached if w/o force
+        assert w['jk'] == 98
+        w = getConfig(conf=typ, force=True)  # loaded from file if force
+        assert w['jk'] == 33 == getConfig('jk', conf=typ)
+        os.unlink(filec)
 
 
-def test_getConfig_init(getConfig):
+def test_getConfig_init():
 
     # no arg
     v = getConfig()
-    from fdi.pns.config import pnsconfig
+    c = getConfig()
+    clu = Config_Look_Up['pns']
+    assert v is c
+    assert all(v[k] == _v for k, _v in clu.items())
+    assert all(clu[k] == _v for k, _v in v.items())
+
+    from fdi.pns.config import pnsconfig, config as config_
+    defaultconf = copy.deepcopy(pnsconfig)
+    defaultconf.update(config_)
+    # default settings enables default config+pnslocal
     # v is a superset
-    assert all(n in v for n in pnsconfig)
+    assert all(n in v for n in defaultconf)
+
+    # get value of normal item
+    assert getConfig('scheme').startswith('http')
+    assert getConfig('scheme') == clu['scheme']
+    # get url
+    con = getConfig()
+    purl = ''.join((con['scheme'], '://',
+                    con['host'], ':',
+                    str(con['port']),
+                    con['baseurl']
+                    ))
+
+    assert getConfig('poolurl:') == purl + '/'
+    assert getConfig('poolurl:foo') == purl + '/' + 'foo'
+
+    logging.debug('Test non-exisiting.')
+    # builtin is {} means not using the default config.
+    non = getConfig(conf='non-existing', builtin={}, force=True)
+    assert len(non) == 0  # len(pnsconfig) + len(config_)
+
+    f = {'foo': 9}
+    # the order of the right-hand side is {foo}+pnslocal
+    v_diff = dict((n, v[n]) for n in (set(v.keys()-set(defaultconf))))
+    f = getConfig(builtin={'foo': 9})
+    try:
+        # foo is not in maps[0]
+        assert f.pop('foo') == 9
+    except KeyError as e:
+        f['foo']
+        # foo is in maps[0]
+        assert f.pop('foo') == 9
+    # every item in v_diff  is in f
+    assert all(v == f[k] for k, v in v_diff.items())
 
 
-def test_getConfig_noENV(getConfig):
+def test_getConfig_ENV():
 
     # non-default conf type
-    typ = 'abc'
+    typ = 'xyz'
     # environment variable not set
     try:
-        del os.environ['CONF_DIR']
+        del os.environ[typ.upper()+'_CONF_DIR']
     except:
         pass
 
-    # default dir, there is  nothing
-    # put mock in the default directory
-    check_conf('~/.config', typ, getConfig)
-
-
-def test_getConfig_conf(getConfig):
-
-    # non-default conf type
-    typ = 'abc'
-    # specify directory
+    # specify directory set a dir with ENV
     cp = '/tmp'
     # environment variable
-    os.environ['CONF_DIR_' + typ.upper()] = cp
+    os.environ[typ.upper()+'_CONF_DIR'] = cp
     check_conf(cp, typ, getConfig)
-    # non-existing. the file has been deleted by the check_conf in the last line
-    w = getConfig(conf=typ)
+
+    # env var overrides
+    # with prefix
+    before = getConfig(conf=typ)
+
+    assert 98 == before['jk']
+    assert before['username'] != 'fungi'
+    # prefixed_osenv
+    # now 'jk' in pnsconfig
+    assert Config_Look_Up[typ].sources.maps[1]['jk'] == 98
+    os.environ[typ.upper()+'_JK'] = 'shroom'
+    assert typ.upper()+'_JK' in os.environ
+    # with pytest.raises(KeyError):
+    # a new mapping is generated without jk
+    a = [len(x) for x in Config_Look_Up[typ].sources.maps]
+    assert a[:4] == [1, 2, 0, 1]
+    con = getConfig(conf=typ, force=True)
+    b = [len(x) for x in Config_Look_Up[typ].sources.maps]
+    assert b[:4] == [2, 2, 0, 1]
+    assert len(Config_Look_Up[typ].cache) == 0
+    assert Config_Look_Up[typ].sources.maps[0]['jk'] == 'shroom'
+    assert getConfig('jk', conf=typ) == 'shroom'
+
+    # a key not in config
+    con = getConfig(conf=typ, force=True)
+    assert con['username'] == 'foo'
+    os.environ[typ.upper()+'_USERNAME'] = 'fungi'
+    assert os.environ[typ.upper()+'_USERNAME'] == 'fungi'
+
+    a = [len(x) for x in Config_Look_Up[typ].sources.maps]
+    assert a[:4] == [2, 2, 0, 1]
+    con = getConfig(conf=typ, force=True)
+    b = [len(x) for x in Config_Look_Up[typ].sources.maps]
+    assert b[:4] == [3, 2, 0, 1]
+
+    # changed
+    assert con['username'] == 'fungi'
+    assert getConfig('username', conf=typ) == 'fungi'
+
+    # no prefix
+    assert con['no_pre'] == 4
+    os.environ['NO_PRE'] = 'mollen'
+    assert os.environ['NO_PRE'] == 'mollen'
+
+    con_e = getConfig(conf=typ, force=True)
+    b = [len(x) for x in Config_Look_Up[typ].sources.maps]
+    assert b[:4] == [3, 2, 1, 1]
+    assert con_e['no_pre'] == 'mollen'
+
+    os.environ[typ.upper()+'_HOST'] = 'lichen'
+    assert '://lichen:' in getConfig('poolurl:', conf=typ, force=True)
+
+
+def test_getConfig_conf():
+    # clear previous load
+    importlib.invalidate_caches()
+    # non-default conf type
+    typ = 'abc'
+    check_conf('~/.config', typ, getConfig)
+    # the file has been deleted by the check_conf in the last line
 
 
 def test_leapseconds():
@@ -582,17 +704,3 @@ def test_leapseconds():
     assert utc_to_tai(t4) - utc_to_tai(t4 - timedelta(seconds=1)) == \
         timedelta(seconds=2)
     print(_fallback.cache_info())
-
-
-def test_validator():
-    fdi_sch_dir = op.join(op.abspath(op.dirname(__file__)), '../fdi/schemas')
-    sch_dir = op.join(op.abspath(op.dirname(__file__)), 'resources/schemas')
-    sch_path = op.join(sch_dir, 'prd_schema.jsn')
-    with open(sch_path, 'r') as file:
-        sch = json.load(file)
-
-    from fdi.dataset.baseproduct import BaseProduct
-    jsn = BaseProduct().zInfo
-
-    assert getValidator(sch, schema_dir=fdi_sch_dir,
-                        verbose=1).validate(jsn) is None
